// ===========================================================================
// AnotherVision.cpp
// program entry
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include "AnotherVision.h"
#include "common.h"
#include "Profile.h"
#include "Filter.h"
#include "SplashWnd.h"
#include "ColorUtils.h"
#include "OptionDlg.h"
#include "PreviewWnd.h"
#include "MainFrm.h"


// ---------------------------------------------------------------------------

CAppModule _Module;

static
BOOL s_bFatalErrorOccured = FALSE;
static
HMODULE s_hLocaleResource = NULL;
static
CMainFrame* s_pWndMainFrame = NULL;
static
BOOL s_bShuttingDown = FALSE;


// ---------------------------------------------------------------------------
// set fatal error flag
// ---------------------------------------------------------------------------
void SetFatalError(BOOL bEnable/*=TRUE*/)
{
	s_bFatalErrorOccured = bEnable;
}

// ---------------------------------------------------------------------------
// check fatal error flag
// ---------------------------------------------------------------------------
BOOL IsFatalError()
{
	return s_bFatalErrorOccured;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
BOOL IsAppShuttingDown()
{
	return s_bShuttingDown;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void SetAppShuttingDown(BOOL bEnable)
{
	s_bShuttingDown = bEnable;
}

// ---------------------------------------------------------------------------
// get handle of UI resource (external locale resource)
// ---------------------------------------------------------------------------
HMODULE GetResourceHandle()
{
	if (s_hLocaleResource != NULL) {
		return s_hLocaleResource;
	}
	return _Module.get_m_hInstResource();
}

// ---------------------------------------------------------------------------
// get handle of core resource (internal resource)
// ---------------------------------------------------------------------------
HMODULE GetSystemResourceHandle()
{
	return _Module.get_m_hInstResource();
}

// ---------------------------------------------------------------------------
// load locale resource file
// ---------------------------------------------------------------------------
static
void InitLocaleResource()
{
	LANGID lid = GetSystemDefaultLangID();

	TCHAR szDllPath[_MAX_PATH];
	wsprintf(szDllPath, _T("%s%s\\%d\\%s"), GetModuleDir(), LOCALE_RESOURCE_ROOT_DIR, (DWORD)lid, LOCALE_RESOURCE_FILENAME);
	if (! IsFile(szDllPath)) {
		return;
	}
	HMODULE hModule = LoadLibrary(szDllPath);
	if (hModule == NULL) {
		return;
	}
	TCHAR szExePath[_MAX_PATH];
	GetModuleFileName(GetModuleHandle(NULL), szExePath, sizeof(szExePath));
	DWORD dwExeVersion = GetProductVersionMS(szExePath);
	DWORD dwResVersion = GetProductVersionMS(szDllPath);
	if (dwExeVersion != dwResVersion) {
		FreeLibrary(hModule);
		return;
	}
	s_hLocaleResource = hModule;
}

// ---------------------------------------------------------------------------
// launch unicode version application
// ---------------------------------------------------------------------------
BOOL LaunchUnicodeApp()
{
	if (! IsWinNT()) {
		return FALSE;
	}
	TCHAR szTargetAppPath[_MAX_PATH];
	lstrcpy(szTargetAppPath, GetModuleDir());
	PathAppend(szTargetAppPath, ANOTHERVISION_NT);
	if (IsWinNT()) {
		if (IsFile(szTargetAppPath)) {
			BOOL bResult = ExecCmd(szTargetAppPath);
			if (! bResult) {
				ExMessageBoxText(MB_OK | MB_ICONSTOP, MSG_ERROR_EXEC_UNICODEAPP);
			}
			return TRUE;
		}
	}
	return FALSE;
}

// ---------------------------------------------------------------------------
// create main window and handle message loop
// ---------------------------------------------------------------------------
int Run(LPTSTR /*lpstrCmdLine*/ = NULL, int nCmdShow = SW_SHOWDEFAULT)
{
	CMessageLoop theLoop;
	_Module.AddMessageLoop(&theLoop);

	if (! GetAppProfile()->IsNoSplash()) {
		ShowSplashWnd();
	}

	CMainFrame wndMain;

	if (wndMain.CreateEx() == NULL) {
		return 0;
	}

	int nRet = theLoop.Run();

	_Module.RemoveMessageLoop();
	return nRet;
}

// ---------------------------------------------------------------------------
// WinMain
// application entry
// ---------------------------------------------------------------------------
int WINAPI _tWinMain(HINSTANCE hInstance, HINSTANCE /*hPrevInstance*/, LPTSTR lpstrCmdLine, int nCmdShow)
{
#if !defined(UNICODE)
#if !defined(_DEBUG)
	if (LaunchUnicodeApp()) {
		return 0;
	}
#endif
#endif
	HANDLE hMutex = CreateMutex(NULL, FALSE, APP_MUTEX);
	if (hMutex == NULL || GetLastError() == ERROR_ALREADY_EXISTS) {
		if (hMutex != NULL) {
			CloseHandle(hMutex);
		}
		return FALSE;
	}

	InitLocaleResource();

	HRESULT hRes = ::CoInitialize(NULL);
	ATLASSERT(SUCCEEDED(hRes));

	::DefWindowProc(NULL, 0, 0, 0L);

	AtlInitCommonControls(0);

	hRes = _Module.Init(NULL, hInstance);
	ATLASSERT(SUCCEEDED(hRes));

	CColorName::InitConstant();
	COptionDlg::InitConstant();

	TCHAR szLangID[20];
	LANGID lidSystem = GetSystemDefaultLangID();
	wsprintf(szLangID, _T("%d"), lidSystem);
	xstr_t strAppDir = GetModuleDir();
	xstr_t strSystemFilterDir = strAppDir + APP_SYSTEM_FILTER_DIR;
	xstr_t strSystemFilterDirWithLocale = strSystemFilterDir + _T("\\") + szLangID;
	xstr_t strPrivateFilterDir = strAppDir + APP_PRIVATE_FILTER_DIR;
	xstr_t strPrivateFilterDirWithLocale = strPrivateFilterDir + _T("\\") + szLangID;
	GetAppFilterManager()->ReadFilters((LPCTSTR)strPrivateFilterDirWithLocale);
	GetAppFilterManager()->ReadFilters((LPCTSTR)strPrivateFilterDir);
	GetAppFilterManager()->ReadFilters((LPCTSTR)strSystemFilterDirWithLocale);
	GetAppFilterManager()->ReadFilters((LPCTSTR)strSystemFilterDir);

	GetAppProfile()->ReadProfile();

	AppInitialiizeColorExpand(GetAppProfile()->GetExtendColor());
	GetAppFilterManager()->ActivateFilters();

	int nRet = Run(lpstrCmdLine, nCmdShow);

	GetAppProfile()->WriteProfile();

	_Module.Term();
	::CoUninitialize();

	return nRet;
}
