// ===========================================================================
// CaptureImage.h
// capture image
//
//                                                       by kanegon 2006
// ===========================================================================

#pragma once

// ---------------------------------------------------------------------------
// capture image class
// ---------------------------------------------------------------------------
class CCaptureImage {
private:
	HDC m_hdcMem;
	HBITMAP m_hBitmap;
	HBITMAP m_hBitmapOld;
	int m_nWidth;
	int m_nHeight;
	int m_nWidthBytes;
	int m_nBitmapBytes;
	BYTE* m_pBitsOriginal;
	BYTE* m_pBitsFiltered;
	BYTE* m_pBitsIndexes;

	BOOL CreateImage(HDC hdc, int nWidth, int nHeight) {
		m_nWidth = nWidth;
		m_nHeight = nHeight;
		m_nWidthBytes = ((m_nWidth * 3) + 3) & 0xfffffffc;
		m_nBitmapBytes = m_nWidthBytes * m_nHeight;
		m_pBitsOriginal = new BYTE[m_nBitmapBytes];
		if (AppIsColorWordIndexes()) {
			m_pBitsIndexes = new BYTE[m_nWidth * m_nHeight * sizeof(WORD)];
		}
		else {
			m_pBitsIndexes = new BYTE[m_nWidth * m_nHeight * sizeof(DWORD)];
		}

		m_hdcMem = CreateCompatibleDC(hdc);

		BITMAPINFO bi;
		ZeroMemory(&bi, sizeof(bi));
		bi.bmiHeader.biSize        = sizeof(BITMAPINFOHEADER);
		bi.bmiHeader.biWidth       = m_nWidth;
		bi.bmiHeader.biHeight      = -m_nHeight;
		bi.bmiHeader.biPlanes      = 1;
		bi.bmiHeader.biBitCount    = 24;
		bi.bmiHeader.biCompression = BI_RGB;

		m_hBitmap = CreateDIBSection(NULL, &bi, DIB_PAL_COLORS,
				(void**)&m_pBitsFiltered, NULL, 0);
		if (m_hBitmap == NULL) {
			// error
ATLTRACE2(atlTraceGeneral, 0, _T("CreateDIBSection() failed. (code=%d)\n"), GetLastError());
			return FALSE;
		}
		m_hBitmapOld = (HBITMAP)SelectObject(m_hdcMem, m_hBitmap);
		return TRUE;
	}
	void CreateFilterIndexes()
	{
		if (IsWin2K()) {
			GdiFlush();
		}
		if (AppIsColorWordIndexes()) {
			int y;
			for (y = 0; y < m_nHeight; y++) {
				BYTE* pSrc = &m_pBitsOriginal[m_nWidthBytes * y];
				WORD* pIndexes = (WORD*)&m_pBitsIndexes[m_nWidth * y * sizeof(WORD)];
				int x;
				for (x = 0; x < m_nWidth; x++) {
					int b = pSrc[0];
					int g = pSrc[1];
					int r = pSrc[2];
					pIndexes[x] = CFilter::GetColormapExIndex(RGB(r, g, b));
					pSrc += 3;
				}
			}
		}
		else {
			int y;
			for (y = 0; y < m_nHeight; y++) {
				BYTE* pSrc = &m_pBitsOriginal[m_nWidthBytes * y];
				DWORD* pIndexes = (DWORD*)&m_pBitsIndexes[m_nWidth * y * sizeof(DWORD)];
				int x;
				for (x = 0; x < m_nWidth; x++) {
					int b = pSrc[0];
					int g = pSrc[1];
					int r = pSrc[2];
					pIndexes[x] = CFilter::GetColormapExIndex(RGB(r, g, b));
					pSrc += 3;
				}
			}
		}
	}

public:
	CCaptureImage() {
		Clear(TRUE);
	}
	virtual ~CCaptureImage() {
		Clear();
	}

	void Clear(BOOL bInitializeOnly = FALSE) {
		if (! bInitializeOnly) {
			if (m_hdcMem != NULL) {
				if (m_hBitmap != NULL) {
					SelectObject(m_hdcMem, m_hBitmapOld);
					DeleteObject(m_hBitmap);
				}
				DeleteDC(m_hdcMem);
			}
			if (m_pBitsOriginal != NULL) {
				delete[] m_pBitsOriginal;
			}
			if (m_pBitsIndexes != NULL) {
				delete[] m_pBitsIndexes;
			}
		}
		m_hdcMem = NULL;
		m_hBitmap = NULL;
		m_hBitmapOld = NULL;
		m_nWidth = 0;
		m_nHeight = 0;
		m_nWidthBytes = 0;
		m_nBitmapBytes = 0;
		m_pBitsOriginal = NULL;
		m_pBitsFiltered = NULL;
		m_pBitsIndexes = NULL;
	}

	HDC GetDC() {
		return m_hdcMem;
	}
	int GetWidth() {
		return m_nWidth;
	}
	int GetHeight() {
		return m_nHeight;
	}

	BOOL CaptureDesktop() {
		int nDesktopWidth = GetSystemMetrics(SM_CXSCREEN);
		int nDesktopHeight = GetSystemMetrics(SM_CYSCREEN);
		return Capture(0, 0, nDesktopWidth, nDesktopHeight);
	}
	BOOL Capture(int nX, int nY, int nWidth, int nHeight) {
		HDC hdcDesktop = CreateDC(_T("DISPLAY"), NULL, NULL, NULL);
		if (nWidth != m_nWidth ||
			nHeight != m_nHeight) {
			Clear();
			CreateImage(hdcDesktop, nWidth, nHeight);
		}
		else {
			if (IsWin2K()) {
				GdiFlush();
			}
			ZeroMemory(m_pBitsFiltered, m_nBitmapBytes);
		}
		if (IsWin2K()) {
			BitBlt(m_hdcMem, 0, 0, m_nWidth, m_nHeight,
					hdcDesktop, nX, nY,
					SRCCOPY | CAPTUREBLT);
		}
		else {
			BitBlt(m_hdcMem, 0, 0, m_nWidth, m_nHeight,
					hdcDesktop, nX, nY,
					SRCCOPY);
		}
		DeleteDC(hdcDesktop);
		if (IsWin2K()) {
			GdiFlush();
		}
		memcpy(m_pBitsOriginal, m_pBitsFiltered, m_nBitmapBytes);
		CreateFilterIndexes();
		return TRUE;
	}
	void ApplyFilter(CFilter* pFilter) {
		if (m_pBitsFiltered == NULL) {
			return;
		}
		if (IsWin2K()) {
			GdiFlush();
		}
		if (pFilter->IsNormalFilter()) {
			memcpy(m_pBitsFiltered, m_pBitsOriginal, m_nBitmapBytes);
		}
		else {
			COLORREF* pColorMapTable = pFilter->GetColorMapExTable();
			if (AppIsColorWordIndexes()) {
				int y;
				for (y = 0; y < m_nHeight; y++) {
					WORD* pIndexes = (WORD*)&m_pBitsIndexes[m_nWidth * y * sizeof(WORD)];
					BYTE* pDst = &m_pBitsFiltered[m_nWidthBytes * y];
					int x;
					for (x = 0; x < m_nWidth; x++) {
						COLORREF clr = pColorMapTable[pIndexes[x]];
						pDst[2] = (BYTE)(clr & 0x000000ff);
						pDst[1] = (BYTE)((clr & 0x0000ff00) >> 8);
						pDst[0] = (BYTE)((clr & 0x00ff0000) >> 16);
						pDst += 3;
					}
				}
			}
			else {
				int y;
				for (y = 0; y < m_nHeight; y++) {
					DWORD* pIndexes = (DWORD*)&m_pBitsIndexes[m_nWidth * y * sizeof(DWORD)];
					BYTE* pDst = &m_pBitsFiltered[m_nWidthBytes * y];
					int x;
					for (x = 0; x < m_nWidth; x++) {
						COLORREF clr = pColorMapTable[pIndexes[x]];
						pDst[2] = (BYTE)(clr & 0x000000ff);
						pDst[1] = (BYTE)((clr & 0x0000ff00) >> 8);
						pDst[0] = (BYTE)((clr & 0x00ff0000) >> 16);
						pDst += 3;
					}
				}
			}
		}
	}
	COLORREF GetCenterColor(BOOL bOriginal) {
		if (m_pBitsFiltered == NULL) {
			return RGB(0, 0, 0);
		}
		int nCenterX = (m_nWidth - 1) / 2;
		int nCenterY = (m_nHeight - 1) / 2;
		if (bOriginal) {
			BYTE* pBits = &m_pBitsOriginal[m_nWidthBytes * nCenterY] + (3 * nCenterX);
			int b = pBits[0];
			int g = pBits[1];
			int r = pBits[2];
			return RGB(r, g, b);
		}
		else {
			BYTE* pBits = &m_pBitsFiltered[m_nWidthBytes * nCenterY] + (3 * nCenterX);
			int b = pBits[0];
			int g = pBits[1];
			int r = pBits[2];
			return RGB(r, g, b);
		}
	}
	COLORREF GetLeftTopColor(BOOL bOriginal) {
		if (m_pBitsFiltered == NULL) {
			return RGB(0, 0, 0);
		}
		if (bOriginal) {
			BYTE* pBits = &m_pBitsOriginal[0];
			int b = pBits[0];
			int g = pBits[1];
			int r = pBits[2];
			return RGB(r, g, b);
		}
		else {
			BYTE* pBits = &m_pBitsFiltered[0];
			int b = pBits[0];
			int g = pBits[1];
			int r = pBits[2];
			return RGB(r, g, b);
		}
	}
};
