// ===========================================================================
// ColorUtils.cpp
// color utility
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include "AnotherVision.h"
#include "ColorUtils.h"


// ---------------------------------------------------------------------------
xstr_t CColorName::ms_strBright;
xstr_t CColorName::ms_strDark;
xstr_t CColorName::ms_strClear;
xstr_t CColorName::ms_strLight;
xstr_t CColorName::ms_strDull;
xstr_t CColorName::ms_strWhite;
xstr_t CColorName::ms_strBlack;
xstr_t CColorName::ms_strLightGray;
xstr_t CColorName::ms_strDarkGray;
xstr_t CColorName::ms_strColorName[24];

// ---------------------------------------------------------------------------
// return mixamum value
// ---------------------------------------------------------------------------
static inline
int max3(int n1, int n2, int n3)
{
	int tmp1 = (n1 >= n2) ? n1 : n2;
	return (tmp1 >= n3) ? tmp1 : n3;
}

// ---------------------------------------------------------------------------
// return minimum value
// ---------------------------------------------------------------------------
static inline
int min3(int n1, int n2, int n3)
{
	int tmp1 = (n1 <= n2) ? n1 : n2;
	return (tmp1 <= n3) ? tmp1 : n3;
}

// ---------------------------------------------------------------------------
// RGB => HSV
// ---------------------------------------------------------------------------
void CHSV::rgb2hsv(int nR, int nG, int nB, int& nHue, double& fSaturation, double& fValue)
{
	double fR = (double)nR / 255.0;
	double fG = (double)nG / 255.0;
	double fB = (double)nB / 255.0;
	int nMax = max3(nR, nG, nB);
	int nMin = min3(nR, nG, nB);
	double fMax = (double)nMax / 255.0;
	double fMin = (double)nMin / 255.0;
	fValue = fMax;
	double fGap = fMax - fMin;
	if (nMax == 0) {
		fSaturation = 0.0;
	}
	else {
		fSaturation = fGap / fMax;
	}
	if (fSaturation == 0.0) {
		// nHue is undefined
		nHue = 0;
	}
	else {
		double fHue;
		double r1 = (fMax - fR) / fGap;
		double g1 = (fMax - fG) / fGap;
		double b1 = (fMax - fB) / fGap;
		if (nR == nMax) {
			if (nG == nMin) fHue = 5.0 + b1;
			else            fHue = 1.0 - g1;
		}
		else if (nG == nMax) {
 			if (nB == nMin)  fHue = 1.0 + r1;
 			else             fHue = 3.0 - b1;
		}
		else /*if (nB == nMax)*/ {
 			if (nR == nMin)  fHue = 3.0 + g1;
 			else             fHue = 5.0 - r1;
		}
		nHue = (int)(fHue * 60.0);
		if (nHue >= 360) {
			nHue -= 360;
		}
	}
}

// ---------------------------------------------------------------------------
// HSV => RGB
// ---------------------------------------------------------------------------
void CHSV::hsv2rgb(int nHue, double fSaturation, double fValue, int& nR, int& nG, int& nB)
{
	double fHue = ((double)(nHue % 360)) / 60.0;
	int nHueMain = (int)fHue;
	double fHueSub = fHue - nHueMain;

	double fT1 = (1 - fSaturation) * fValue;
	double fT2 = (1 -(fSaturation * fHueSub)) * fValue;
	double fT3 = (1 -(fSaturation * (1 - fHueSub))) * fValue;
	double fR, fG, fB;
	switch (nHueMain) {
	case 0:
		fR = fValue;
		fG = fT3;
		fB = fT1;
		break;
	case 1:
		fR = fT2;
		fG = fValue;
		fB = fT1;
		break;
	case 2:
		fR = fT1;
		fG = fValue;
		fB = fT3;
		break;
	case 3:
		fR = fT1;
		fG = fT2;
		fB = fValue;
		break;
	case 4:
		fR = fT3;
		fG = fT1;
		fB = fValue;
		break;
	//case 5:
	default:
		fR = fValue;
		fG = fT1;
		fB = fT2;
		break;
	}
	int r = (int)(fR * 255.0);
	int g = (int)(fG * 255.0);
	int b = (int)(fB * 255.0);
	if (r < 0)   r = 0;
	if (r > 255) r = 255;
	if (g < 0)   g = 0;
	if (g > 255) g = 255;
	if (b < 0)   b = 0;
	if (b > 255) b = 255;
	nR = r;
	nG = g;
	nB = b;
}

// ---------------------------------------------------------------------------
// RGB => HSI
// ---------------------------------------------------------------------------
void CHSI::rgb2hsi(int nR, int nG, int nB, int& nHue, double& fSaturation, double& fIntensity)
{
	double fR = (double)nR / 255.0;
	double fG = (double)nG / 255.0;
	double fB = (double)nB / 255.0;
	int nMax = max3(nR, nG, nB);
	int nMin = min3(nR, nG, nB);
	double fMax = (double)nMax / 255.0;
	double fMin = (double)nMin / 255.0;
	fIntensity = (fMax + fMin) / 2.0;
	double fGap = fMax - fMin;

	if (fGap == 0.0) {
		fSaturation = 0.0;
	}
	else {
		fSaturation = (fIntensity < 0.5) ? (fGap / 2) / fIntensity : (fGap / 2) / (1 - fIntensity);
	}
	if (fSaturation == 0.0) {
		nHue = 0;
	}
	else if (nR == nMax) {
		nHue = (int)((fG - fB) * 60.0 / fGap);
	}
	else if (nG == nMax) {
		nHue = (int)((fB - fR) * 60.0 / fGap) + 120;
	}
	else /*if (nB == nMax)*/ {
		nHue = (int)((fR - fG) * 60.0 / fGap) + 240;
	}
	if (nHue < 0) {
		nHue += 360;
	}
}

// ---------------------------------------------------------------------------
// HSI => RGB
// ---------------------------------------------------------------------------
void CHSI::hsi2rgb(int nHue, double fSaturation, double fIntensity, int& nR, int& nG, int& nB)
{
	double fHue = ((double)(nHue % 360)) / 60.0;
	int nHueMain = (int)fHue;
	double fHueSub = fHue - nHueMain;

	double fMax = fIntensity + ((fIntensity <= 0.5) ? fIntensity : (1 - fIntensity)) * fSaturation;
	double fMin = fIntensity - ((fIntensity <= 0.5) ? fIntensity : (1 - fIntensity)) * fSaturation;
	double fMid = (nHueMain % 2) ?
			fMax - (fMax - fMin) * fHueSub:
			fMin + (fMax - fMin) * fHueSub;

	double fR, fG, fB;
	switch(nHueMain) {
	case 0:
		fR = fMax;
		fG = fMid;
		fB = fMin;
		break;
	case 1:
		fR = fMid;
		fG = fMax;
		fB = fMin;
		break;
	case 2:
		fR = fMin;
		fG = fMax;
		fB = fMid;
		break;
	case 3:
		fR = fMin;
		fG = fMid;
		fB = fMax;
		break;
	case 4:
		fR = fMid;
		fG = fMin;
		fB = fMax;
		break;
	case 5:
	default:
		fR = fMax;
		fG = fMin;
		fB = fMid;
		break;
	}
	nR = (int)(fR * 255);
	nG = (int)(fG * 255);
	nB = (int)(fB * 255);
}

// ---------------------------------------------------------------------------
// initialize constant
// ---------------------------------------------------------------------------
BOOL CColorName::InitConstant()
{
	TCHAR szBuffer[TEXT_BUFFER_SIZE] = _T("");
	BOOL bResult = FALSE;
	bResult |= ExLoadString(IDS_COLOR_DESCRIPTION_BRIGHT, szBuffer, sizeof(szBuffer));
	ms_strBright = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_DESCRIPTION_DARK, szBuffer, sizeof(szBuffer));
	ms_strDark = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_DESCRIPTION_CLEAR, szBuffer, sizeof(szBuffer));
	ms_strClear = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_DESCRIPTION_LIGHT, szBuffer, sizeof(szBuffer));
	ms_strLight = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_DESCRIPTION_DULL, szBuffer, sizeof(szBuffer));
	ms_strDull = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_NAME_WHITE, szBuffer, sizeof(szBuffer));
	ms_strWhite = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_NAME_BLACK, szBuffer, sizeof(szBuffer));
	ms_strBlack = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_NAME_LIGHT_GRAY, szBuffer, sizeof(szBuffer));
	ms_strLightGray = szBuffer;
	bResult |= ExLoadString(IDS_COLOR_NAME_DARK_GRAY, szBuffer, sizeof(szBuffer));
	ms_strDarkGray = szBuffer;
	int i;
	for (i = 0; i < 24; i++) {
		bResult |= ExLoadString(IDS_COLOR_NAME_HUE_00 + i, szBuffer, sizeof(szBuffer));
		ms_strColorName[i] = szBuffer;
	}

	return bResult;
}

// ---------------------------------------------------------------------------
// return color range
// ---------------------------------------------------------------------------
xstr_t CColorName::GetColorRange(COLORREF clr)
{
	return GetColorRange(CHSV(clr));
}

// ---------------------------------------------------------------------------
// return color range
// ---------------------------------------------------------------------------
xstr_t CColorName::GetColorRange(const CHSV& hsv)
{
	static LPCTSTR s_szColorGuideMnochrome = _T("[ - ] ");
	static LPCTSTR s_szColorGuide[] = {
		_T("[ R ] "), _T("[R>Y] "), _T("[R=Y] "), _T("[R<Y] "),
		_T("[ Y ] "), _T("[Y>G] "), _T("[Y=G] "), _T("[Y<G] "),
		_T("[ G ] "), _T("[G>C] "), _T("[G=C] "), _T("[G<C] "),
		_T("[ C ] "), _T("[C>B] "), _T("[C=B] "), _T("[C<B] "),
		_T("[ B ] "), _T("[B>M] "), _T("[B=M] "), _T("[B<M] "),
		_T("[ M ] "), _T("[M>R] "), _T("[M=R] "), _T("[M<R] "),
	};
	int h = hsv.GetHue();
	double s = hsv.GetSaturation();
	double v = hsv.GetValue();
	if (s < 0.1 || v < 0.1) {
		return s_szColorGuideMnochrome;
	}
	else {
		int nColorGuideIndex = ((h + 8) % 360) / 15;
		return s_szColorGuide[nColorGuideIndex];
	}
}

// ---------------------------------------------------------------------------
// return color name
// ---------------------------------------------------------------------------
xstr_t CColorName::GetColorName(COLORREF clr, BOOL bNoColorRange)
{
	return GetColorName(CHSV(clr), bNoColorRange);
}

// ---------------------------------------------------------------------------
// return color name
// ---------------------------------------------------------------------------
xstr_t CColorName::GetColorName(const CHSV& hsv, BOOL bNoColorRange)
{
	int h = hsv.GetHue();
	double s = hsv.GetSaturation();
	double v = hsv.GetValue();

	TCHAR sz[80] = _T("");
	if (! bNoColorRange) {
		lstrcat(sz, (LPCTSTR)GetColorRange(hsv));
	}
	if (s < 0.1 || v < 0.1) {
		if (v < 0.1) {
			lstrcat(sz, ms_strBlack);
		}
		else if (v >= 0.99) {
			lstrcat(sz, ms_strWhite);
		}
		else if (v >= 0.5) {
			lstrcat(sz, ms_strLightGray);
		}
		else {
			lstrcat(sz, ms_strDarkGray);
		}
	}
	else {
		if (v >= 0.9) {
			lstrcat(sz, ms_strBright);
		}
		else if (v < 0.6) {
			lstrcat(sz, ms_strDark);
		}
		if (s >= 0.9) {
			lstrcat(sz, ms_strClear);
		}
		else if (s < 0.7) {
			if (v >= 0.6) {
				lstrcat(sz, ms_strLight);
			}
			else {
				lstrcat(sz, ms_strDull);
			}
		}
		int nColorIndex = h / 15;
		lstrcat(sz, ms_strColorName[nColorIndex]);
	}
	return sz;
}

// ---------------------------------------------------------------------------
// return color value text
// ---------------------------------------------------------------------------
xstr_t CColorName::GetRGBValue(COLORREF clr, BOOL bHexadecimal)
{
	TCHAR szText[20];
	if (bHexadecimal) {
		_sntprintf(szText, sizeof(szText) - 1, _T("%02X:%02X:%02X"),
				GetRValue(clr), GetGValue(clr), GetBValue(clr));
	}
	else {
		_sntprintf(szText, sizeof(szText) - 1, _T("%d:%d:%d"),
				GetRValue(clr), GetGValue(clr), GetBValue(clr));
	}
	return szText;
}

// ---------------------------------------------------------------------------
// return color value text
// ---------------------------------------------------------------------------
xstr_t CColorName::GetHSVValue(const CHSV& hsv, BOOL bCustom)
{
	TCHAR szText[20];
	TCHAR chColor;
	TCHAR chSign;
	if (bCustom) {
		int h = hsv.GetHue();
		if (hsv.GetSaturation() == 0.0) {
			_sntprintf(szText, sizeof(szText) - 1, _T("-, %.2f, %.2f"),
					hsv.GetSaturation(), hsv.GetValue());
		}
		else {
			int h1 = (hsv.GetHue() + 30) % 360;
			int h2 = h1 / 60;
			int h3 = h1 % 60 - 30;
			TCHAR s_szColor[] = _T("RYGCBM");
			chColor = s_szColor[h2];
			if (h3 < 0) {
				chSign = _T('-');
				h3 = -h3;
			}
			else {
				chSign = _T('+');
			}
			_sntprintf(szText, sizeof(szText) - 1, _T("%c%c%d, %.2f, %.2f"),
					chColor, chSign, h3, hsv.GetSaturation(), hsv.GetValue());
		}
	}
	else {
		if (hsv.GetSaturation() == 0.0) {
			_sntprintf(szText, sizeof(szText) - 1, _T("-, %.2f, %.2f"),
					hsv.GetSaturation(), hsv.GetValue());
		}
		else {
			_sntprintf(szText, sizeof(szText) - 1, _T("%d, %.2f, %.2f"),
					hsv.GetHue(), hsv.GetSaturation(), hsv.GetValue());
		}
	}
	return szText;
}

// ---------------------------------------------------------------------------
// return color value text
// ---------------------------------------------------------------------------
xstr_t CColorName::GetHSVSimpleValue(COLORREF clr)
{
	return GetHSVSimpleValue(CHSV(clr));
}

// ---------------------------------------------------------------------------
// return color value text
// ---------------------------------------------------------------------------
xstr_t CColorName::GetHSVSimpleValue(const CHSV& hsv)
{
	int s = (int)(hsv.GetSaturation() * 10.0 + 0.5);
	int v = (int)(hsv.GetValue() * 10.0 + 0.5);
	TCHAR sz[80];
	wsprintf(sz, _T("%s %d %d"), (LPCTSTR)GetColorRange(hsv), s, v);
	return sz;
}

