// ===========================================================================
// ColorUtils.h
// color utility
//
//                                                       by kanegon 2006
// ===========================================================================

#pragma once

#include "xstr.h"


// ---------------------------------------------------------------------------
// RGB class
// ---------------------------------------------------------------------------
class CExRGB {
private:
	int m_nRed;
	int m_nGreen;
	int m_nBlue;

public:
	CExRGB() {
		m_nRed = 0;
		m_nGreen = 0;
		m_nBlue = 0;
	}
	CExRGB(COLORREF clr) {
		m_nRed = GetRValue(clr);
		m_nGreen = GetGValue(clr);
		m_nBlue = GetBValue(clr);
	}
	CExRGB(const CExRGB& rgb) {
		m_nRed = rgb.m_nRed;
		m_nGreen = rgb.m_nGreen;
		m_nBlue = rgb.m_nBlue;
	}
	virtual ~CExRGB() {
	}

	void SetRed(int r) {
		if (r < 0) {
			m_nRed = 0;
		}
		if (r > 255) {
			m_nRed = 255;
		}
		m_nRed = r;
	}
	int GetRed() const {
		return m_nRed;
	}
	void SetGreen(int g) {
		if (g < 0) {
			m_nGreen = 0;
		}
		if (g > 255) {
			m_nGreen = 255;
		}
		m_nGreen = g;
	}
	int GetGreen() const {
		return m_nGreen;
	}
	void SetBlue(int b) {
		if (b < 0) {
			m_nBlue = 0;
		}
		if (b > 255) {
			m_nBlue = 255;
		}
		m_nBlue = b;
	}
	int GetBlue() const {
		return m_nBlue;
	}
};

// ---------------------------------------------------------------------------
// HSV class
// ---------------------------------------------------------------------------
class CHSV {
public:
	static void rgb2hsv(int nR, int nG, int nB, int& nHue, double& fSaturation, double& fValue);
	static void hsv2rgb(int nHue, double fSaturation, double fValue, int& nR, int& nG, int& nB);

private:
	int m_nHue;
	double m_fSaturation;
	double m_fValue;

public:
	CHSV() {
		m_nHue = 0;
		m_fSaturation = 0.0;
		m_fValue = 0.0;
	}
	CHSV(COLORREF clr) {
		rgb2hsv(GetRValue(clr), GetGValue(clr), GetBValue(clr), m_nHue, m_fSaturation, m_fValue);
	}
	CHSV(const CHSV& hsv) {
		m_nHue = hsv.m_nHue;
		m_fSaturation = hsv.m_fSaturation;
		m_fValue = hsv.m_fValue;
	}
	virtual ~CHSV() {
	}

	int GetHue() const {
		return m_nHue;
	}
	double GetSaturation() const {
		return m_fSaturation;
	}
	double GetValue() const {
		return m_fValue;
	}
};

// ---------------------------------------------------------------------------
// HSI class
// ---------------------------------------------------------------------------
class CHSI {
public:
	static void rgb2hsi(int nR, int nG, int nB, int& nHue, double& fSaturation, double& fIntensity);
	static void hsi2rgb(int nHue, double fSaturation, double fIntensity, int& nR, int& nG, int& nB);

private:
	int m_nHue;
	double m_fSaturation;
	double m_fIntensity;

public:
	CHSI() {
		m_nHue = 0;
		m_fSaturation = 0.0;
		m_fIntensity = 0.0;
	}
	CHSI(COLORREF clr) {
		rgb2hsi(GetRValue(clr), GetGValue(clr), GetBValue(clr), m_nHue, m_fSaturation, m_fIntensity);
	}
	CHSI(const CHSI& hsi) {
		m_nHue = hsi.m_nHue;
		m_fSaturation = hsi.m_fSaturation;
		m_fIntensity = hsi.m_fIntensity;
	}
	virtual ~CHSI() {
	}

	int GetHue() const {
		return m_nHue;
	}
	double GetSaturation() const {
		return m_fSaturation;
	}
	double GetIntensity() const {
		return m_fIntensity;
	}
};

// ---------------------------------------------------------------------------
// color name class
// ---------------------------------------------------------------------------
class CColorName {
private:
	static xstr_t ms_strBright;
	static xstr_t ms_strDark;
	static xstr_t ms_strClear;
	static xstr_t ms_strLight;
	static xstr_t ms_strDull;
	static xstr_t ms_strWhite;
	static xstr_t ms_strBlack;
	static xstr_t ms_strLightGray;
	static xstr_t ms_strDarkGray;
	static xstr_t ms_strColorName[24];

public:
	static BOOL InitConstant();
	static xstr_t GetColorRange(COLORREF clr);
	static xstr_t GetColorRange(const CHSV& hsv);
	static xstr_t GetColorName(COLORREF clr, BOOL bNoColorRange);
	static xstr_t GetColorName(const CHSV& hsv, BOOL bNoColorRange);
	static xstr_t GetRGBValue(COLORREF clr, BOOL bHexadecimal);
	static xstr_t GetHSVValue(const CHSV& hsv, BOOL bCustom);
	static xstr_t GetHSVSimpleValue(COLORREF clr);
	static xstr_t GetHSVSimpleValue(const CHSV& hsv);
};

