// ===========================================================================
// Filter.h
// color filter
//
//                                                       by kanegon 2006
// ===========================================================================

#pragma once

#include <vector>
#include "xstr.h"
#include "common.h"

#import "C:\WINDOWS\system32\msxml2.dll" named_guids
using namespace MSXML2;


// ---------------------------------------------------------------------------

// default active filter
#define DEFAULT_ENABLE_FILTERS _T("_normal,protanopia,deutanopia,tritanoptia,hue_gbr,hue_brg")

#define FILTER_ID_NORMAL                  _T("_normal")
#define FILTER_INITIALIZE_NAME_NORMAL     _T("normal")

#define MAX_FILTER       512
#define MAX_FILTER_ID    40
#define MAX_FILTER_NAME  40

#define FILTERTYPE_DEFINITION   0
#define FILTERTYPE_REFERENCE    1


// ---------------------------------------------------------------------------

class CFilter;
class CFilterManager;

CFilterManager* GetAppFilterManager();

void AppInitialiizeColorExpand(int nColorExpand);
int AppGetColorExpand();
BOOL AppIsColorExpand();
BOOL AppIsColorWordIndexes();


// ---------------------------------------------------------------------------
// color filter reference item
// ---------------------------------------------------------------------------
class CFilterRef {
private:
	xstr_t m_strID;
	CFilter* m_pFilter;
	DWORD m_dwTimeout;

public:
	CFilterRef() {
		m_strID = _T("");
		m_pFilter = NULL;
		m_dwTimeout = 100;
	}
	CFilterRef(LPCTSTR pszID, CFilter* pFilter, DWORD dwTimeout) {
		m_strID = pszID;
		m_pFilter = pFilter;
		m_dwTimeout = dwTimeout;
	}
	CFilterRef(const CFilterRef& ref) {
		*this = ref;
	}
	virtual ~CFilterRef() {
	}
	CFilterRef& operator=(const CFilterRef& ref) {
		m_strID = ref.m_strID;
		m_pFilter = ref.m_pFilter;
		m_dwTimeout = ref.m_dwTimeout;
		return *this;
	}

	void SetID(LPCTSTR pszID) {
		m_strID = pszID;
	}
	LPCTSTR GetID() {
		return m_strID;
	}
	void SetFilter(CFilter* pFilter) {
		m_pFilter = pFilter;
	}
	CFilter* GetFilter() {
		return m_pFilter;
	}
	void SetTimeout(DWORD dwTimeout) {
		m_dwTimeout = dwTimeout;
	}
	DWORD GetTimeout() {
		return m_dwTimeout;
	}
};

// ---------------------------------------------------------------------------

typedef std::vector<CFilterRef> COLOR_FILTER_REF_ARRAY;


// ---------------------------------------------------------------------------
// color filter class
// ---------------------------------------------------------------------------
class CFilter {
private:
	xstr_t m_strID;
	xstr_t m_strName;
	UINT m_uType;
	BOOL m_bVisible;
	BOOL m_bEnableInner;
	BOOL m_bNormalFilter;
	COLORREF m_aColorMap[216];
	COLORREF* m_pColorMapEx;
	COLOR_FILTER_REF_ARRAY m_RefFilters;
	int m_nRepeat;
	DWORD m_dwHotKey;

	BOOL IsValidFilterXml(MSXML2::IXMLDOMDocumentPtr pXMLDoc);
	BOOL IsValidRootTag(MSXML2::IXMLDOMDocumentPtr& pXMLDoc);
	BOOL IsValidProfileTag(MSXML2::IXMLDOMElementPtr& pRootElement);
	BOOL IsValidColorMapTag(MSXML2::IXMLDOMElementPtr& pRootElement);
	BOOL IsValidColorTags(MSXML2::IXMLDOMElementPtr& pColorMapElement);
	BOOL IsValidFilterListTag(MSXML2::IXMLDOMElementPtr& pRootElement);
	BOOL IsValidFilterTags(MSXML2::IXMLDOMElementPtr& pFilterListElement);
	BOOL IsValidFilterName(LPCTSTR pszName);
	BOOL IsValidFilterID(LPCTSTR pszID);
	BOOL IsValidType(LPCTSTR pszType);
	BOOL IsValidVisible(LPCTSTR pszVisible);
	BOOL IsValidRepeat(LPCTSTR pszRepeat);
	BOOL IsValidTimeout(LPCTSTR pszTimeout);

public:
	BOOL CreateColorMapEx();
	void DeleteColorMapEx();

public:
	CFilter();
	CFilter(LPCTSTR pszID, LPCTSTR pszName, COLORREF* pColorMap);
	CFilter(const CFilter& filter);
	virtual ~CFilter();
	CFilter& operator=(const CFilter& filter);

	BOOL IsDefFilter() {
		return (m_uType == FILTERTYPE_DEFINITION);
	}
	BOOL IsRefFilter() {
		return (m_uType == FILTERTYPE_REFERENCE);
	}
	void SetID(LPCTSTR pszID) {
		m_strID = pszID;
	}
	LPCTSTR GetID() {
		return m_strID;
	}
	void SetName(LPCTSTR pszName) {
		m_strName = pszName;
	}
	LPCTSTR GetName() {
		return m_strName;
	}
	void SetNormalFilter(BOOL bEnable) {
		m_bNormalFilter = bEnable;
	}
	BOOL IsNormalFilter() const {
		return m_bNormalFilter;
	}
	void SstVisible(BOOL bVisible) {
		m_bVisible = bVisible;
	}
	BOOL IsVisible() const {
		return m_bVisible;
	}
	void SstEnableInner(BOOL bEnable) {
		m_bEnableInner = bEnable;
	}
	BOOL IsEnableInner() const {
		return m_bEnableInner;
	}
	void SetHotKey(DWORD dwHotKey) {
		m_dwHotKey = dwHotKey;
	}
	DWORD GetHotKey() {
		return m_dwHotKey;
	}
	void SetColorMap(COLORREF* pColorMap) {
		memcpy(m_aColorMap, pColorMap, sizeof(m_aColorMap));
	}

#if 0
	COLORREF ConvertColor(COLORREF clr);
	COLORREF ConvertColorFromIndex(int nColorIndex);
	COLORREF* GetColorMapTable();
#endif
	COLORREF ConvertColorEx(COLORREF clr);
	COLORREF ConvertColorExFromIndex(int nColorIndex);
	COLORREF* GetColorMapExTable();

	BOOL ReadFile(LPCTSTR pszPath);

#if 0
	static int GetColormapIndex(COLORREF clr);
#endif
	static int GetColormapExIndex(COLORREF clr);
	static CFilter* CreateNormalFileter();

	int GetRefFilterCount();
	void SetRepeat(int nRepeat);
	int GetRepeat();
	BOOL GetFilterRef(int nIndex, CFilterRef& ref);

	BOOL IsValidateRefFilter();
};


// ---------------------------------------------------------------------------

typedef std::vector<CFilter> COLOR_FILTER_ARRAY;
typedef std::vector<CFilter*> COLOR_FILTER_PTR_ARRAY;


// ---------------------------------------------------------------------------
// color filter manage class
// ---------------------------------------------------------------------------
class CFilterManager {
private:
//	COLOR_FILTER_ARRAY m_Filters;
	COLOR_FILTER_PTR_ARRAY m_Filters;
	COLOR_FILTER_PTR_ARRAY m_EnableFilters;

	void Initialize();

public:
	CFilterManager();
	virtual ~CFilterManager();

public:
	BOOL IsRegistered(LPCTSTR pszID) {
		int i;
		for (i = 0; i < (int)m_Filters.size(); i++) {
			CFilter* pFilter = m_Filters[i];
			if (lstrcmpi(pFilter->GetID(), pszID) == 0) {
				return TRUE;
			}
		}
		return FALSE;
	}
	BOOL IsRegisteredName(LPCTSTR pszName) {
		int i;
		for (i = 0; i < (int)m_Filters.size(); i++) {
			CFilter* pFilter = m_Filters[i];
			if (lstrcmp(pFilter->GetName(), pszName) == 0) {
				return TRUE;
			}
		}
		return FALSE;
	}
	BOOL Register(CFilter* pFilter) {
		if (pFilter == NULL) {
			return FALSE;
		}
		if (GetFilter(pFilter->GetID()) != NULL) {
			return FALSE;
		}
		if ((int)m_Filters.size() >= MAX_FILTER) {
			return FALSE;
		}
		m_Filters.push_back(pFilter);
		return TRUE;
	}
	void ClearEnableFilter() {
		m_EnableFilters.clear();
	}
	BOOL RegisterEnableFilter(CFilter* pFilter) {
		m_EnableFilters.push_back(pFilter);
		return TRUE;
	}
	int GetFilterCount() {
		return m_Filters.size();
	}
	int GetEnableFilterCount() {
		return m_EnableFilters.size();
	}
	CFilter* GetNormalFilter() {
		return GetFilter(FILTER_ID_NORMAL);
	}
	CFilter* GetFilter(int index) {
		if (index < 0 || index > (int)m_Filters.size()) {
			return NULL;
		}
		CFilter* pFilter = m_Filters[index];
		return pFilter;
	}
	CFilter* GetFilter(LPCTSTR pszID) {
		int i;
		for (i = 0; i < (int)m_Filters.size(); i++) {
			CFilter* pFilter = m_Filters[i];
			if (lstrcmpi(pFilter->GetID(), pszID) == 0) {
				return pFilter;
			}
		}
		return NULL;
	}
	CFilter* GetEnableFilter(int index) {
		if (index < 0 || index > (int)m_EnableFilters.size()) {
			return NULL;
		}
		CFilter* pFilter = m_EnableFilters[index];
		return pFilter;
	}
	CFilter* GetEnableFilter(LPCTSTR pszID) {
		int i;
		for (i = 0; i < (int)m_EnableFilters.size(); i++) {
			CFilter* pFilter = m_EnableFilters[i];
			if (lstrcmpi(pFilter->GetID(), pszID) == 0) {
				return pFilter;
			}
		}
		return NULL;
	}
	void SetEnableFilter(XSTR_ARRAY& FilterList) {
		int i;
		for (i = 0; i < (int)FilterList.size(); i++) {
			xstr_t& s = FilterList[i];
			CFilter* pFilter = GetFilter(s);
			if (pFilter != NULL) {
				if (GetEnableFilter(s) == NULL) {
					if (pFilter->IsVisible()) {
						RegisterEnableFilter(pFilter);
					}
				}
			}
		}
		CFilter* pNormalFilter = GetNormalFilter();
		if (pNormalFilter != NULL) {
			if (GetEnableFilter(FILTER_ID_NORMAL) == NULL) {
				RegisterEnableFilter(pNormalFilter);
			}
		}
	}
	void GetEnableFilter(XSTR_ARRAY& FilterList) {
		FilterList.clear();
		int i;
		for (i = 0; i < (int)m_EnableFilters.size(); i++) {
			CFilter* pFilter = m_EnableFilters[i];
			FilterList.push_back(pFilter->GetID());
		}
	}
	int GetEnableFilterIndex(CFilter* pTargetFilter) {
		if (pTargetFilter == NULL) {
			return -1;
		}
		int i;
		for (i = 0; i < (int)m_EnableFilters.size(); i++) {
			CFilter* pFilter = m_EnableFilters[i];
			if (pFilter == pTargetFilter) {
				return i;
			}
		}
		return -1;
	}
	int GetEnableFilterIndexFromID(LPCTSTR pszID) {
		int i;
		for (i = 0; i < (int)m_EnableFilters.size(); i++) {
			CFilter* pFilter = m_EnableFilters[i];
			if (lstrcmp(pFilter->GetID(), pszID) == 0) {
				return i;
			}
		}
		return 0;
	}
	void GetFilter(XSTR_ARRAY& FilterList) {
		FilterList.clear();
		int i;
		for (i = 0; i < (int)m_Filters.size(); i++) {
			CFilter* pFilter = m_Filters[i];
			FilterList.push_back(pFilter->GetID());
		}
	}
	COLOR_FILTER_PTR_ARRAY& GetFilters() {
		return m_Filters;
	}
	COLOR_FILTER_PTR_ARRAY& GetEnableFilters() {
		return m_EnableFilters;
	}
	void GetDisableFilter(COLOR_FILTER_PTR_ARRAY& FilterPtrList) {
		FilterPtrList.clear();
		int i;
		for (i = 0; i < (int)m_Filters.size(); i++) {
			CFilter* pFilter = GetFilter(i);
			if (pFilter == NULL) {
				// error
				continue;
			}
			if (GetEnableFilter(pFilter->GetID()) == NULL) {
				FilterPtrList.push_back(pFilter);
			}
		}
	}
	BOOL ReadFilters(LPCTSTR pszFilterDir);

	BOOL ActivateFilters() {
		int i;
		for (i = 0; i < (int)m_Filters.size(); i++) {
			CFilter* pFilter = GetFilter(i);
			if (pFilter->IsRefFilter()) {
				continue;
			}
			pFilter->SstEnableInner(FALSE);
		}
		int nEnableCount = GetEnableFilterCount();
		for (i = 0; i < nEnableCount; i++) {
			CFilter* pFilter = GetEnableFilter(i);
			if (pFilter->IsDefFilter()) {
				pFilter->SstEnableInner(TRUE);
			}
			else {
				int nRefCount = pFilter->GetRefFilterCount();
				int j;
				for (j = 0; j < nRefCount; j++) {
					CFilterRef ref;
					if (! pFilter->GetFilterRef(j, ref)) {
						continue;
					}
					CFilter* pRefFilter = ref.GetFilter();
					pRefFilter->SstEnableInner(TRUE);
				}
			}
		}
		for (i = 0; i < (int)m_Filters.size(); i++) {
			CFilter* pFilter = GetFilter(i);
			if (pFilter->IsRefFilter()) {
				continue;
			}
			if (pFilter->IsEnableInner()) {
				pFilter->CreateColorMapEx();
			}
			else {
				pFilter->DeleteColorMapEx();
			}
		}
		return TRUE;
	}
};
