// ===========================================================================
// FilterOptionDlg.cpp
// filter option dialog
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include <CeLib.h>
#include "AnotherVision.h"
#include "Profile.h"
#include "Filter.h"
#include "FilterWnd.h"
#include "PreviewWnd.h"
#include "OptionDlg.h"
#include "FilterOptionDlg.h"


// ---------------------------------------------------------------------------

HWND CFilterOptionDlg::ms_hwndDlg = NULL;
volatile
BOOL CFilterOptionDlg::ms_bShowDlg = FALSE;


// ---------------------------------------------------------------------------
// check dialog showed
// ---------------------------------------------------------------------------
BOOL CFilterOptionDlg::IsShowed()
{
	return ms_bShowDlg;
}

// ---------------------------------------------------------------------------
// show dialog
// ---------------------------------------------------------------------------
BOOL CFilterOptionDlg::Show(CHotKeyManager& HotKeyManager)
{
	if (COptionDlg::IsShowed() ||
		CPreviewWnd::IsShowed()) {
		return FALSE;
	}
	if (CFilterWnd::GetWndListCount() > 0) {
		return FALSE;
	}

	BOOL bResult = FALSE;
	if (ms_bShowDlg) {
		if (ms_hwndDlg != NULL) {
			SetForegroundWindow(ms_hwndDlg);
		}
	}
	else {
		ms_bShowDlg = TRUE;
		CFilterOptionDlg dlg;
		dlg.SetHotKeyManager(HotKeyManager);
		INT_PTR nResult = dlg.DoModal();
		if (nResult == IDOK) {
			HotKeyManager = dlg.GetHotKeyManager();
			HotKeyManager.SaveProfile();
			GetAppProfile()->WriteProfile();
			bResult = TRUE;
		}
		ms_bShowDlg = FALSE;
	}
	return bResult;
}

// ---------------------------------------------------------------------------
// load profile
// ---------------------------------------------------------------------------
BOOL CFilterOptionDlg::LoadProfile()
{
	CAppProfile* pProfile = GetAppProfile();

	WORD wKeyCode = LOWORD(pProfile->GetViewHotKey());
	WORD wModifiers = HIWORD(pProfile->GetViewHotKey());

	COLOR_FILTER_PTR_ARRAY& FilterPtrList = GetAppFilterManager()->GetEnableFilters();
	int i;
	for (i = 0; i < (int)FilterPtrList.size(); i++) {
		m_EnableFilterPtrList.push_back(FilterPtrList[i]);
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// check hotkey
// ---------------------------------------------------------------------------
BOOL CFilterOptionDlg::IsValidHotkeySetting()
{
	UpdateHotKeyData();
	xstr_t strErrorMsg;
	if (m_HotKeyManager.IsLimitExceeded(strErrorMsg)) {
		ExMessageBoxText(MB_OK, (LPCTSTR)strErrorMsg);
		return FALSE;
	}
	if (m_HotKeyManager.IsConflict(TRUE, strErrorMsg)) {
		ExMessageBoxText(MB_OK, (LPCTSTR)strErrorMsg);
		return FALSE;
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// save profile
// ---------------------------------------------------------------------------
BOOL CFilterOptionDlg::SaveProfile()
{
	UpdateHotKeyData();
	CAppProfile* pProfile = GetAppProfile();
	CFilterManager* pFilterManager = GetAppFilterManager();
	COLOR_FILTER_PTR_ARRAY& FilterPtrList = pFilterManager->GetEnableFilters();
	FilterPtrList.clear();
	int nListCount = (int)m_EnableFilterPtrList.size();
	int i;
	for (i = 0; i < nListCount; i++) {
		FilterPtrList.push_back(m_EnableFilterPtrList[i]);
	}

	return TRUE;
}

// ---------------------------------------------------------------------------
// control-valiable mapping
// ---------------------------------------------------------------------------
void CFilterOptionDlg::ControlVariableMapping()
{
	m_FilterListbox = GetDlgItem(IDC_FOPTION_FILTER_E_LIST);
	m_FilterUpBtn = GetDlgItem(IDC_FOPTION_FILTER_UP_BUTTON);
	m_FilterDownBtn = GetDlgItem(IDC_FOPTION_FILTER_DOWN_BUTTON);
	m_FilterEnableBtn = GetDlgItem(IDC_FOPTION_FILTER_ENABLE_BUTTON);
	m_FilterDisableBtn = GetDlgItem(IDC_FOPTION_FILTER_DISABLE_BUTTON);
	m_DisabledFilterListbox = GetDlgItem(IDC_FOPTION_FILTER_D_LIST);
	m_QuickHotkey = GetDlgItem(IDC_FOPTION_QUICK_HOTKEY);
	m_IDEdit = GetDlgItem(IDC_FOPTION_ID_EDIT);
	m_NameEdit = GetDlgItem(IDC_FOPTION_NAME_EDIT);
}

// ---------------------------------------------------------------------------
// initialize control
// ---------------------------------------------------------------------------
void CFilterOptionDlg::InitControls()
{
}

// ---------------------------------------------------------------------------
// OnInitDialog
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	ms_hwndDlg = m_hWnd;
	m_pPrevSelectFilter = NULL;
	CenterWindow(GetParent());
	ControlVariableMapping();
	InitControls();
	LoadProfile();
	EnableButtons(TRUE);

	return TRUE;
}

// ---------------------------------------------------------------------------
// OnOK
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnOK(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (! IsValidHotkeySetting()) {
		return 0;
	}
	SaveProfile();
	GetAppProfile()->WriteProfile();
	EndDialog(IDOK);
	ms_hwndDlg = NULL;
	return 0;
}

// ---------------------------------------------------------------------------
// OnCancel
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnCancel(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	EndDialog(IDCANCEL);
	ms_hwndDlg = NULL;
	return 0;
}

// ---------------------------------------------------------------------------
// OnControlStatusChanged
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnControlStatusChanged(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	EnableButtons();
	return 0;
}

// ---------------------------------------------------------------------------
// EnableButtons
// ---------------------------------------------------------------------------
void CFilterOptionDlg::EnableButtons(BOOL bUpdateList/*= FALSE*/)
{
	int i;
	if (bUpdateList) {
		int nIndex = m_FilterListbox.GetCurSel();
		m_FilterListbox.ResetContent();
		for (i = 0; i < (int)m_EnableFilterPtrList.size(); i++) {
			CFilter* pFilter = m_EnableFilterPtrList[i];
			int nInsertPos = m_FilterListbox.AddString(pFilter->GetName());
			m_FilterListbox.SetItemData(nInsertPos, (LPARAM)(void*)pFilter);
		}
		if (nIndex >= (int)m_EnableFilterPtrList.size()) {
			nIndex = m_EnableFilterPtrList.size() - 1;
		}
		if (nIndex < 0) {
			nIndex = 0;
		}
		m_FilterListbox.SetCurSel(nIndex);

		COLOR_FILTER_PTR_ARRAY DisabledFilterPtrList;
		GetDisabledFilterList(DisabledFilterPtrList);
		m_DisabledFilterListbox.ResetContent();
		for (i = 0; i < (int)DisabledFilterPtrList.size(); i++) {
			CFilter* pFilter = DisabledFilterPtrList[i];
			int nInsertPos = m_DisabledFilterListbox.AddString(pFilter->GetName());
			m_DisabledFilterListbox.SetItemData(nInsertPos, (LPARAM)(void*)pFilter);
		}
	}
	int nEnabledIndex = m_FilterListbox.GetCurSel();
	int nDisabledSelCount = m_DisabledFilterListbox.GetSelCount();
	HWND hwndFocus = GetFocus();
	BOOL bButtonFocus = FALSE;
	if (hwndFocus == m_FilterUpBtn ||
		hwndFocus == m_FilterDownBtn ||
		hwndFocus == m_FilterDisableBtn ||
		hwndFocus == m_FilterEnableBtn) {
		bButtonFocus = TRUE;
	}
	int nEnableFilterCount = m_EnableFilterPtrList.size();
	int nDisableFilterCount = GetAppFilterManager()->GetFilterCount() - m_EnableFilterPtrList.size();
	if (nEnabledIndex != LB_ERR) {
		BOOL bEnableUpButton = (nEnabledIndex != 0);
		BOOL bEnableDownButton = (nEnabledIndex != nEnableFilterCount - 1);
		BOOL bEnableDisableButton = (! m_EnableFilterPtrList[nEnabledIndex]->IsNormalFilter());
		BOOL bEnableEnableButton = (nDisabledSelCount > 0);
		if (bButtonFocus) {
			if (hwndFocus == m_FilterUpBtn &&
				! bEnableUpButton) {
				hwndFocus = m_FilterDownBtn;
			}
			if (hwndFocus == m_FilterDownBtn &&
				! bEnableDownButton) {
				hwndFocus = m_FilterDisableBtn;
			}
			if (hwndFocus == m_FilterDisableBtn &&
				! bEnableDisableButton) {
				hwndFocus = m_FilterEnableBtn;
			}
			if (hwndFocus == m_FilterEnableBtn &&
				! bEnableEnableButton) {
				hwndFocus = m_DisabledFilterListbox;
			}
		}
		m_FilterUpBtn.EnableWindow(bEnableUpButton);
		m_FilterDownBtn.EnableWindow(bEnableDownButton);
		m_FilterDisableBtn.EnableWindow(bEnableDisableButton);
		m_FilterEnableBtn.EnableWindow(bEnableEnableButton);
		if (bButtonFocus) {
			::SetFocus(hwndFocus);
		}
	}
	else {
		if (bButtonFocus) {
			if (hwndFocus == m_FilterUpBtn ||
				hwndFocus == m_FilterDownBtn ||
				hwndFocus == m_FilterDisableBtn) {
				hwndFocus = m_FilterEnableBtn;
			}
			if (hwndFocus == m_FilterEnableBtn &&
				nDisableFilterCount <= 0) {
				hwndFocus = m_DisabledFilterListbox;
			}
		}
		m_FilterUpBtn.EnableWindow(FALSE);
		m_FilterDownBtn.EnableWindow(FALSE);
		m_FilterDisableBtn.EnableWindow(FALSE);
		m_FilterEnableBtn.EnableWindow(nDisableFilterCount > 0);
		if (bButtonFocus) {
			::SetFocus(hwndFocus);
		}
	}
	if (nEnabledIndex != LB_ERR) {
		CFilter* pFilter = m_EnableFilterPtrList[nEnabledIndex];
		m_IDEdit.SetWindowText(pFilter->GetID());
		m_NameEdit.SetWindowText(pFilter->GetName());
	}
	else {
		m_IDEdit.SetWindowText(_T(""));
		m_NameEdit.SetWindowText(_T(""));
	}
	UpdateHotKeyData();
}

// ---------------------------------------------------------------------------
// UpdateHotKeyData
// ---------------------------------------------------------------------------
void CFilterOptionDlg::UpdateHotKeyData()
{
	CFilter* pPrevFilter = m_pPrevSelectFilter;
	CFilter* pCurrentFilter = NULL;
	int nIndex = m_FilterListbox.GetCurSel();
	if (nIndex != LB_ERR) {
		pCurrentFilter = (CFilter* )m_FilterListbox.GetItemData(nIndex);
	}
	if (pPrevFilter != NULL) {
		m_HotKeyManager.SetQuickHotKeyForControl(pPrevFilter, m_QuickHotkey.GetHotKey());
	}
	if (pPrevFilter != pCurrentFilter) {
		if (pCurrentFilter == NULL) {
			m_QuickHotkey.SetHotKey(0, 0);
		}
		else {
			DWORD dwHotKey = m_HotKeyManager.GetQuickHotKeyForControl(pCurrentFilter);
			WORD wKeyCode = LOWORD(dwHotKey);
			WORD wModifiers = HIWORD(dwHotKey);
			m_QuickHotkey.SetHotKey(wKeyCode, wModifiers);
		}
	}
	m_pPrevSelectFilter = pCurrentFilter;
}

// ---------------------------------------------------------------------------
// OnUpItem
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnUpItem(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int nIndex = m_FilterListbox.GetCurSel();
	if (nIndex == LB_ERR || nIndex == 0) {
		return 0;
	}
	CFilter* pFilter = m_EnableFilterPtrList[nIndex];
	m_EnableFilterPtrList.erase(m_EnableFilterPtrList.begin() + nIndex);
	m_EnableFilterPtrList.insert(m_EnableFilterPtrList.begin() + (nIndex - 1), pFilter);

	m_FilterListbox.DeleteString(nIndex);
	m_FilterListbox.InsertString(nIndex - 1, pFilter->GetName());
	m_FilterListbox.SetItemData(nIndex - 1, (LPARAM)(void*)pFilter);
	m_FilterListbox.SetCurSel(nIndex - 1);

	EnableButtons(FALSE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnDownItem
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnDownItem(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int nIndex = m_FilterListbox.GetCurSel();
	if (nIndex == LB_ERR || nIndex == m_EnableFilterPtrList.size() - 1) {
		return 0;
	}
	CFilter* pFilter = m_EnableFilterPtrList[nIndex];
	m_EnableFilterPtrList.erase(m_EnableFilterPtrList.begin() + nIndex);
	m_EnableFilterPtrList.insert(m_EnableFilterPtrList.begin() + (nIndex + 1), pFilter);

	m_FilterListbox.DeleteString(nIndex);
	m_FilterListbox.InsertString(nIndex + 1, pFilter->GetName());
	m_FilterListbox.SetItemData(nIndex + 1, (LPARAM)(void*)pFilter);
	m_FilterListbox.SetCurSel(nIndex + 1);

	EnableButtons(FALSE);
	return 0;
}

// ---------------------------------------------------------------------------
// EnableFilter
// ---------------------------------------------------------------------------
void CFilterOptionDlg::EnableFilter(COLOR_FILTER_PTR_ARRAY& EnableFilterList)
{
	int nCount = EnableFilterList.size();
	int i;
	for (i = 0; i < nCount; i++) {
		m_EnableFilterPtrList.push_back(EnableFilterList[i]);
	}
	return;
}

// ---------------------------------------------------------------------------
// DisableFilter
// ---------------------------------------------------------------------------
void CFilterOptionDlg::DisableFilter(int nIndex)
{
	if (nIndex < 0 || nIndex >= (int)m_EnableFilterPtrList.size()) {
		return;
	}
	CFilter* pFilter = m_EnableFilterPtrList[nIndex];
	m_HotKeyManager.SetQuickHotKey(pFilter, 0);
	m_EnableFilterPtrList.erase(m_EnableFilterPtrList.begin() + nIndex);
	m_QuickHotkey.SetHotKey(0, 0);
	return;
}

// ---------------------------------------------------------------------------
// OnFilterDisableItem
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnFilterDisableItem(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int nIndex = m_FilterListbox.GetCurSel();
	if (nIndex != LB_ERR) {
		DisableFilter(nIndex);
		EnableButtons(TRUE);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnFilterEnableItem
// ---------------------------------------------------------------------------
LRESULT CFilterOptionDlg::OnFilterEnableItem(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int nCount = m_DisabledFilterListbox.GetSelCount();
	if (nCount == 0) {
		return 0;
	}
	int* pnFilterIndexes = new int[nCount];
	if (pnFilterIndexes == NULL) {
		// error
		return 0;
	}
	m_DisabledFilterListbox.GetSelItems(nCount, pnFilterIndexes);
	int i;
	for (i = 0; i < nCount; i++) {
		CFilter* pFilter = (CFilter*)m_DisabledFilterListbox.GetItemData(pnFilterIndexes[i]);
		m_EnableFilterPtrList.push_back(pFilter);
	}
	delete[] pnFilterIndexes;
	EnableButtons(TRUE);
	return 0;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterOptionDlg::GetDisabledFilterList(COLOR_FILTER_PTR_ARRAY& DisabledFilterPtrList)
{
	DisabledFilterPtrList.clear();
	m_EnableFilterPtrList;
	COLOR_FILTER_PTR_ARRAY& FilterList = GetAppFilterManager()->GetFilters();
	int i;
	for (i = 0; i < (int)FilterList.size(); i++) {
		CFilter* pFilter = FilterList[i];
		LPCTSTR pszID = pFilter->GetID();
		BOOL bFound = FALSE;
		int j;
		for (j = 0; j < (int)m_EnableFilterPtrList.size(); j++) {
			CFilter* pFilter = m_EnableFilterPtrList[j];
			if (lstrcmpi(pFilter->GetID(), pszID) == 0) {
				bFound = TRUE;
				break;
			}
		}
		if (! bFound) {
			if (pFilter->IsVisible()) {
				DisabledFilterPtrList.push_back(pFilter);
			}
		}
	}
}
