// ===========================================================================
// FilterStatus.cpp
// color filter status
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#define _USE_MATH_DEFINES
#include <math.h>
#include "AnotherVision.h"
#include "common.h"
#include "profile.h"
#include "AppException.h"
#include "Filter.h"
#include "FilterStatus.h"


CFilterStatus::CFilterStatus()
{
	m_bQuickMode = FALSE;
	m_nSelectedEnableFilterIndex = 0;
	m_pCurrentFilter = NULL;
	m_nCurrentRepeatCount = 0;
	m_nCurrentRefFilterIndex = 0;
	m_nCurrentTimeout = 0;
	m_pCurrentRawFilter = NULL;
	m_dwTime = 0;
}

CFilterStatus::CFilterStatus(const CFilterStatus& status)
{
	*this = status;
}

CFilterStatus::~CFilterStatus()
{
}

CFilterStatus& CFilterStatus::operator=(const CFilterStatus& status)
{
	m_bQuickMode = status.m_bQuickMode;
	m_nSelectedEnableFilterIndex = status.m_nSelectedEnableFilterIndex;
	m_pCurrentFilter = status.m_pCurrentFilter;
	m_nCurrentRepeatCount = status.m_nCurrentRepeatCount;
	m_nCurrentRefFilterIndex = status.m_nCurrentRefFilterIndex;
	m_nCurrentTimeout = status.m_nCurrentTimeout;
	m_pCurrentRawFilter = status.m_pCurrentRawFilter;
	m_dwTime = status.m_dwTime;
	return *this;
}

void CFilterStatus::SetQuickMode(BOOL bEnable/*= TRUE*/)
{
	m_bQuickMode = bEnable;
}

void CFilterStatus::ResetSelectedEnableFilterIndex()
{
	m_nSelectedEnableFilterIndex = 0;
	m_pCurrentFilter = NULL;
}

void CFilterStatus::SetSelectedEnableFilterIndex(int nIndex)
{
	int nCount = GetAppFilterManager()->GetEnableFilterCount();
	if (nIndex > nCount) {
		return;
	}
	m_nSelectedEnableFilterIndex = nIndex;
	m_pCurrentFilter = GetAppFilterManager()->GetEnableFilter(nIndex);
}

int CFilterStatus::GetSelectedEnableFilterIndex()
{
	return m_nSelectedEnableFilterIndex;
}

// safe
CFilter* CFilterStatus::GetSelectedEnableFilter()
{
	CFilter* pFilter = GetAppFilterManager()->GetEnableFilter(m_nSelectedEnableFilterIndex);
	if (pFilter == NULL) {
		return GetAppFilterManager()->GetNormalFilter();
	}
	return pFilter;
}

void CFilterStatus::ResetFilterItem()
{
	if (m_pCurrentFilter == NULL) {
		return;
	}
	if (! m_pCurrentFilter->IsRefFilter()) {
		return;
	}
	m_nRepeat = m_pCurrentFilter->GetRepeat();
	if (m_bQuickMode) {
		if (m_nRepeat == 0) {
			m_nRepeat = 1;
		}
	}
	else {
		m_nRepeat = 0;
	}
	m_nCurrentRepeatCount = 0;
	m_nCurrentRefFilterIndex = 0;
	m_nCurrentTimeout = 0;
	m_dwTime = GetTickCount();
}

BOOL CFilterStatus::HasNextFilterItem()
{
	if (m_pCurrentFilter == NULL) {
		return FALSE;
	}
	if (! m_pCurrentFilter->IsRefFilter()) {
		return FALSE;
	}
	if (m_nCurrentRefFilterIndex >= m_pCurrentFilter->GetRefFilterCount()) {
		return FALSE;
	}
	if (m_nRepeat != 0) {
		if (m_nCurrentRepeatCount >= m_nRepeat) {
			return FALSE;
		}
	}
	return TRUE;
}

BOOL CFilterStatus::IsNextFilterItemReady()
{
	if (! HasNextFilterItem()) {
		return FALSE;
	}
	if (m_nCurrentRefFilterIndex >= m_pCurrentFilter->GetRefFilterCount()) {
		return FALSE;
	}
	if (m_nRepeat != 0) {
		if (m_nCurrentRepeatCount >= m_nRepeat) {
			return FALSE;
		}
	}
	DWORD dwTime = GetTickCount() - m_dwTime;
	if ((int)dwTime < m_nCurrentTimeout) {
		return FALSE;
	}
	return TRUE;
}

CFilter* CFilterStatus::NextFilterItem()
{
	if (! IsNextFilterItemReady()) {
		return NULL;
	}
	CFilterRef ref;
	m_pCurrentFilter->GetFilterRef(m_nCurrentRefFilterIndex, ref);
	CFilter* pFilter = ref.GetFilter();
	m_nCurrentTimeout = ref.GetTimeout();
	++m_nCurrentRefFilterIndex;
	if (m_nCurrentRefFilterIndex >= m_pCurrentFilter->GetRefFilterCount()) {
		m_nCurrentRefFilterIndex = 0;
		if (m_nRepeat != 0) {
			++m_nCurrentRepeatCount;
		}
	}
	m_dwTime = GetTickCount();
	m_pCurrentRawFilter = pFilter;

	return pFilter;
}

CFilter* CFilterStatus::LastFilterItem()
{
	return m_pCurrentRawFilter;
}
