// ===========================================================================
// FilterWnd.cpp
// operate panel window
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include <celib.h>
#include "AnotherVision.h"
#include "Profile.h"
#include "OptionDlg.h"
#include "FilterOptionDlg.h"
#include "Filter.h"
#include "ColorUtils.h"
#include "CaptureImage.h"
#include "FilterWnd.h"


// ---------------------------------------------------------------------------

#define FILTERWND_TIMER_ID         100
#define FILTERWND_TIMER_ELAPSE     50


// ---------------------------------------------------------------------------

FILTER_WND_PTR_ARRAY CFilterWnd::ms_FilterWndPtrList;
RECT CFilterWnd::ms_rcPrevWindowRect = {0, 0, 0, 0};
BOOL CFilterWnd::ms_bUsePrevWindowRect = FALSE;


// ---------------------------------------------------------------------------
// create filter window
// ---------------------------------------------------------------------------
BOOL CreateFilterWnd(CWindowSetting* pSetting/* = NULL*/)
{
	if (CFilterWnd::GetWndListCount() >= MAX_FILTER_WND) {
		return TRUE;
	}
	if (CFilterOptionDlg::IsShowed()) {
		return TRUE;
	}

	CFilterWnd* pWnd = new CFilterWnd();
	if (pWnd == NULL) {
		return FALSE;
	}
	if (! pWnd->CreateHelper(pSetting)) {
		delete pWnd;
		return FALSE;
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::PostMessageToAllWindow(UINT uMsg, WPARAM wParam/*=0*/, LPARAM lParam/*=0*/)
{
	int i;
	for (i = 0; i < (int)ms_FilterWndPtrList.size(); i++) {
		CFilterWnd* pWnd = ms_FilterWndPtrList[i];
		pWnd->PostMessage(uMsg, wParam, lParam);
	}
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
int CFilterWnd::GetWndListCount()
{
	return (int)ms_FilterWndPtrList.size();
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::AddToWndList(CFilterWnd* pWnd)
{
	ms_FilterWndPtrList.push_back(pWnd);
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::EraseFromWndList(CFilterWnd* pTargetWnd)
{
	int i;
	for (i = 0; i < (int)ms_FilterWndPtrList.size(); i++) {
		CFilterWnd* pWnd = ms_FilterWndPtrList[i];
		if (pTargetWnd == pWnd) {
			ms_FilterWndPtrList.erase(ms_FilterWndPtrList.begin() + i);
			break;
		}
	}
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
CFilterWnd* CFilterWnd::GetTopElementOfWndList()
{
	if (ms_FilterWndPtrList.size() == 0) {
		return NULL;
	}
	return ms_FilterWndPtrList[0];
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::ClearWndList()
{
	ms_FilterWndPtrList.clear();
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::CloseAllWindow()
{
	PostMessageToAllWindow(WM_CLOSE);
	ClearWndList();
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::ActivateAllWindow()
{
	PostMessageToAllWindow(WM_EX_WINDOW_TOP);
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::LoadAllWindowSetting()
{
	int nWndCount = GetAppProfile()->GetMultiWindowSettingCount();
	if (nWndCount == 0) {
		return;
	}
	int i;
	for (i = 0; i < (int)ms_FilterWndPtrList.size(); i++) {
		CFilterWnd* pWnd = ms_FilterWndPtrList[i];
		pWnd->m_bInvalidated = TRUE;
	}
	CloseAllWindow();
	for (i = 0; i < nWndCount; i++) {
		CWindowSetting setting = GetAppProfile()->GetMultiWindowSetting(i);
		CreateFilterWnd(&setting);
	}
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::SaveAllWindowSetting()
{
	if ((int)ms_FilterWndPtrList.size() <= 0) {
		return;
	}
	GetAppProfile()->ClearMultiWindowSettings();
	int i;
	for (i = 0; i < (int)ms_FilterWndPtrList.size(); i++) {
		CFilterWnd* pWnd = ms_FilterWndPtrList[i];
		GetAppProfile()->AddToMultiWindowSetting(pWnd->SaveWindowSetting());
	}
}

// ---------------------------------------------------------------------------
// helper of create window
// ---------------------------------------------------------------------------
BOOL CFilterWnd::CreateHelper(CWindowSetting* pSetting)
{
	SetInitialSetting(pSetting);
	DWORD dwStyle = WS_VISIBLE | WS_CAPTION | WS_POPUP | WS_THICKFRAME | WS_SYSMENU | WS_MINIMIZEBOX;
	DWORD dwExStyle = 0;
	if (GetAppProfile()->GetWindowUnvisibleTaskBar()) {
		dwExStyle |= WS_EX_TOOLWINDOW;
	}
	if (m_InitialSetting.IsTopmost()) {
		dwExStyle |= WS_EX_TOPMOST;
	}
	RECT rc;
	GetInitialWindowRect(&rc, &m_InitialSetting, m_bEnableInitialSetting);
	if (Create(NULL, &rc, APP_WINDOW_TITLE, dwStyle, dwExStyle, (HMENU)NULL, NULL) == NULL) {
		return FALSE;
	}
	AddToWndList(this);
	return TRUE;
}

// ---------------------------------------------------------------------------
// create filter window
// ---------------------------------------------------------------------------
void CFilterWnd::GetInitialWindowRect(LPRECT pRect, CWindowSetting* pSetting, BOOL bEnableInitialSetting)
{
	static RECT rcInit = {0, 0, 200, 200};
//	CWindowSetting& setting = GetAppProfile()->GetWindowSetting();
	RECT rc;
	pSetting->GetRect(&rc);
	if (rc.left == 0 && rc.top == 0 &&
		rc.right == 0 && rc.bottom == 0) {
		rc = rcInit;
	}
	if (! bEnableInitialSetting) {
		if (ms_bUsePrevWindowRect) {
			rc = ms_rcPrevWindowRect;
			int nOffset = GetSystemMetrics(SM_CYCAPTION);
			OffsetRect(&rc, nOffset, nOffset);
		}
		ms_bUsePrevWindowRect = TRUE;
	}
	int nDesktopWidth = GetSystemMetrics(SM_CXSCREEN);
	int nDesktopHeight = GetSystemMetrics(SM_CYSCREEN);
	if (rc.left < 0 ||
		rc.right >= nDesktopWidth) {
		OffsetRect(&rc, -rc.left, 0);
	}
	if (rc.top < 0 ||
		rc.bottom >= nDesktopHeight) {
		OffsetRect(&rc, 0, -rc.top);
	}
	ms_rcPrevWindowRect = rc;
	*pRect = rc;
}

// ---------------------------------------------------------------------------
// constructor
// ---------------------------------------------------------------------------
CFilterWnd::CFilterWnd()
{
	m_bEnableInitialSetting = FALSE;
}

// ---------------------------------------------------------------------------
// CreateStatusBar
// ---------------------------------------------------------------------------
BOOL CFilterWnd::CreateStatusBar()
{
	if (m_StatusBar.m_hWnd != NULL) {
		return TRUE;
	}
	RECT rc = {0, 0, 0, 0};
	m_StatusBar.Create(m_hWnd, rc, NULL,
			WS_CHILD | CCS_BOTTOM | SBARS_SIZEGRIP | WS_VISIBLE,
			0,
			(HMENU)NULL, NULL);
	if (m_StatusBar.m_hWnd == NULL) {
		return FALSE;
	}
	SetStatusBarParts();
	return TRUE;
}

// ---------------------------------------------------------------------------
// DestroyStatusBar
// ---------------------------------------------------------------------------
BOOL CFilterWnd::SetStatusBarParts()
{
	static int s_anRGBColorWidth[RGB_DISPLAY_TYPE_MAX]       = {70, 60, 80, 90, 80};
	static int s_anRGBColorWidth_Vista[RGB_DISPLAY_TYPE_MAX] = {90, 80, 100, 110, 90};
	static int s_nPositionWidth       = 70;
	static int s_nPositionWidth_Vista = 90;
	static int s_nZoomWidth           = 30;
	static int s_nZoomWidth_Vista     = 40;
	static int s_nColorNameWidth = -1;
	int nCount = 0;
	int anPartsSize[4] = {0};
	int anParts[4] = {0};

	anPartsSize[nCount++] =
			IsWinVista() ?
			s_anRGBColorWidth_Vista[GetAppProfile()->GetWindowColorDisplayFormat()] :
				s_anRGBColorWidth[GetAppProfile()->GetWindowColorDisplayFormat()];
	if (! m_bSimpleStatusbar) {
		anPartsSize[nCount++] = IsWinVista() ? s_nPositionWidth_Vista : s_nPositionWidth;
		anPartsSize[nCount++] = IsWinVista() ? s_nZoomWidth_Vista : s_nZoomWidth;
	}
	anPartsSize[nCount++] = s_nColorNameWidth;

	int nTotalSize = 0;
	int i;
	for (i = 0; i < nCount; i++) {
		if (anPartsSize[i] == -1) {
			anParts[i] = -1;
		}
		else {
			nTotalSize = nTotalSize + anPartsSize[i];
			anParts[i] = nTotalSize;
		}
	}
	m_StatusBar.SetParts(nCount, anParts);

	RefreshStatusBarInfo();

	return TRUE;
}

// ---------------------------------------------------------------------------
// DestroyStatusBar
// ---------------------------------------------------------------------------
void CFilterWnd::DestroyStatusBar()
{
	if (m_StatusBar.m_hWnd == NULL) {
		return;
	}
	m_StatusBar.DestroyWindow();
}

// ---------------------------------------------------------------------------
// InitControls
// ---------------------------------------------------------------------------
BOOL CFilterWnd::InitControls()
{
	if (m_bShowStatusBar) {
		if (! CreateStatusBar()) {
			return FALSE;
		}
	}

	return TRUE;
}

// ---------------------------------------------------------------------------
// OnCreate
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_bCaptureCursor = FALSE;
	m_bSimpleStatusbar = GetAppProfile()->IsWindowSimpleStatusbarInfo();
	m_bInvalidated = FALSE;
	LoadWindowSetting();
	m_bStatusChanged = FALSE;
	m_bLeftTopOrigin = GetAppProfile()->IsWindowLeftTopOrigin();

	if (! InitControls()) {
		return -1;
	}

	CaptureScreen();
	CFilter* pFilter = m_FilterStatus.GetSelectedEnableFilter();
	if (pFilter == NULL) {
		pFilter = GetAppFilterManager()->GetNormalFilter();
	}
	ChangeFilter(pFilter);
	UpdateCaption();
	m_nTimerID = SetTimer(FILTERWND_TIMER_ID, FILTERWND_TIMER_ELAPSE, NULL);
	m_bClosing = FALSE;

	return 0;
}

// ---------------------------------------------------------------------------
// OnClose
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (! m_bClosing) {
		m_bClosing = TRUE;
		PostMessage(WM_CLOSE, 0, 0);
		return 0;
	}
	DestroyWindow();
	return 0;
}

// ---------------------------------------------------------------------------
// OnDestroy
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (m_nTimerID != 0) {
		KillTimer(FILTERWND_TIMER_ID);
		m_nTimerID = 0;
	}
	if (! m_bInvalidated) {
		CWindowSetting setting = SaveWindowSetting();
		GetAppProfile()->SetWindowSetting(setting);
		setting.GetRect(&ms_rcPrevWindowRect);
		ms_bUsePrevWindowRect = FALSE;
		EraseFromWndList(this);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnPaint
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (IsIconic()) {
		return 0;
	}
	if (m_CaptureImage.GetDC() == NULL) {
		return 0;
	}
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(&ps);

	if (m_nZoom >= 1) {
		if (IsWin2K()) {
			SetStretchBltMode(hdc, BLACKONWHITE);
		}
		StretchBlt(hdc, 0, 0, m_CaptureImage.GetWidth() * m_nZoom, m_CaptureImage.GetHeight() * m_nZoom,
				m_CaptureImage.GetDC(), 0, 0, m_CaptureImage.GetWidth(), m_CaptureImage.GetHeight(),
				SRCCOPY);

		static COLORREF clrLightGray = RGB(0x9f, 0x9f, 0x9f);
		static COLORREF clrDarkGray = RGB(0x60, 0x60, 0x60);
		if (m_bCenterMark && ! m_bLeftTopOrigin) {
			int nCenterX = (m_CaptureImage.GetWidth() - 1) / 2;
			int nCenterY = (m_CaptureImage.GetHeight() - 1) / 2;
			HPEN hPenLightGray = CreatePen(PS_SOLID, 1, clrLightGray);
			HPEN hPenOrig = (HPEN)SelectObject(hdc, hPenLightGray);
			HBRUSH hbrNull = (HBRUSH)GetStockObject(NULL_BRUSH);
			HBRUSH hbrOrig = (HBRUSH)SelectObject(hdc, hbrNull);
			Rectangle(hdc, nCenterX * m_nZoom -1, nCenterY * m_nZoom -1,
					(nCenterX + 1) * m_nZoom + 1, (nCenterY + 1) * m_nZoom + 1);
			SetPixel(hdc, nCenterX * m_nZoom -1, nCenterY * m_nZoom -1, clrDarkGray);
			SetPixel(hdc, (nCenterX + 1) * m_nZoom, nCenterY * m_nZoom -1, clrDarkGray);
			SetPixel(hdc, nCenterX * m_nZoom - 1, (nCenterY + 1) * m_nZoom, clrDarkGray);
			SetPixel(hdc, (nCenterX + 1) * m_nZoom, (nCenterY + 1) * m_nZoom, clrDarkGray);
			SelectObject(hdc, hbrOrig);
			SelectObject(hdc, hPenOrig);
			DeleteObject(hPenLightGray);
		}
	}
	else {
		int nZoomI = -m_nZoom + 2;
		if (IsWin2K()) {
			SetStretchBltMode(hdc, HALFTONE);
		}
		StretchBlt(hdc, 0, 0, m_CaptureImage.GetWidth() / nZoomI, m_CaptureImage.GetHeight() / nZoomI,
				m_CaptureImage.GetDC(), 0, 0, m_CaptureImage.GetWidth(), m_CaptureImage.GetHeight(),
				SRCCOPY);
	}

	EndPaint(&ps);
	return 0;
}

// ---------------------------------------------------------------------------
// OnKeyDown
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnKeyDown(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	if (wParam == VK_APPS) {
		ShowPopupMenu(FALSE);
		return 0;
	}
	if ((wParam >= '0' && wParam <= '9') ||
		(wParam >= VK_NUMPAD0 && wParam <= VK_NUMPAD9) ||
		wParam == VK_PRIOR || wParam == VK_NEXT) {
		BOOL bFilterChange = FALSE;
		int nIndex = m_FilterStatus.GetSelectedEnableFilterIndex();
		if (wParam == VK_PRIOR &&
			nIndex > 0) {
			--nIndex;
			bFilterChange = TRUE;
		}
		else if (wParam == VK_NEXT &&
			nIndex < GetAppFilterManager()->GetEnableFilterCount() - 1) {
			++nIndex;
			bFilterChange = TRUE;
		}
		if ((wParam >= '0' && wParam <= '9') ||
			(wParam >= VK_NUMPAD0 && wParam <= VK_NUMPAD9)) {
			int nNewIndex = 0;
			if (wParam == '0' || wParam == VK_NUMPAD0) {
				nNewIndex = 9;
			}
			else if (wParam >= '1' && wParam <= '9') {
				nNewIndex = wParam - '1';
			}
			else if (wParam >= VK_NUMPAD1 && wParam <= VK_NUMPAD9) {
				nNewIndex = wParam - VK_NUMPAD1;
			}
			if (nNewIndex < GetAppFilterManager()->GetEnableFilterCount()) {
				nIndex = nNewIndex;
				bFilterChange = TRUE;
			}
		}
		if (bFilterChange) {
			m_FilterStatus.SetSelectedEnableFilterIndex(nIndex);
			ChangeFilter(m_FilterStatus.GetSelectedEnableFilter());
			UpdateCaption();
		}
		return 0;
	}
	switch (wParam) {
	case 'C':
		if (GetKeyState(VK_CONTROL) < 0) {
			if (GetKeyState(VK_SHIFT) < 0) {
				CopyCenterColor(TRUE);
			}
			else {
				CopyCenterColor(FALSE);
			}
		}
		return 0;
	case 'F':
		if (GetKeyState(VK_CONTROL) < 0) {
			PostMessageToAllWindow(WM_EX_FIX_MOUSE_POSITION, (! m_bFixedMousePosition) ? 1 : 0);
		}
		else {
			SetFixMousePosition(! m_bFixedMousePosition);
		}
		return 0;
	case 'M':
		if (GetKeyState(VK_CONTROL) < 0) {
			PostMessageToAllWindow(WM_EX_CENTER_MARK, (! m_bCenterMark) ? 1 : 0);
		}
		else {
			SetCenterMark(! m_bCenterMark);
		}
		return 0;
	case 'N': {
		if (GetKeyState(VK_CONTROL) < 0) {
			NewWindow();
		}
		return 0;
	}
	case VK_ADD:
	case VK_SUBTRACT:
	case 'Z':
	case 'X': {
		int nZoom = m_nZoom;
		if (wParam == VK_ADD || wParam == 'Z') {
			if (nZoom < MAX_ZOOM) {
				++nZoom;
			}
		}
		else if (wParam == VK_SUBTRACT || wParam == 'X') {
			if (nZoom > MIN_ZOOM) {
				--nZoom;
			}
		}
		if (GetKeyState(VK_CONTROL) < 0) {
			PostMessageToAllWindow(WM_EX_ZOOM, (WPARAM)nZoom);
		}
		else {
			PostMessage(WM_EX_ZOOM, (WPARAM)nZoom);
		}
		return 0;
	}
	case VK_UP:
	case VK_DOWN:
	case VK_LEFT:
	case VK_RIGHT: {
		int dx = 0;
		int dy = 0;
		switch (wParam) {
		case VK_UP:    dy = -1; break;
		case VK_DOWN:  dy = 1;  break;
		case VK_LEFT:  dx = -1; break;
		case VK_RIGHT: dx = 1;  break;
		}
		if (GetKeyState(VK_CONTROL) < 0) {
			dx *= 8;
			dy *= 8;
		}
		MoveCursor(dx, dy);
		return 0;
	}
	default:
		break;
	}
	bHandled = FALSE;
	return 0;
}

// ---------------------------------------------------------------------------
// OnSysKeyDown
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnSysKeyDown(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	if (wParam == VK_F10 && GetKeyState(VK_SHIFT) < 0) {
		ShowPopupMenu(FALSE);
		return 0;
	}
	bHandled = FALSE;
	return 0;
}

// ---------------------------------------------------------------------------
// ShowPopupMenu
// ---------------------------------------------------------------------------
void CFilterWnd::ShowPopupMenu(BOOL bMouseEvent)
{
	m_menuPopup.DestroyMenu();
	m_menuPopup = LoadMenu(GetResourceHandle(), MAKEINTRESOURCE(IDM_FILTER_WINDOW_CONTEXT_MENU));
	CMenuHandle menu = m_menuPopup.GetSubMenu(0);
	CMenuHandle menuFilter = menu.GetSubMenu(MENUPOS_WINDOW_CONTEXT_FILTER);
	CMenuHandle menuView = menu.GetSubMenu(MENUPOS_WINDOW_CONTEXT_VIEW);
	menuFilter.DeleteMenu(0, MF_BYPOSITION);
	int nEnableFilterNum = GetAppFilterManager()->GetEnableFilterCount();
	int i;
	for (i = 0; i < nEnableFilterNum; i++) {
		CFilter* pFilter = GetAppFilterManager()->GetEnableFilter(i);
		TCHAR szShortCut[TEXT_BUFFER_SIZE] = _T("");
		if (i < 9) {
			wsprintf(szShortCut, _T("\t%d"), i + 1);
		}
		else if (i == 9) {
			lstrcpy(szShortCut, _T("\t0"));
		}
		TCHAR szName[TEXT_BUFFER_SIZE];
		wsprintf(szName, _T("%s%s"), pFilter->GetName(), szShortCut);
		MENUITEMINFO MenuItemInfo;
		ZeroMemory(&MenuItemInfo, sizeof(MenuItemInfo));
		MenuItemInfo.cbSize = sizeof(MenuItemInfo);
		MenuItemInfo.fMask = MIIM_FTYPE | MIIM_ID | MIIM_STATE | MIIM_STRING;
		MenuItemInfo.fType = MFT_STRING;
		MenuItemInfo.fState = (i == m_FilterStatus.GetSelectedEnableFilterIndex()) ? MFS_CHECKED : 0;
		MenuItemInfo.wID = ID_WINDOW_FILTER_BEGIN + i;
		MenuItemInfo.dwTypeData = szName;
		MenuItemInfo.cch = 0;
		menuFilter.InsertMenuItem(i, TRUE, &MenuItemInfo);
	}
	int nZoomIndex = -1;
	static int anZooms[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 10, 12, 16, 20};
	for (i = 0; i < ARRAYSIZE(anZooms); i++) {
		if (m_nZoom == anZooms[i]) {
			nZoomIndex = i;
			break;
		}
	}
	if (nZoomIndex != -1) {
		MENUITEMINFO MenuItemInfo;
		ZeroMemory(&MenuItemInfo, sizeof(MenuItemInfo));
		MenuItemInfo.cbSize = sizeof(MenuItemInfo);
		MenuItemInfo.fMask = MIIM_STATE;
		MenuItemInfo.fState = MFS_CHECKED;
		CMenuHandle menuZoom = menu.GetSubMenu(1);
		menuZoom.SetMenuItemInfo(nZoomIndex, TRUE, &MenuItemInfo);
	}
	if (m_bCenterMark) {
		MENUITEMINFO MenuItemInfo;
		ZeroMemory(&MenuItemInfo, sizeof(MenuItemInfo));
		MenuItemInfo.cbSize = sizeof(MenuItemInfo);
		MenuItemInfo.fMask = MIIM_STATE;
		MenuItemInfo.fState = MFS_CHECKED;
		menu.SetMenuItemInfo(MENUPOS_WINDOW_CONTEXT_CENTER_MARK, TRUE, &MenuItemInfo);
	}
	if (m_bFixedMousePosition) {
		MENUITEMINFO MenuItemInfo;
		ZeroMemory(&MenuItemInfo, sizeof(MenuItemInfo));
		MenuItemInfo.cbSize = sizeof(MenuItemInfo);
		MenuItemInfo.fMask = MIIM_STATE;
		MenuItemInfo.fState = MFS_CHECKED;
		menu.SetMenuItemInfo(MENUPOS_WINDOW_CONTEXT_FIX_MOUSE_POSITION, TRUE, &MenuItemInfo);
	}
	if (m_bTopmost) {
		MENUITEMINFO MenuItemInfo;
		ZeroMemory(&MenuItemInfo, sizeof(MenuItemInfo));
		MenuItemInfo.cbSize = sizeof(MenuItemInfo);
		MenuItemInfo.fMask = MIIM_STATE;
		MenuItemInfo.fState = MFS_CHECKED;
		menuView.SetMenuItemInfo(MENUPOS_WINDOW_CONTEXT_VIEW_TOPMOST, TRUE, &MenuItemInfo);
	}
	if (m_bShowStatusBar) {
		MENUITEMINFO MenuItemInfo;
		ZeroMemory(&MenuItemInfo, sizeof(MenuItemInfo));
		MenuItemInfo.cbSize = sizeof(MenuItemInfo);
		MenuItemInfo.fMask = MIIM_STATE;
		MenuItemInfo.fState = MFS_CHECKED;
		menuView.SetMenuItemInfo(MENUPOS_WINDOW_CONTEXT_VIEW_SHOW_STATUSBAR, TRUE, &MenuItemInfo);
	}
	POINT pos;
	if (bMouseEvent) {
		if (! GetCursorPos(&pos)) {
			pos.x = 0;
			pos.y = 0;
			ClientToScreen(&pos);
		}
	}
	else {
		pos.x = 0;
		pos.y = 0;
		ClientToScreen(&pos);
	}
	SetForegroundWindow(m_hWnd);
	menu.TrackPopupMenu(TPM_LEFTALIGN | TPM_TOPALIGN, pos.x, pos.y, m_hWnd);
}

// ---------------------------------------------------------------------------
// OnSetFocus
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnImeSetContext
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnImeSetContext(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnSize
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnSize(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	if (m_bShowStatusBar) {
		m_StatusBar.SendMessage(WM_SIZE, wParam, lParam);
	}
	m_bStatusChanged = TRUE;
	return 0;
}

// ---------------------------------------------------------------------------
// OnLButtonDown
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnLButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetCapture();
	m_bCaptureCursor = TRUE;
	return 0;
}

// ---------------------------------------------------------------------------
// OnLButtonUp
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnLButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (! m_bCaptureCursor) {
		return 0;
	}
	ReleaseCapture();
	m_bCaptureCursor = FALSE;
	RECT rc;
	GetWindowRect(&rc);
	POINT pt;
	GetCursorPos(&pt);
	if (! PtInRect(&rc, pt)) {
		if (GetAsyncKeyState(VK_CONTROL) < 0) {
			PostMessageToAllWindow(WM_EX_FIX_MOUSE_POSITION, 1);
		}
		else {
			SetFixMousePosition(TRUE);
		}
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnRButtonUp
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnRButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	ShowPopupMenu(TRUE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnLButtonDblClk
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnLButtonDblClk(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (GetAsyncKeyState(VK_CONTROL) < 0) {
		PostMessageToAllWindow(WM_EX_SHOW_STATUSBAR, (! m_bShowStatusBar) ? 1 : 0);
	}
	else {
		ShowStatusBar(! m_bShowStatusBar);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnMouseMove
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (! m_bCaptureCursor) {
		return 0;
	}
	if (m_bFixedMousePosition) {
		RECT rc;
		GetWindowRect(&rc);
		POINT pt;
		GetCursorPos(&pt);
		if (! PtInRect(&rc, pt)) {
			SetFixMousePosition(FALSE);
		}
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnTimer
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnTimer(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (wParam != FILTERWND_TIMER_ID) {
		return 0;
	}
	BOOL bCapture = FALSE;
	if (m_bStatusChanged) {
		bCapture = TRUE;
		m_bStatusChanged = FALSE;
	}
	DWORD dwDiff = GetTickCount() - m_dwPrevCaptureTime;
	if (dwDiff >= GetAppProfile()->GetWindowUpdateInterval()) {
		bCapture = TRUE;
	}
	BOOL bRedraw = FALSE;
	if (bCapture) {
		CaptureScreen();
		bRedraw = TRUE;
	}
	CFilter* pLowFilter = NULL;
	CFilter* pFilter = m_FilterStatus.GetSelectedEnableFilter();
	if (! pFilter->IsRefFilter()) {
		pLowFilter = pFilter;
	}
	else {
		if (m_FilterStatus.IsNextFilterItemReady()) {
			pLowFilter = m_FilterStatus.NextFilterItem();
			if (pLowFilter == NULL) {
//XATLTRACE(_T("filter item empty"));
			}
			else {
				bRedraw = TRUE;
			}
		}
		else {
			pLowFilter = m_FilterStatus.LastFilterItem();
		}
	}
	if (bRedraw) {
		if (pLowFilter != NULL) {
			ApplyFilter(pLowFilter);
			RECT rc;
			GetModifiedClientRect(&rc);
			RedrawWindow(&rc, NULL, RDW_INTERNALPAINT | RDW_INVALIDATE);
		}
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnZoom
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnZoom(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetZoom((int)wParam);
	return 0;
}

// ---------------------------------------------------------------------------
// OnCenterMark
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCenterMark(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetCenterMark((wParam != 0) ? TRUE : FALSE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnTopmost
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnTopmost(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetTopmost((wParam != 0) ? TRUE : FALSE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnFixMousePosition
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnFixMousePosition(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetFixMousePosition((wParam != 0) ? TRUE : FALSE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnShowStatusBar
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnShowStatusBar(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	ShowStatusBar((wParam != 0) ? TRUE : FALSE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnLeftTopOrigin
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnLeftTopOrigin(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetLeftTopOrigin((wParam != 0) ? TRUE : FALSE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnStatusBarPartsChange
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnStatusBarPartsChange(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetStatusBarParts();
	return 0;
}

// ---------------------------------------------------------------------------
// OnWindowTop
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnWindowTop(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	SetWindowPos(HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnFinalMessage
// ---------------------------------------------------------------------------
void CFilterWnd::OnFinalMessage(HWND /*hWnd*/)
{
	delete this;
}

// ---------------------------------------------------------------------------
// capture screen
// ---------------------------------------------------------------------------
BOOL CFilterWnd::CaptureScreen()
{
	if (IsIconic()) {
		return 0;
	}
	POINT pos;
	if (! m_bFixedMousePosition) {
		GetCursorPos(&pos);
	}
	else {
		pos = m_FixedCursorPos;
	}
	m_CursorPos = pos;
	RECT rcClient;
	GetModifiedClientRect(&rcClient);

	int nClientWidth = rcClient.right;
	int nClientHeight = rcClient.bottom;
	int nWidth;
	int nHeight;
	if (m_nZoom >= 1) {
		nWidth = (nClientWidth + (m_nZoom - 1)) / m_nZoom;
		nHeight = (nClientHeight + (m_nZoom - 1)) / m_nZoom;
	}
	else {
		int nZoomI = -m_nZoom + 2;
		nWidth = nClientWidth * nZoomI;
		nHeight = nClientHeight * nZoomI;
	}
	int nSrcPosX;
	int nSrcPosY;
	if (m_bLeftTopOrigin) {
		nSrcPosX = pos.x;
		nSrcPosY = pos.y;
	}
	else {
		nSrcPosX = pos.x - ((nWidth - 1) / 2);
		nSrcPosY = pos.y - ((nHeight - 1) / 2);
	}

	if (! m_CaptureImage.Capture(nSrcPosX, nSrcPosY, nWidth, nHeight)) {
		return 0;
	}
	m_dwPrevCaptureTime = GetTickCount();

	return TRUE;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::ChangeFilter(CFilter* pFilter)
{
	BOOL bRedraw = FALSE;
	if (pFilter->IsDefFilter()) {
		ApplyFilter(pFilter);
		bRedraw = TRUE;
	}
	else {
		m_FilterStatus.ResetFilterItem();
		if (m_FilterStatus.IsNextFilterItemReady()) {
			CFilter* pTmpFilter = m_FilterStatus.NextFilterItem();
			if (pTmpFilter != NULL) {
				ApplyFilter(pTmpFilter);
				bRedraw = TRUE;
			}
			else {
ATLTRACE2(atlTraceGeneral, 0, _T("filter item empty"));
			}
		}
	}
	if (bRedraw) {
		RECT rc;
		GetModifiedClientRect(&rc);
		RedrawWindow(&rc, NULL, RDW_INTERNALPAINT | RDW_INVALIDATE);
	}
}

// ---------------------------------------------------------------------------
// apply filter to image
// ---------------------------------------------------------------------------
void CFilterWnd::ApplyFilter(CFilter* pFilter)
{
	m_CaptureImage.ApplyFilter(pFilter);
	UpdateCenterColor(FALSE);
	UpdateStatusBar();
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::UpdateCenterColor(BOOL bNormalFilter)
{
	if (m_CaptureImage.GetDC() == NULL) {
		return;
	}
	if (m_bLeftTopOrigin) {
		m_clrOriginal = m_CaptureImage.GetLeftTopColor(TRUE);
		m_clrFiltered = m_CaptureImage.GetLeftTopColor(FALSE);
	}
	else {
		m_clrOriginal = m_CaptureImage.GetCenterColor(TRUE);
		m_clrFiltered = m_CaptureImage.GetCenterColor(FALSE);
	}
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
BOOL CFilterWnd::GetModifiedClientRect(LPRECT pRect)
{
	RECT rc;
	GetClientRect(&rc);
	if (m_bShowStatusBar) {
		RECT rcStatusbar;
		m_StatusBar.GetWindowRect(&rcStatusbar);
		POINT posStatusbar;
		posStatusbar.x = rcStatusbar.right;
		posStatusbar.y = rcStatusbar.top;
		ScreenToClient(&posStatusbar);
		SetRect(pRect, 0, 0, rc.right, (posStatusbar.y >= 0) ? posStatusbar.y : 1);
	}
	else {
		*pRect = rc;
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::UpdateCaption()
{
	TCHAR szCaption[TEXT_BUFFER_SIZE];
	CFilter* pFilter = m_FilterStatus.GetSelectedEnableFilter();
	if (pFilter != NULL) {
		_sntprintf(szCaption, sizeof(szCaption) - 1, _T("%s - %s"),
				pFilter->GetName(), APP_WINDOW_TITLE);
	}
	else {
		lstrcpy(szCaption, APP_WINDOW_TITLE);
	}
	SetWindowText(szCaption);
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::UpdateStatusBar()
{
	if (IsIconic()) {
		return;
	}
	if (! m_bShowStatusBar) {
		return;
	}
	TCHAR szText[TEXT_BUFFER_SIZE];
	COLORREF clr;
	if (GetAppProfile()->IsWindowShowOriginalColorInfo()) {
		clr = (GetAsyncKeyState(VK_SHIFT) < 0) ? m_clrFiltered : m_clrOriginal;
	}
	else {
		clr = (GetAsyncKeyState(VK_SHIFT) < 0) ? m_clrOriginal : m_clrFiltered;
	}
	if (clr != m_clrPrevStatusBarColor) {
		xstr_t strColor;
		switch (GetAppProfile()->GetWindowColorDisplayFormat()) {
		case RGB_DISPLAY_TYPE_RGB_DEC:
			strColor = CColorName::GetRGBValue(clr, FALSE);
			break;
		case RGB_DISPLAY_TYPE_RGB_HEX:
			strColor = CColorName::GetRGBValue(clr, TRUE);
			break;
		case RGB_DISPLAY_TYPE_RGB_HSV:
			strColor = CColorName::GetHSVValue(CHSV(clr), FALSE);
			break;
		case RGB_DISPLAY_TYPE_RGB_HSV_CUSTOM:
			strColor = CColorName::GetHSVValue(CHSV(clr), TRUE);
			break;
		case RGB_DISPLAY_TYPE_RGB_HSV_SIMPLE:
			strColor = CColorName::GetHSVSimpleValue(clr);
			break;
		default:
			strColor = CColorName::GetRGBValue(clr, FALSE);
			break;
		}
		if (m_bCenterMark) {
			m_StatusBar.SetText(0, (LPCTSTR)strColor);
		}
		else {
			m_StatusBar.SetText(0, _T(""));
		}
	}
	if (! m_bSimpleStatusbar) {
		if (m_CursorPos.x != m_PrevStatusBarPos.x ||
			m_CursorPos.y != m_PrevStatusBarPos.y) {
			_sntprintf(szText, sizeof(szText) - 1, _T("%d, %d"),
					m_CursorPos.x, m_CursorPos.y);
			if (m_bCenterMark) {
				m_StatusBar.SetText(1, szText);
			}
			else {
				m_StatusBar.SetText(1, _T(""));
			}
		}
		if (m_nZoom != m_nPrevStatusBarZoom) {
			if (m_nZoom >= 1) {
				_sntprintf(szText, sizeof(szText) - 1, _T("x%d"), m_nZoom);
			}
			else {
				_sntprintf(szText, sizeof(szText) - 1, _T("x1/%d"), -m_nZoom+2);
			}
			m_StatusBar.SetText(2, szText);
		}
	}
	if (clr != m_clrPrevStatusBarColor) {
		BOOL bNoColorRange = GetAppProfile()->GetWindowColorDisplayFormat() == RGB_DISPLAY_TYPE_RGB_HSV_SIMPLE;
		CHSV hsv(clr);
		if (m_bCenterMark) {
			m_StatusBar.SetText(m_bSimpleStatusbar ? 1 : 3,
					CColorName::GetColorName(hsv, bNoColorRange));
		}
		else {
			m_StatusBar.SetText(m_bSimpleStatusbar ? 1 : 3, _T(""));
		}
	}
	m_clrPrevStatusBarColor = clr;
	m_nPrevStatusBarZoom = m_nZoom;
	m_PrevStatusBarPos = m_CursorPos;
}

// ---------------------------------------------------------------------------
// OnCmdZoom
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdZoom(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/)
{
	int nZoom = wID - ID_WINDOW_ZOOM_BEGIN;
	if (GetAsyncKeyState(VK_CONTROL) < 0) {
		PostMessageToAllWindow(WM_EX_ZOOM, (WPARAM)nZoom);
	}
	else {
		SetZoom(nZoom);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnCmdFilter
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdFilter(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/)
{
	int nFilterIndex = wID - ID_WINDOW_FILTER_BEGIN;
	SetFilter(nFilterIndex);
	return 0;
}

// ---------------------------------------------------------------------------
// OnCmdCenterMark
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdCenterMark(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (GetAsyncKeyState(VK_CONTROL) < 0) {
		PostMessageToAllWindow(WM_EX_CENTER_MARK, (! m_bCenterMark) ? 1 : 0);
	}
	else {
		SetCenterMark(! m_bCenterMark);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnCmdTopmost
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdTopmost(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (GetAsyncKeyState(VK_CONTROL) < 0) {
		PostMessageToAllWindow(WM_EX_TOPMOST, (! m_bTopmost) ? 1 : 0);
	}
	else {
		SetTopmost(! m_bTopmost);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnCmdFixMousePosition
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdFixMousePosition(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (GetAsyncKeyState(VK_CONTROL) < 0) {
		PostMessageToAllWindow(WM_EX_FIX_MOUSE_POSITION, (! m_bFixedMousePosition) ? 1 : 0);
	}
	else {
		SetFixMousePosition(! m_bFixedMousePosition);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnCmdShowStatusBar
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdShowStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (GetAsyncKeyState(VK_CONTROL) < 0) {
		PostMessageToAllWindow(WM_EX_SHOW_STATUSBAR, (! m_bShowStatusBar) ? 1 : 0);
	}
	else {
		ShowStatusBar(! m_bShowStatusBar);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnCmdLeftTopOrigin
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdLeftTopOrigin(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	if (GetAsyncKeyState(VK_CONTROL) < 0) {
		PostMessageToAllWindow(WM_EX_LEFTTOP_ORIGIN, (! m_bLeftTopOrigin) ? 1 : 0);
	}
	else {
		SetLeftTopOrigin(! m_bLeftTopOrigin);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// SetZoom
// ---------------------------------------------------------------------------
void CFilterWnd::SetZoom(int nZoom)
{
	if (nZoom < MIN_ZOOM || nZoom > 20) {
		return;
	}
	m_nZoom = nZoom;
	m_bStatusChanged = TRUE;
}

// ---------------------------------------------------------------------------
// SetFilter
// ---------------------------------------------------------------------------
void CFilterWnd::SetFilter(int nFilterIndex)
{
	CFilter* pFilter = GetAppFilterManager()->GetEnableFilter(nFilterIndex);
	if (pFilter != NULL) {
		if (pFilter != m_FilterStatus.GetSelectedEnableFilter()) {
			m_FilterStatus.SetSelectedEnableFilterIndex(nFilterIndex);
			ChangeFilter(pFilter);
			UpdateCaption();
		}
	}
}

// ---------------------------------------------------------------------------
// SetCenterMark
// ---------------------------------------------------------------------------
void CFilterWnd::SetCenterMark(BOOL bEnable)
{
	m_bCenterMark = bEnable;

	RefreshStatusBarInfo();
}

// ---------------------------------------------------------------------------
// SetTopmost
// ---------------------------------------------------------------------------
void CFilterWnd::SetTopmost(BOOL bEnable)
{
	m_bTopmost = bEnable;
	if (m_bTopmost) {
		SetWindowPos(HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	}
	else {
		SetWindowPos(HWND_NOTOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE);
	}
}

// ---------------------------------------------------------------------------
// SetFixMousePosition
// ---------------------------------------------------------------------------
void CFilterWnd::SetFixMousePosition(BOOL bEnable)
{
	m_bFixedMousePosition = bEnable;
	if (m_bFixedMousePosition) {
		GetCursorPos(&m_FixedCursorPos);
	}
	UpdateCaption();
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::ShowStatusBar(BOOL bShow)
{
	m_bShowStatusBar = bShow;
	if (m_bShowStatusBar) {
		if (! CreateStatusBar()) {
			m_bShowStatusBar = FALSE;
		}
	}
	else {
		DestroyStatusBar();
	}
	m_bStatusChanged = TRUE;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdNewWindow(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	NewWindow();
	return 0;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
LRESULT CFilterWnd::OnCmdClose(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	PostMessage(WM_CLOSE);
	return 0;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::SetLeftTopOrigin(BOOL bEnable)
{
	m_bLeftTopOrigin = bEnable;
	m_bStatusChanged = TRUE;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::NewWindow()
{
	CWindowSetting setting = SaveWindowSetting();
	GetAppProfile()->SetWindowSetting(setting);
	setting.GetRect(&ms_rcPrevWindowRect);
	ms_bUsePrevWindowRect = TRUE;
	CreateFilterWnd();
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::RefreshStatusBarInfo()
{
	m_clrPrevStatusBarColor = (COLORREF)0xffffffff;
	m_PrevStatusBarPos.x = -1;
	m_PrevStatusBarPos.y = -1;
	m_nPrevStatusBarZoom = -9999;
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::MoveCursor(int dx, int dy)
{
	POINT pos;
	GetCursorPos(&pos);
	pos.x = pos.x + dx;
	pos.y = pos.y + dy;
	SetCursorPos(pos.x, pos.y);
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CFilterWnd::LoadWindowSetting()
{
	xstr_t strID = m_InitialSetting.GetFilter();
	int nIndex = GetAppFilterManager()->GetEnableFilterIndexFromID((LPCTSTR)strID);
	m_FilterStatus.SetSelectedEnableFilterIndex(nIndex);
	m_bCenterMark = m_InitialSetting.IsCenterMark();
	m_bTopmost = m_InitialSetting.IsTopmost();
	m_nZoom = m_InitialSetting.GetZoom();
	m_InitialSetting.GetFixedMousePosition(m_bFixedMousePosition, m_FixedCursorPos);
	m_bShowStatusBar = m_InitialSetting.IsShowStatusBar();
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
CWindowSetting CFilterWnd::SaveWindowSetting()
{
	CWindowSetting setting;
	setting.SetFilter(m_FilterStatus.GetSelectedEnableFilter()->GetID());
	RECT rc;
	GetWindowRect(&rc);
	setting.SetRect(&rc);
	setting.SetCenterMark(m_bCenterMark);
	setting.SetTopmost(m_bTopmost);
	setting.SetZoom(m_nZoom);
	setting.SetFixedMousePosition(m_bFixedMousePosition, &m_FixedCursorPos);
	setting.SetShowStatusBar(m_bShowStatusBar);
	return setting;
}

// ---------------------------------------------------------------------------
// copy center color to clipboard
// ---------------------------------------------------------------------------
// color text format: %0?[1-9]?[dxX]?[rRgGbB]
//       %        special charactor
//       0        zero suppress
//       [1-9]    column
//       [dxX]    d:decimal, x:hex(lower case), X:hex(upper case)
//       [rRgGbB] R/G/B
// example: (color: RGB(0x7f,0x7f,0x7f))
//   "#%02XR%02XG%02XB"  => #7F7F7F
//   "%dR,%dG,%dB"       => 127,127,127
// ---------------------------------------------------------------------------
void CFilterWnd::CopyCenterColor(BOOL bOriginal)
{
	COLORREF clr;
	if (bOriginal) {
		clr = m_clrOriginal;
	}
	else {
		clr = m_clrFiltered;
	}
	xstr_t strFmt = GetAppProfile()->GetCopyColorFormat();
	xstr_t strText = _T("");
	LPCTSTR psz = (LPCTSTR)strFmt;
	while (*psz != _T('\0')) {
		BOOL bDone = FALSE;
		if (*psz == _T('%')) {
			LPCTSTR pszScan = psz + 1;
			if (*pszScan == '%') {
				strText += _T("%");
				psz += 2;
				continue;
			}
			BOOL bZeroPadding = FALSE;
			int nColumn = 0;
			BOOL bHex = FALSE;
			BOOL bLowerCase = TRUE;
			if (*pszScan == '0') {
				bZeroPadding = TRUE;
				++pszScan;
			}
			if (*pszScan >= '1' && *pszScan <= '9') {
				nColumn = (*pszScan - '1') + 1;
				++pszScan;
			}
			if (*pszScan == 'd' || *pszScan == 'x' || *pszScan == 'X') {
				if (*pszScan == 'x') {
					bHex = TRUE;
					bLowerCase = TRUE;
				}
				else if (*pszScan == 'X') {
					bHex = TRUE;
					bLowerCase = FALSE;
				}
				++pszScan;
			}
			int nValue;
			if (*pszScan == 'r' || *pszScan == 'R') {
				nValue = GetRValue(clr);
				++pszScan;
			}
			else if (*pszScan == 'g' || *pszScan == 'G') {
				nValue = GetGValue(clr);
				++pszScan;
			}
			else if (*pszScan == 'b' || *pszScan == 'B') {
				nValue = GetBValue(clr);
				++pszScan;
			}
			else {
				strText += _T("%");
				++psz;
				continue;
			}
			xstr_t strFmt = _T("%");
			if (bZeroPadding) {
				strFmt += _T("0");
			}
			if (nColumn > 0) {
				TCHAR szNum[2];
				szNum[0] = '0' + nColumn;
				szNum[1] = '\0';
				strFmt += szNum;
			}
			if (bHex) {
				if (bLowerCase) {
					strFmt += _T("x");
				}
				else {
					strFmt += _T("X");
				}
			}
			else {
				strFmt += _T("d");
			}
			TCHAR szBuffer[20];
			wsprintf(szBuffer, (LPCTSTR)strFmt, nValue);
			strText += szBuffer;
			psz = pszScan;
		}
		else {
			TCHAR szChar[2];
			szChar[0] = *psz;
			szChar[1] = _T('\0');
			strText += szChar;
			++psz;
		}
	}
	CopyClipboard((LPCTSTR)strText);
}

// ---------------------------------------------------------------------------
// SetInitialSetting
// ---------------------------------------------------------------------------
void CFilterWnd::SetInitialSetting(CWindowSetting* pSetting)
{
	if (pSetting == NULL) {
		m_bEnableInitialSetting = FALSE;
		m_InitialSetting = GetAppProfile()->GetWindowSetting();
	}
	else {
		m_bEnableInitialSetting = TRUE;
		m_InitialSetting = *pSetting;
	}
}
