// ===========================================================================
// FilterWnd.h
// filter operate panel
//
//                                                       by kanegon 2006
// ===========================================================================

#pragma once

#include <vector>
#include "Filter.h"
#include "FilterStatus.h"
#include "CaptureImage.h"


// ---------------------------------------------------------------------------

class CFilterWnd;
typedef std::vector<CFilterWnd*> FILTER_WND_PTR_ARRAY;


// ---------------------------------------------------------------------------

BOOL CreateFilterWnd(CWindowSetting* pSetting = NULL);


// ---------------------------------------------------------------------------
// filter operate panel class
// ---------------------------------------------------------------------------
class CFilterWnd : public CWindowImpl<CFilterWnd>
{
private:
	static FILTER_WND_PTR_ARRAY ms_FilterWndPtrList;
	static RECT ms_rcPrevWindowRect;
	static BOOL ms_bUsePrevWindowRect;

public:
	static void PostMessageToAllWindow(UINT uMsg, WPARAM wParam = 0, LPARAM lParam = 0);
	static int GetWndListCount();
	static void AddToWndList(CFilterWnd* pWnd);
	static void EraseFromWndList(CFilterWnd* pTargetWnd);
	static CFilterWnd* GetTopElementOfWndList();
	static void GetInitialWindowRect(LPRECT pRect, CWindowSetting* pSetting, BOOL bEnableInitialSetting);
	static void LoadAllWindowSetting();
	static void SaveAllWindowSetting();
	static void ClearWndList();
	static void CloseAllWindow();
	static void ActivateAllWindow();

private:
	CCaptureImage m_CaptureImage;
	CStatusBarCtrl m_StatusBar;
	CMenu m_menuPopup;
	UINT_PTR m_nTimerID;
	BOOL m_bClosing;
	CFilterStatus m_FilterStatus;
	BOOL m_bInvalidated;
	BOOL m_bEnableInitialSetting;
	CWindowSetting m_InitialSetting;
	LRESULT m_nLastHitTestResult;
	COLORREF m_clrPrevStatusBarColor;
	POINT m_PrevStatusBarPos;
	int m_nPrevStatusBarZoom;
	BOOL m_bSimpleStatusbar;
	BOOL m_bCaptureCursor;

	BOOL m_bStatusChanged;
	DWORD m_dwPrevCaptureTime;
	int m_nZoom;
	BOOL m_bCenterMark;
	BOOL m_bTopmost;
	BOOL m_bFixedMousePosition;
	POINT m_FixedCursorPos;
	POINT m_CursorPos;
	COLORREF m_clrOriginal;
	COLORREF m_clrFiltered;
	BOOL m_bShowStatusBar;
	BOOL m_bLeftTopOrigin;

	BOOL CreateStatusBar();
	void DestroyStatusBar();
	BOOL InitControls();
	void ShowPopupMenu(BOOL bMouseEvent);
	BOOL CaptureScreen();
	void ChangeFilter(CFilter* pFilter);
	void ApplyFilter(CFilter* pFilter);
	BOOL GetModifiedClientRect(LPRECT pRect);
	void UpdateCaption();
	void UpdateStatusBar();
	void UpdateCenterColor(BOOL bNormalFilter);
	void MoveCursor(int dx, int dy);
	void RefreshStatusBarInfo();
	CWindowSetting SaveWindowSetting();
	void LoadWindowSetting();
	void CopyCenterColor(BOOL bOriginal);
	void SetInitialSetting(CWindowSetting* pSetting);
	void SetZoom(int nZoom);
	void SetFilter(int nFilterIndex);
	void SetCenterMark(BOOL bEnable);
	void SetTopmost(BOOL bEnable);
	void SetFixMousePosition(BOOL bEnable);
	void ShowStatusBar(BOOL bShow);
	void SetLeftTopOrigin(BOOL bEnable);
	void NewWindow();
	BOOL SetStatusBarParts();

public:
	CFilterWnd();

	BOOL CreateHelper(CWindowSetting* pSetting);

public:
	static ATL::CWndClassInfo& GetWndClassInfo() {
		static ATL::CWndClassInfo wc = {
				{sizeof(WNDCLASSEX), CS_DBLCLKS, StartWindowProc, 0, 0,
				NULL, LoadIcon(_Module.get_m_hInstResource(), MAKEINTRESOURCE(IDR_MAINFRAME)), NULL,
				(HBRUSH)NULL, NULL, NULL, NULL},
				NULL, NULL, IDC_ARROW, TRUE, 0, _T("") 
			};
		return wc;
	}

	BEGIN_MSG_MAP(CFilterWnd)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		MESSAGE_HANDLER(WM_CLOSE, OnClose)
		MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
		MESSAGE_HANDLER(WM_PAINT, OnPaint)
		MESSAGE_HANDLER(WM_KEYDOWN, OnKeyDown)
		MESSAGE_HANDLER(WM_SYSKEYDOWN, OnKeyDown)
		MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
		MESSAGE_HANDLER(WM_IME_SETCONTEXT, OnImeSetContext)
		MESSAGE_HANDLER(WM_SIZE, OnSize)
		MESSAGE_HANDLER(WM_LBUTTONDOWN, OnLButtonDown)
		MESSAGE_HANDLER(WM_LBUTTONUP, OnLButtonUp)
		MESSAGE_HANDLER(WM_RBUTTONUP, OnRButtonUp)
		MESSAGE_HANDLER(WM_LBUTTONDBLCLK, OnLButtonDblClk)
		MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
		MESSAGE_HANDLER(WM_TIMER, OnTimer)
		MESSAGE_HANDLER(WM_EX_ZOOM, OnZoom)
		MESSAGE_HANDLER(WM_EX_CENTER_MARK, OnCenterMark)
		MESSAGE_HANDLER(WM_EX_TOPMOST, OnTopmost)
		MESSAGE_HANDLER(WM_EX_FIX_MOUSE_POSITION, OnFixMousePosition)
		MESSAGE_HANDLER(WM_EX_SHOW_STATUSBAR, OnShowStatusBar)
		MESSAGE_HANDLER(WM_EX_LEFTTOP_ORIGIN, OnLeftTopOrigin)
		MESSAGE_HANDLER(WM_EX_STATUSBAR_PARTS_CHANGE, OnStatusBarPartsChange)
		MESSAGE_HANDLER(WM_EX_WINDOW_TOP, OnWindowTop)
		COMMAND_RANGE_HANDLER_EX(ID_WINDOW_ZOOM_BEGIN, ID_WINDOW_ZOOM_END, OnCmdZoom)
		COMMAND_RANGE_HANDLER_EX(ID_WINDOW_FILTER_BEGIN, ID_WINDOW_FILTER_END, OnCmdFilter)
		COMMAND_ID_HANDLER(ID_WINDOW_CENTER_MARK, OnCmdCenterMark)
		COMMAND_ID_HANDLER(ID_WINDOW_TOPMOST, OnCmdTopmost)
		COMMAND_ID_HANDLER(ID_WINDOW_FIX_MOUSE_POSITION, OnCmdFixMousePosition)
		COMMAND_ID_HANDLER(ID_WINDOW_SHOW_STATUSBAR, OnCmdShowStatusBar)
		COMMAND_ID_HANDLER(ID_WINDOW_NEW_WINDOW, OnCmdNewWindow)
		COMMAND_ID_HANDLER(ID_WINDOW_CLOSE, OnCmdClose)
	END_MSG_MAP()

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnKeyDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnSysKeyDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnImeSetContext(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnLButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnLButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnRButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnLButtonDblClk(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnTimer(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnZoom(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnCenterMark(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnTopmost(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnFixMousePosition(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnShowStatusBar(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnLeftTopOrigin(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnStatusBarPartsChange(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnWindowTop(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/);
	LRESULT OnCmdZoom(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/);
	LRESULT OnCmdFilter(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/);
	LRESULT OnCmdCenterMark(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnCmdTopmost(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnCmdFixMousePosition(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnCmdShowStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnCmdNewWindow(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnCmdClose(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);
	LRESULT OnCmdLeftTopOrigin(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/);

	virtual void OnFinalMessage(HWND /*hWnd*/);
};
