// ===========================================================================
// HotKeyManager.h
// hotkey manager
//
//                                                       by kanegon 2006
// ===========================================================================

#pragma once

#include "Filter.h"

// ---------------------------------------------------------------------------
// quickkey element
// ---------------------------------------------------------------------------
class CQuickKey {
private:
	CFilter* m_pFilter;
	DWORD m_dwHotKey;
	BOOL m_bRegistered;

public:
	CQuickKey() {
		m_pFilter = NULL;
		m_dwHotKey = 0;
		m_bRegistered = FALSE;
	}
	CQuickKey(CFilter* pFilter, DWORD dwHotKey) {
		m_pFilter = pFilter;
		m_dwHotKey = dwHotKey;
		m_bRegistered = FALSE;
	}
	CQuickKey(const CQuickKey& key) {
		m_pFilter = key.m_pFilter;
		m_dwHotKey = key.m_dwHotKey;
		m_bRegistered = key.m_bRegistered;
	}
	virtual ~CQuickKey() {
	}

	void SetFilter(CFilter* pFilter) {
		m_pFilter = pFilter;
	}
	CFilter* GetFilter() {
		return m_pFilter;
	}
	void SetHotKey(DWORD dwHotKey) {
		m_dwHotKey = dwHotKey;
	}
	DWORD GetHotKey() {
		return m_dwHotKey;
	}

	void SetRegistered(BOOL bRegistered) {
		m_bRegistered = bRegistered;
	}
	BOOL IsRegistered() {
		return m_bRegistered;
	}
};

// ---------------------------------------------------------------------------

typedef std::vector<CQuickKey> QUICKKEY_ARRAY;

// ---------------------------------------------------------------------------
// hotkey manager class
// ---------------------------------------------------------------------------
class CHotKeyManager {
private:
	DWORD m_dwViewKey;
	BOOL m_bViewKeyRegistered;
	QUICKKEY_ARRAY m_QuickKeyList;

public:
	CHotKeyManager() {
		m_dwViewKey = 0;
		m_bViewKeyRegistered = FALSE;
	}
	CHotKeyManager(const CHotKeyManager& manager) {
		*this = manager;
	}
	virtual ~CHotKeyManager() {
	}

	CHotKeyManager& operator=(const CHotKeyManager& manager) { 
		m_dwViewKey = manager.m_dwViewKey;
		m_QuickKeyList.clear();
		int i;
		for (i = 0; i < (int)manager.m_QuickKeyList.size(); i++) {
			m_QuickKeyList.push_back(manager.m_QuickKeyList[i]);
		}
		m_bViewKeyRegistered = manager.m_bViewKeyRegistered;
		return *this; 
	} 

	void SetViewHotKey(DWORD dwHotKey) {
		m_dwViewKey = dwHotKey;
	}
	void SetViewHotKeyForControl(DWORD dwHotKey) {
		WORD wKeyCode = LOBYTE(LOWORD(dwHotKey));
		WORD wModifiers = HotKeyModifiersFromControl(HIBYTE(LOWORD(dwHotKey)));
		m_dwViewKey = MAKELONG(wKeyCode, wModifiers);
	}
	DWORD GetViewHotKey() {
		return m_dwViewKey;
	}
	DWORD GetViewHotKeyForControl() {
		WORD wKeyCode = LOWORD(m_dwViewKey);
		WORD wModifiers = HotKeyModifiersToControl(HIWORD(m_dwViewKey));
		return MAKELONG(wKeyCode, wModifiers);
	}
	WORD HotKeyModifiersToControl(WORD wValue) {
		WORD wModifiers = 0;
		if (IsFlag(wValue, MOD_ALT)) {
			wModifiers |= HOTKEYF_ALT;
		}
		if (IsFlag(wValue, MOD_CONTROL)) {
			wModifiers |= HOTKEYF_CONTROL;
		}
		if (IsFlag(wValue, MOD_SHIFT)) {
			wModifiers |= HOTKEYF_SHIFT;
		}
		if (IsFlag(wValue, MOD_WIN)) {
			wModifiers |= HOTKEYF_EXT;
		}
		return wModifiers;
	}
	WORD HotKeyModifiersFromControl(WORD wValue) {
		WORD wModifiers = 0;
		if (IsFlag(wValue, HOTKEYF_ALT)) {
			wModifiers |= MOD_ALT;
		}
		if (IsFlag(wValue, HOTKEYF_CONTROL)) {
			wModifiers |= MOD_CONTROL;
		}
		if (IsFlag(wValue, HOTKEYF_SHIFT)) {
			wModifiers |= MOD_SHIFT;
		}
		if (IsFlag(wValue, HOTKEYF_EXT)) {
			wModifiers |= MOD_WIN;
		}
		return wModifiers;
	}
	void ClearQuickKey() {
		m_QuickKeyList.clear();
	}
	void SetQuickHotKey(CFilter* pFilter, DWORD dwHotKey) {
		if (pFilter == NULL) {
			return;
		}
		DeleteQuickHotKey(pFilter);
		if (dwHotKey != 0) {
			m_QuickKeyList.push_back(CQuickKey(pFilter, dwHotKey));
		}
	}
	void SetQuickHotKeyForControl(CFilter* pFilter, DWORD dwHotKey) {
		WORD wKeyCode = LOBYTE(LOWORD(dwHotKey));
		WORD wModifiers = HotKeyModifiersFromControl(HIBYTE(LOWORD(dwHotKey)));
		SetQuickHotKey(pFilter, MAKELONG(wKeyCode, wModifiers));
	}
	DWORD GetQuickHotKey(CFilter* pFilter) {
		if (pFilter == NULL) {
			return 0;
		}
		int i;
		for (i = 0; i < (int)m_QuickKeyList.size(); i++) {
			CQuickKey& QuickKey = m_QuickKeyList[i];
			if (QuickKey.GetFilter() == pFilter) {
				return QuickKey.GetHotKey();
			}
		}
		return 0;
	}
	DWORD GetQuickHotKeyForControl(CFilter* pFilter) {
		DWORD dwHotKey = GetQuickHotKey(pFilter);
		return MAKELONG(LOWORD(dwHotKey), HotKeyModifiersToControl(HIWORD(dwHotKey)));
	}
	void DeleteQuickHotKey(CFilter* pFilter) {
		int i;
		for (i = 0; i < (int)m_QuickKeyList.size(); i++) {
			CQuickKey& QuickKey = m_QuickKeyList[i];
			if (QuickKey.GetFilter() == pFilter) {
				m_QuickKeyList.erase(m_QuickKeyList.begin() + i);
				break;
			}
		}
	}
	CFilter* GetFilter(int nIndex) {
		if (nIndex < 0 || nIndex >= (int)m_QuickKeyList.size()) {
			return NULL;
		}
		CQuickKey& QuickKey = m_QuickKeyList[nIndex];
		return QuickKey.GetFilter();
	}
	DWORD GetFilter(CFilter* pFilter) {
		if (pFilter == NULL) {
			return 0;
		}
		int i;
		for (i = 0; i < (int)m_QuickKeyList.size(); i++) {
			CQuickKey& QuickKey = m_QuickKeyList[i];
			if (QuickKey.GetFilter() == pFilter) {
				return QuickKey.GetHotKey();
			}
		}
		return 0;
	}
	BOOL IsConflict(BOOL bFilter, xstr_t& strMsg) {
		if (! bFilter) {
			if (m_dwViewKey == 0) {
				return FALSE;
			}
			int i;
			for (i = 0; i < (int)m_QuickKeyList.size(); i++) {
				CQuickKey& QuickKey = m_QuickKeyList[i];
				if (m_dwViewKey == QuickKey.GetHotKey()) {
					//TODO: string from resource
					//ExLoadString(UINT nID, LPTSTR pszBuffer, int nBufferSize)
					strMsg = _T("Error: Hotkey conflict.");
					return TRUE;
				}
			}
		}
		else {
			int i;
			for (i = 0; i < (int)m_QuickKeyList.size(); i++) {
				CQuickKey& QuickKey1 = m_QuickKeyList[i];
				if (QuickKey1.GetHotKey() == m_dwViewKey) {
					//TODO: string from resource
					strMsg = _T("Error: Hotkey conflict.");
					return TRUE;
				}
				int j;
				for (j = i + 1; j < (int)m_QuickKeyList.size(); j++) {
					CQuickKey& QuickKey2 = m_QuickKeyList[j];
					if (QuickKey1.GetHotKey() == QuickKey2.GetHotKey()) {
						//TODO: string from resource
						strMsg = _T("Error: Hotkey conflict.");
						return TRUE;
					}
				}
			}
		}
		return FALSE;
	}
	BOOL IsLimitExceeded(xstr_t& strMsg) {
		if (m_QuickKeyList.size() > MAX_HOTKEY) {
			//TODO: string from resource
			strMsg = _T("Error: Hotkey limit exceeded. (MAX: 10)");
			return TRUE;
		}
		return FALSE;
	}
	void LoadProfile();
	void SaveProfile();

	void ClearRegistered() {
		m_bViewKeyRegistered = FALSE;
		int i;
		for (i = 0; i < (int)m_QuickKeyList.size(); i++) {
			CQuickKey& QuickKey = m_QuickKeyList[i];
			QuickKey.SetRegistered(FALSE);
		}
	}

	BOOL RegisterViewHotKey();
	void UnregisterViewHotKey();
	BOOL RegisterQuickHotKey();
	void UnregisterQuickHotKey();
};
