// ===========================================================================
// OperateWnd.cpp
// operate panel window
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include "AnotherVision.h"
#include "Profile.h"
#include "Filter.h"
#include "OperateWnd.h"


// ---------------------------------------------------------------------------
// create filter window
// ---------------------------------------------------------------------------
void COperateWnd::GetInitialWindowRect(LPRECT pRect)
{
	static RECT rcInit = {0, 0, 200, 200};
	RECT rc = GetAppProfile()->GetViewPanelRect();
	if (rc.left == 0 && rc.top == 0 &&
		rc.right == 0 && rc.bottom == 0) {
		rc = rcInit;
	}
	int nDesktopWidth = GetSystemMetrics(SM_CXSCREEN);
	int nDesktopHeight = GetSystemMetrics(SM_CYSCREEN);
	if (rc.right - rc.left >= nDesktopWidth) {
		rc.right = rc.left + nDesktopWidth;
	}
	if (rc.bottom - rc.top >= nDesktopHeight) {
		rc.bottom = rc.top + nDesktopHeight;
	}
	if (rc.right >= nDesktopWidth) {
		OffsetRect(&rc, -(rc.right - nDesktopWidth), 0);
	}
	if (rc.bottom > nDesktopHeight) {
		OffsetRect(&rc, 0, -(rc.bottom - nDesktopHeight));
	}
	if (rc.left < 0) {
		OffsetRect(&rc, -rc.left, 0);
	}
	if (rc.top < 0) {
		OffsetRect(&rc, 0, -rc.top);
	}
	*pRect = rc;
}

// ---------------------------------------------------------------------------
// recalc layout
// ---------------------------------------------------------------------------
void COperateWnd::RecalcLayout()
{
	RECT rcWindow;
	GetWindowRect(&rcWindow);
	OffsetRect(&rcWindow, -rcWindow.left, -rcWindow.top);
	int nWinWidth = rcWindow.right - rcWindow.left;
	int nWinHeight = rcWindow.bottom - rcWindow.top;
	RECT rcClient;
	GetClientRect(&rcClient);
	int nClientWidth = rcClient.right;
	int nClientHeight = rcClient.bottom;

	int nCloseBtnWidth = m_nFontHeight * 7;
	int nCloseBtnHeight = (m_nFontHeight * 14) / 8;

	int nListboxWidth = nClientWidth - m_nFontHeight * 2;
	int nListboxHeight = nClientHeight - m_nFontHeight * 3 - nCloseBtnHeight;

	m_FilterListbox.MoveWindow(m_nFontHeight, m_nFontHeight, nListboxWidth, nListboxHeight, TRUE);

	int nCloseBtnTop = m_nFontHeight * 2 + nListboxHeight;
	int nCloseBtnLeft = (nClientWidth - nCloseBtnWidth) / 2;
	m_CloseBtn.MoveWindow(nCloseBtnLeft, nCloseBtnTop, nCloseBtnWidth, nCloseBtnHeight, TRUE);
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void COperateWnd::AdjustInitialPanelHeight()
{
	if (GetAppProfile()->IsViewAdjustInitialPanelHeight()) {
		int nFilterCount = GetAppFilterManager()->GetEnableFilterCount();
		if (nFilterCount > 30) {
			nFilterCount = 30;
		}
		RECT rc;
		GetWindowRect(&rc);
		int nItemHeight = m_FilterListbox.SendMessage(LB_GETITEMHEIGHT, 0, 0);
		int nListboxHeight = nItemHeight * nFilterCount + GetSystemMetrics(SM_CYEDGE) * 2;
		int nClientHeight = nListboxHeight + m_nFontHeight * 3 + (m_nFontHeight * 14) / 8;
		int nHeight = nClientHeight + GetSystemMetrics(SM_CYSIZEFRAME) * 2 + GetSystemMetrics(SM_CYCAPTION);
		rc.bottom = rc.top + nHeight;
		MoveWindow(rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, FALSE);
	}
}

// ---------------------------------------------------------------------------
// initialize control
// ---------------------------------------------------------------------------
BOOL COperateWnd::InitControls()
{
	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	LOGFONT logFont;
	GetObject((HGDIOBJ)hFont, sizeof(logFont), &logFont);
	int nFontHeight = abs(logFont.lfHeight);
	m_nFontHeight = nFontHeight;

	SendMessage(WM_SETFONT, (WPARAM)hFont, MAKELPARAM(0, 0));

	HWND hwndList = CreateWindowEx(
				WS_EX_CLIENTEDGE,
				_T("LISTBOX"), _T(""),
				WS_VISIBLE | WS_CHILD | WS_TABSTOP | WS_VSCROLL | LBS_NOTIFY | LBS_NOINTEGRALHEIGHT,
				0, 0, 0, 0,
				m_hWnd, (HMENU)IDC_PANEL_FILTER_LIST, _Module.m_hInst, NULL);
	if (hwndList == NULL) {
		ExMessageBox(MB_OK | MB_ICONSTOP, IDS_ERROR_CREATE_WINDOW, GetLastError());
		return FALSE;
	}
	m_FilterListbox = hwndList;
	m_FilterListbox.SendMessage(WM_SETFONT, (WPARAM)hFont, MAKELPARAM(0, 0));

	TCHAR szText[TEXT_BUFFER_SIZE];
	ExLoadString(IDS_FILTER_CLOSEBTN_LBL, szText, sizeof(szText));
	HWND hwndCloseBtn = CreateWindow(
				_T("BUTTON"), szText,
				WS_VISIBLE | WS_CHILD | WS_TABSTOP | BS_PUSHBUTTON,
				0, 0, 0, 0,
				m_hWnd, (HMENU)IDC_PANEL_CLOSE_BUTTON, _Module.m_hInst, NULL);
	if (hwndCloseBtn == NULL) {
		ExMessageBox(MB_OK | MB_ICONSTOP, IDS_ERROR_CREATE_WINDOW, GetLastError());
		return FALSE;
	}
	m_CloseBtn = hwndCloseBtn;
	m_CloseBtn.SendMessage(WM_SETFONT, (WPARAM)hFont, MAKELPARAM(0, 0));

	return TRUE;
}

// ---------------------------------------------------------------------------
// initialize filter
// ---------------------------------------------------------------------------
void COperateWnd::InitFilters()
{
	COLOR_FILTER_PTR_ARRAY& FilterPtrList = GetAppFilterManager()->GetEnableFilters();
	int i;
	for (i = 0; i < (int)FilterPtrList.size(); i++) {
		CFilter* pFilter = FilterPtrList[i];
		int nInsertPos = m_FilterListbox.AddString(pFilter->GetName());
		m_FilterListbox.SetItemData(nInsertPos, (LPARAM)(LPCTSTR)pFilter);
	}
	m_FilterListbox.SendMessage(LB_SETCURSEL, 0, 0);
}

// ---------------------------------------------------------------------------
// save window position
// ---------------------------------------------------------------------------
void COperateWnd::SaveWindowPosition()
{
	RECT rect;
	GetWindowRect(&rect);
	GetAppProfile()->SetViewPanelRect(rect);
}

// ---------------------------------------------------------------------------
// helper of create window
// ---------------------------------------------------------------------------
BOOL COperateWnd::CreateHelper(HWND hwnd)
{
	DWORD dwStyle = WS_VISIBLE | WS_CAPTION | WS_POPUP | WS_THICKFRAME | WS_SYSMENU;
	DWORD dwExStyle = 0;

	SetInitialized(FALSE);
	RECT rc;
	GetInitialWindowRect(&rc);
	if (Create(hwnd, &rc, APP_WINDOW_TITLE, dwStyle, dwExStyle, (HMENU)NULL, NULL) == NULL) {
		return FALSE;
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// OnCreate
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_nCurrentFilterIndex = 0;

	if (! InitControls()) {
		return -1;
	}
	InitFilters();
	AdjustInitialPanelHeight();

	CMessageLoop* pLoop = _Module.GetMessageLoop();
	ATLASSERT(pLoop != NULL);
	pLoop->AddMessageFilter(this);

	SetInitialized(TRUE);

	return 0;
}

// ---------------------------------------------------------------------------
// OnClose
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	GetParent().PostMessage(WM_CLOSE, 0, 0);
	return 0;
}

// ---------------------------------------------------------------------------
// OnDestroy
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	CMessageLoop* pLoop = _Module.GetMessageLoop();
	pLoop->RemoveMessageFilter(this);
	SaveWindowPosition();
	return 0;
}

// ---------------------------------------------------------------------------
// OnPaint
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(&ps);
	HBRUSH hbr3DFace = CreateSolidBrush(GetSysColor(COLOR_3DFACE));
	HBRUSH hbrOld = (HBRUSH)SelectObject(hdc, hbr3DFace);
	RECT rc;
	GetClientRect(&rc);
	FillRect(hdc, &rc, hbr3DFace);
	SelectObject(hdc, hbrOld);
	DeleteObject((HGDIOBJ)hbr3DFace);
	EndPaint(&ps);
	return 0;
}

// ---------------------------------------------------------------------------
// OnKeyDown
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnKeyDown(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	switch (wParam) {
	case VK_ESCAPE:
		return SendMessage(WM_EX_ESCAPE, 0, 0);
	}
	bHandled = FALSE;
	return 0;
}

// ---------------------------------------------------------------------------
// OnSetFocus
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	GetDlgItem(IDC_PANEL_FILTER_LIST).SetFocus();
	return 0;
}

// ---------------------------------------------------------------------------
// OnImeSetContext
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnImeSetContext(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnGetMinMaxInfo
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnGetMinMaxInfo(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
{
	if (IsInitialized()) {
		LPMINMAXINFO pMinMaxInfo = (LPMINMAXINFO)lParam;
		int nItemHeight = m_FilterListbox.SendMessage(LB_GETITEMHEIGHT, 0, 0);
		int nListboxHeight = nItemHeight + GetSystemMetrics(SM_CYEDGE) * 2;
		int nClientHeight = nListboxHeight + m_nFontHeight * 3 + (m_nFontHeight * 14) / 8;
		int nMinHeight = nClientHeight + GetSystemMetrics(SM_CYSIZEFRAME) * 2 + GetSystemMetrics(SM_CYCAPTION);
		pMinMaxInfo->ptMinTrackSize.y = nMinHeight;
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnSize
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (IsInitialized()) {
		RecalcLayout();
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnEscape
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnEscape(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	PostMessage(WM_CLOSE, 0, 0);
	return 0;
}

// ---------------------------------------------------------------------------
// OnCloseBtn
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnCloseBtn(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	PostMessage(WM_CLOSE, 0, 0);
	return 0;
}

// ---------------------------------------------------------------------------
// OnFilterSelChange
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnFilterSelChange(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	int nIndex = m_FilterListbox.GetCurSel();
	if (nIndex != LB_ERR) {
		GetParent().PostMessage(WM_EX_FILTER_CHANGED, nIndex, 0);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnOK
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnOK(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnCancel
// ---------------------------------------------------------------------------
LRESULT COperateWnd::OnCancel(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	PostMessage(WM_CLOSE, 0, 0);
	return 0;
}

// ---------------------------------------------------------------------------
// PreTranslateMessage2
// ---------------------------------------------------------------------------
BOOL COperateWnd::PreTranslateMessage2(MSG* pMsg)
{
	BOOL bResult = FALSE;
	if (pMsg->message == WM_KEYDOWN) {
		if ((pMsg->wParam >= '0' && pMsg->wParam <= '9') ||
			(pMsg->wParam >= VK_NUMPAD0 && pMsg->wParam <= VK_NUMPAD9)) {
			int nIndex = 0;
			if (pMsg->wParam == '0' || pMsg->wParam == VK_NUMPAD0) {
				nIndex = 9;
			}
			else if (pMsg->wParam >= '1' && pMsg->wParam <= '9') {
				nIndex = pMsg->wParam - '1';
			}
			else if (pMsg->wParam >= VK_NUMPAD1 && pMsg->wParam <= VK_NUMPAD9) {
				nIndex = pMsg->wParam - VK_NUMPAD1;
			}
			if (nIndex < m_FilterListbox.GetCount()) {
				int nOldIndex = m_FilterListbox.GetCurSel();
				if (nIndex != nOldIndex) {
					m_FilterListbox.SetCurSel(nIndex);
					GetParent().PostMessage(WM_EX_FILTER_CHANGED, nIndex, 0);
				}
			}
			bResult = TRUE;
		}
	}
	return bResult;
}
