// ===========================================================================
// OptionDlg.cpp
// option dialog
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include <CeLib.h>
#include "AnotherVision.h"
#include "Profile.h"
#include "Filter.h"
#include "FilterWnd.h"
#include "PreviewWnd.h"
#include "FilterOptionDlg.h"
#include "OptionDlg.h"


// ---------------------------------------------------------------------------

HWND COptionDlg::ms_hwndDlg = NULL;
volatile
BOOL COptionDlg::ms_bShowDlg = FALSE;
xstr_t COptionDlg::ms_strRGBType[RGB_DISPLAY_TYPE_MAX];
xstr_t COptionDlg::ms_strImageQualityLevel[IMAGE_QUALITY_LEVEL_MAX];


// ---------------------------------------------------------------------------
// initialize constant (load resource string)
// ---------------------------------------------------------------------------
BOOL COptionDlg::InitConstant()
{
	TCHAR szBuffer[TEXT_BUFFER_SIZE] = _T("");
	BOOL bResult = FALSE;
	int i;
	for (i = 0; i < RGB_DISPLAY_TYPE_MAX; i++) {
		bResult |= ExLoadString(IDS_RGB_DISPLAY_TYPE_BEGIN + i, szBuffer, sizeof(szBuffer));
		ms_strRGBType[i] = szBuffer;
	}
	for (i = 0; i < IMAGE_QUALITY_LEVEL_MAX; i++) {
		bResult |= ExLoadString(IDS_IMAGE_QUALITY_BEGIN + i, szBuffer, sizeof(szBuffer));
		ms_strImageQualityLevel[i] = szBuffer;
	}

	return bResult;
}

// ---------------------------------------------------------------------------
// check dialog showed
// ---------------------------------------------------------------------------
BOOL COptionDlg::IsShowed()
{
	return ms_bShowDlg;
}

// ---------------------------------------------------------------------------
// show dialog
// ---------------------------------------------------------------------------
BOOL COptionDlg::Show(CHotKeyManager& HotKeyManager)
{
	if (CFilterOptionDlg::IsShowed()) {
		return FALSE;
	}
	BOOL bResult = FALSE;
	if (ms_bShowDlg) {
		if (ms_hwndDlg != NULL) {
			SetForegroundWindow(ms_hwndDlg);
		}
	}
	else {
		ms_bShowDlg = TRUE;
		COptionDlg dlg;
		dlg.SetHotKeyManager(HotKeyManager);
		INT_PTR nResult = dlg.DoModal();
		if (nResult == IDOK) {
			HotKeyManager = dlg.GetHotKeyManager();
			HotKeyManager.SaveProfile();
			GetAppProfile()->WriteProfile();
			bResult = TRUE;
		}
		ms_bShowDlg = FALSE;
	}
	return bResult;
}

// ---------------------------------------------------------------------------
// get checked radio button
// ---------------------------------------------------------------------------
static
int GetCheckedRadioButton(CWindow* pWnd, int nIDFirstButton, int nIDLastButton)
{
	for (int nID = nIDFirstButton; nID <= nIDLastButton; nID++) {
		if (pWnd->IsDlgButtonChecked(nID)) {
			return nID;
		}
	}
	return 0;
}

// ---------------------------------------------------------------------------
// load profile
// ---------------------------------------------------------------------------
BOOL COptionDlg::LoadProfile()
{
	CAppProfile* pProfile = GetAppProfile();

	DWORD dwHotKey = m_HotKeyManager.GetViewHotKeyForControl();
	WORD wKeyCode = LOWORD(dwHotKey);
	WORD wModifiers = HIWORD(dwHotKey);
	m_ViewHotkey.SetHotKey(wKeyCode, wModifiers);
	m_ViewAdjustInitialPanelHeightCheck.SetCheck(pProfile->IsViewAdjustInitialPanelHeight() ? BST_CHECKED : BST_UNCHECKED);

	UINT nWaitingTime = pProfile->GetQuickWaitingTime();
	TCHAR szNum[10];
	wsprintf(szNum, _T("%u"), nWaitingTime);
	int nIndex = m_QuickWaitingTimeCombo.FindStringExact(NULL, szNum);
	if (nIndex != CB_ERR) {
		m_QuickWaitingTimeCombo.SetCurSel(nIndex);
	}
	else {
		m_QuickWaitingTimeCombo.SetWindowText(szNum);
	}

	UINT nInterval = pProfile->GetWindowUpdateInterval();
	wsprintf(szNum, _T("%u"), nInterval);
	nIndex = m_WindowUpdateIntervalCombo.FindStringExact(NULL, szNum);
	if (nIndex != CB_ERR) {
		m_WindowUpdateIntervalCombo.SetCurSel(nIndex);
	}
	else {
		m_WindowUpdateIntervalCombo.SetWindowText(szNum);
	}
	m_WindowRGBTypeCombo.SetCurSel(pProfile->GetWindowColorDisplayFormat());
	m_WindowUnvisibleCheck.SetCheck(pProfile->GetWindowUnvisibleTaskBar() ? BST_CHECKED : BST_UNCHECKED);
	m_WindowSimpleStatusbarInfoCheck.SetCheck(pProfile->IsWindowSimpleStatusbarInfo() ? BST_CHECKED : BST_UNCHECKED);
	m_WindowShowOriginalColorInfoCheck.SetCheck(pProfile->IsWindowShowOriginalColorInfo() ? BST_CHECKED : BST_UNCHECKED);

	int nQualityLevel = IMAGE_QUALITY_LEVEL_NORMAL;
	if (pProfile->GetExtendColor() == 12) {
		nQualityLevel = IMAGE_QUALITY_LEVEL_HIGH;
	}
	else if (pProfile->GetExtendColor() >= 1 && pProfile->GetExtendColor() <= 6) {
		nQualityLevel = IMAGE_QUALITY_LEVEL_LOW;
	}
	m_ImageQualityCombo.SetCurSel(nQualityLevel);
	m_NoSplashCheck.SetCheck(pProfile->IsNoSplash() ? BST_CHECKED : BST_UNCHECKED);

	return TRUE;
}

// ---------------------------------------------------------------------------
// save profile
// ---------------------------------------------------------------------------
BOOL COptionDlg::SaveProfile()
{
	UpdateHotKeyData();
	CAppProfile* pProfile = GetAppProfile();

	m_HotKeyManager.SetViewHotKeyForControl(m_ViewHotkey.GetHotKey());
	pProfile->SetViewHotKey(m_ViewHotkey.GetHotKey());
	pProfile->SetViewAdjustInitialPanelHeight(m_ViewAdjustInitialPanelHeightCheck.GetCheck() == BST_CHECKED ? TRUE : FALSE);

	TCHAR szBuffer[TEXT_BUFFER_SIZE];
	m_QuickWaitingTimeCombo.GetWindowText(szBuffer, sizeof(szBuffer));
	UINT nTime = _tcstoul(szBuffer, NULL, 10);
	pProfile->SetQuickWaitingTime(nTime);

	m_WindowUpdateIntervalCombo.GetWindowText(szBuffer, sizeof(szBuffer));
	UINT nInterval = _tcstoul(szBuffer, NULL, 10);
	pProfile->SetWindowUpdateInterval(nInterval);

	int nIndex = m_WindowRGBTypeCombo.GetCurSel();
	if (nIndex == CB_ERR) {
		nIndex = 0;
	}
	pProfile->SetWindowColorDisplayFormat(nIndex);
	pProfile->SetWindowUnvisibleTaskBar((m_WindowUnvisibleCheck.GetCheck() == BST_CHECKED) ? TRUE : FALSE);
	pProfile->SetWindowSimpleStatusbarInfo((m_WindowSimpleStatusbarInfoCheck.GetCheck() == BST_CHECKED) ? TRUE : FALSE);
	pProfile->SetWindowShowOriginalColorInfo((m_WindowShowOriginalColorInfoCheck.GetCheck() == BST_CHECKED) ? TRUE : FALSE);

	nIndex = m_ImageQualityCombo.GetCurSel();
	if (nIndex == CB_ERR) {
		nIndex = IMAGE_QUALITY_LEVEL_NORMAL;
	}
	int nExtendColor;
	switch (nIndex) {
	case IMAGE_QUALITY_LEVEL_LOW:
		nExtendColor = 3;
		break;
	case IMAGE_QUALITY_LEVEL_HIGH:
		nExtendColor = 12;
		break;
	case IMAGE_QUALITY_LEVEL_NORMAL:
	default:
		nExtendColor = 7;
		break;
	}
	pProfile->SetExtendColor(nExtendColor);
	pProfile->SetNoSplash((m_NoSplashCheck.GetCheck() == BST_CHECKED) ? TRUE : FALSE);

	return TRUE;
}

// ---------------------------------------------------------------------------
// control-valiable mapping
// ---------------------------------------------------------------------------
void COptionDlg::ControlVariableMapping()
{
	m_ViewHotkey = GetDlgItem(IDC_OPTION_VIEW_HOTKEY);
	m_ViewAdjustInitialPanelHeightCheck = GetDlgItem(IDC_OPTION_VIEW_ADJUST_PANEL_HEIGHT_CHECK);
	m_QuickWaitingTimeCombo = GetDlgItem(IDC_OPTION_QUICK_WAITING_TIME_COMBO);
	m_WindowUpdateIntervalCombo = GetDlgItem(IDC_OPTION_WND_INTERVAL_COMBO);
	m_WindowRGBTypeCombo = GetDlgItem(IDC_OPTION_WND_RGB_TYPE_COMBO);
	m_WindowUnvisibleCheck = GetDlgItem(IDC_OPTION_WND_UNVISIBLE_TASKTRAY);
	m_WindowSimpleStatusbarInfoCheck = GetDlgItem(IDC_OPTION_WND_SIMPLE_STATUSBAR_INFO);
	m_WindowShowOriginalColorInfoCheck = GetDlgItem(IDC_OPTION_WND_ORIGINAL_COLOR_CHECK);
	m_ImageQualityCombo = GetDlgItem(IDC_OPTION_IMG_QUALITY_COMBO);
	m_NoSplashCheck = GetDlgItem(IDC_OPTION_NOSPLASH_CHECK);
}

// ---------------------------------------------------------------------------
// initialize control
// ---------------------------------------------------------------------------
void COptionDlg::InitControls()
{
	int i;
	static LPCTSTR apszWaitingTimeText[] = {
			_T("0"), _T("200"), _T("500"), _T("1000"), _T("2000"), _T("5000")};
	for (i = 0; i < ARRAYSIZE(apszWaitingTimeText); i++) {
		m_QuickWaitingTimeCombo.InsertString(-1, apszWaitingTimeText[i]);
	}
	static LPCTSTR apszUpdateIntervalText[] = {
			_T("50"), _T("100"), _T("200"), _T("300"), _T("500"), _T("1000")};
	for (i = 0; i < ARRAYSIZE(apszUpdateIntervalText); i++) {
		m_WindowUpdateIntervalCombo.InsertString(-1, apszUpdateIntervalText[i]);
	}
	for (i = 0; i < RGB_DISPLAY_TYPE_MAX; i++) {
		m_WindowRGBTypeCombo.InsertString(-1, (LPCTSTR)ms_strRGBType[i]);
	}
	for (i = 0; i < IMAGE_QUALITY_LEVEL_MAX; i++) {
		m_ImageQualityCombo.InsertString(-1, (LPCTSTR)ms_strImageQualityLevel[i]);
	}
}

// ---------------------------------------------------------------------------
// OnInitDialog
// ---------------------------------------------------------------------------
LRESULT COptionDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	ms_hwndDlg = m_hWnd;
	CenterWindow(GetParent());
	ControlVariableMapping();
	InitControls();
	LoadProfile();
	EnableButtons(TRUE);

	return TRUE;
}

// ---------------------------------------------------------------------------
// OnOK
// ---------------------------------------------------------------------------
LRESULT COptionDlg::OnOK(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	SaveProfile();
	GetAppProfile()->WriteProfile();
	EndDialog(IDOK);
	ms_hwndDlg = NULL;
	return 0;
}

// ---------------------------------------------------------------------------
// OnCancel
// ---------------------------------------------------------------------------
LRESULT COptionDlg::OnCancel(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	EndDialog(IDCANCEL);
	ms_hwndDlg = NULL;
	return 0;
}

// ---------------------------------------------------------------------------
// OnControlStatusChanged
// ---------------------------------------------------------------------------
LRESULT COptionDlg::OnControlStatusChanged(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
	EnableButtons();
	return 0;
}

// ---------------------------------------------------------------------------
// EnableButtons
// ---------------------------------------------------------------------------
void COptionDlg::EnableButtons(BOOL bUpdateList/*= FALSE*/)
{
}

// ---------------------------------------------------------------------------
// UpdateHotKeyData
// ---------------------------------------------------------------------------
void COptionDlg::UpdateHotKeyData()
{
}
