// ===========================================================================
// PreviewWnd.cpp
// preview window
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include "AnotherVision.h"
#include "common.h"
#include "Profile.h"
#include "AboutDlg.h"
#include "OptionDlg.h"
#include "OperateWnd.h"
#include "FilterOptionDlg.h"
#include "PreviewWnd.h"


// ---------------------------------------------------------------------------

#define PREVIEW_TIMER_ID         100
#define PREVIEW_TIMER_ELAPSE     100


// ---------------------------------------------------------------------------

static
CPreviewWnd s_PreviewWnd;


// ---------------------------------------------------------------------------
// trace helper
// for debug
// ---------------------------------------------------------------------------
static
void XATLTRACE(LPCTSTR pszMsg)
{
	static DWORD dwPrevTickCount = 0;
	DWORD dwTickCount = GetTickCount();
	DWORD dwDistance = dwTickCount - dwPrevTickCount;
	dwPrevTickCount = dwTickCount;
	ATLTRACE2(atlTraceGeneral, 0, _T("%s (%d) %d\n"), pszMsg, dwTickCount, dwDistance);
}

// ---------------------------------------------------------------------------
// create preview window
// ---------------------------------------------------------------------------
BOOL CPreviewWnd::CreateHelper(BOOL bQuick, int nFilterIndex)
{
	if (s_PreviewWnd.IsShowed()) {
		return FALSE;
	}
	DWORD dwStyle = WS_VISIBLE | WS_POPUP | WS_MAXIMIZE;
#if defined(_DEBUG)
	DWORD dwExStyle = 0;
#else
	DWORD dwExStyle = WS_EX_TOPMOST;
#endif
#if 0
	if (IsWin2K()) {
		dwExStyle |= WS_EX_NOACTIVATE;
	}
#endif
	s_PreviewWnd.m_bQuickMode = bQuick;
	s_PreviewWnd.m_nInitialFilterIndex = nFilterIndex;
	if (s_PreviewWnd.Create(NULL, NULL, APP_WINDOW_TITLE, dwStyle, dwExStyle, (HMENU)NULL, NULL) == NULL) {
		return FALSE;
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// destroy preview window
// ---------------------------------------------------------------------------
void DestroyPreviewWnd()
{
	if (s_PreviewWnd.IsShowed()) {
		s_PreviewWnd.PostMessage(WM_CLOSE, 0, 0);
	}
}

// ---------------------------------------------------------------------------
// check preview window showed
// ---------------------------------------------------------------------------
BOOL CPreviewWnd::IsShowed()
{
	return s_PreviewWnd.m_bShowed;
}

// ---------------------------------------------------------------------------
// show window (create or activate)
// ---------------------------------------------------------------------------
void CPreviewWnd::Activate(BOOL bQuick/*= FALSE*/, int nFilterIndex/*= 0*/)
{
	if (COptionDlg::IsShowed() ||
		CFilterOptionDlg::IsShowed()) {
		return;
	}
	HDC hdcDesktop = ::GetDC(GetDesktopWindow()); // DC of primary monitor
	int nBitsPixcel	= GetDeviceCaps(hdcDesktop, BITSPIXEL);
	::ReleaseDC(GetDesktopWindow(), hdcDesktop);
	if (nBitsPixcel <= 8) {
		// notice: no enough color
		return;
	}
	if (CAboutDlg::IsShowed()) {
		CAboutDlg::Close();
	}
	if (! s_PreviewWnd.IsShowed()) {
		CreateHelper(bQuick, nFilterIndex);
	}
	if (! bQuick) {
		s_PreviewWnd.ActivateOperateWnd();
	}
}

// ---------------------------------------------------------------------------
// window activate
// ---------------------------------------------------------------------------
void CPreviewWnd::ActivateOperateWnd()
{
	ExSetForegroundWindow(m_OperateWnd.m_hWnd);
}

// ---------------------------------------------------------------------------
// constructor
// ---------------------------------------------------------------------------
CPreviewWnd::CPreviewWnd()
{
	m_bShowed = FALSE;
}

// ---------------------------------------------------------------------------
// OnCreate
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_FilterStatus.SetQuickMode(m_bQuickMode);
	m_bQuickFilterEnd = FALSE;
	m_FilterStatus.ResetSelectedEnableFilterIndex();

	CaptureScreen();

	int nFilterIndex = 0;
	if (m_bQuickMode) {
		nFilterIndex = m_nInitialFilterIndex;
	}
	m_FilterStatus.SetSelectedEnableFilterIndex(nFilterIndex);
	ChangeFilter(m_FilterStatus.GetSelectedEnableFilter());

	if (! m_bQuickMode) {
		if (! m_OperateWnd.CreateHelper(m_hWnd)) {
			return -1;
		}
	}
	m_nTimerID = SetTimer(PREVIEW_TIMER_ID, PREVIEW_TIMER_ELAPSE, NULL);
	m_bShowed = TRUE;
	m_bClosing = FALSE;

	return 0;
}

// ---------------------------------------------------------------------------
// OnClose
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnClose(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
{
	if (! m_bClosing) {
		m_bClosing = TRUE;
		PostMessage(WM_CLOSE, 0, 0);
		return 0;
	}
	DestroyWindow();
	return 0;
}

// ---------------------------------------------------------------------------
// OnDestroy
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (m_nTimerID != 0) {
		KillTimer(PREVIEW_TIMER_ID);
		m_nTimerID = 0;
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnKeyDown
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnKeyDown(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	if (wParam == VK_ESCAPE) {
		m_bClosing = TRUE;
		PostMessage(WM_CLOSE, 0, 0);
		return 0;
	}
	bHandled = FALSE;
	return 0;
}

// ---------------------------------------------------------------------------
// OnPaint
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnPaint(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(&ps);

#if 0
	BitBlt(hdc, 0, 0, m_CaptureImage.GetWidth(), m_CaptureImage.GetHeight(),
			m_CaptureImage.GetDC(), 0, 0,
			SRCCOPY);
#else
	BitBlt(hdc, ps.rcPaint.left, ps.rcPaint.top, ps.rcPaint.right - ps.rcPaint.left, ps.rcPaint.bottom - ps.rcPaint.top,
			m_CaptureImage.GetDC(), ps.rcPaint.left, ps.rcPaint.top,
			SRCCOPY);
#endif

	EndPaint(&ps);

	return 0;
}

// ---------------------------------------------------------------------------
// OnEraseBkGnd
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnEraseBkGnd(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnSetFocus
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnSetFocus(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnMouseActivate
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnMouseActivate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (m_bQuickMode) {
		return MA_ACTIVATE;
	}
	ActivateOperateWnd();
	return MA_NOACTIVATE;
}

// ---------------------------------------------------------------------------
// OnIMESetContext
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnIMESetContext(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnTimer
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnTimer(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (wParam != PREVIEW_TIMER_ID) {
		return 0;
	}
	if (m_bQuickMode) {
		if (m_bQuickFilterEnd) {
			DWORD dwWaiting = GetTickCount() - m_dwQuickFilterEndTime;
			if (dwWaiting >= GetAppProfile()->GetQuickWaitingTime()) {
				m_bClosing = TRUE;
				PostMessage(WM_CLOSE);
			}
			return 0;
		}
	}
	CFilter* pFilter = m_FilterStatus.GetSelectedEnableFilter();
	if (! pFilter->IsRefFilter()) {
		return 0;
	}
	if (m_FilterStatus.IsNextFilterItemReady()) {
		CFilter* pDefFilter = m_FilterStatus.NextFilterItem();
		if (pDefFilter == NULL) {
XATLTRACE(_T("filter item empty"));
		}
		else {
			ApplyFilter(pDefFilter);
			RedrawWindow(NULL, NULL, RDW_INTERNALPAINT | RDW_INVALIDATE);
		}
	}
	else {
		if (m_bQuickMode) {
			if (! m_bQuickFilterEnd) {
				if (! m_FilterStatus.HasNextFilterItem()) {
					QuickFilterEnd();
				}
			}
		}
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnLButtonDblClk
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnLButtonDblClk(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_bClosing = TRUE;
	PostMessage(WM_CLOSE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnFilterChange
// ---------------------------------------------------------------------------
LRESULT CPreviewWnd::OnFilterChange(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
//	if (m_pDesktopBitsFilter == NULL) {
//		return 0;
//	}
	int nFilterIndex = (int)wParam;
	if (nFilterIndex == m_FilterStatus.GetSelectedEnableFilterIndex()) {
		return 0;
	}
	m_FilterStatus.SetSelectedEnableFilterIndex(nFilterIndex);
	ChangeFilter(m_FilterStatus.GetSelectedEnableFilter());
	RedrawWindow(NULL, NULL, RDW_INTERNALPAINT | RDW_INVALIDATE);
	return 0;
}

// ---------------------------------------------------------------------------
// OnFinalMessage
// ---------------------------------------------------------------------------
void CPreviewWnd::OnFinalMessage(HWND /*hWnd*/)
{
	m_bShowed = FALSE;
}

// ---------------------------------------------------------------------------
// capture screen
// ---------------------------------------------------------------------------
BOOL CPreviewWnd::CaptureScreen()
{
	m_nDesktopWidth = GetSystemMetrics(SM_CXSCREEN);
	m_nDesktopHeight = GetSystemMetrics(SM_CYSCREEN);
	if (! m_CaptureImage.Capture(0, 0, m_nDesktopWidth, m_nDesktopHeight)) {
		return FALSE;
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// apply filter to image
// ---------------------------------------------------------------------------
void CPreviewWnd::ApplyFilter(CFilter* pFilter)
{
	m_CaptureImage.ApplyFilter(pFilter);
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CPreviewWnd::ChangeFilter(CFilter* pFilter)
{
	if (pFilter->IsDefFilter()) {
		ApplyFilter(pFilter);
		QuickFilterEnd();
	}
	else {
		m_FilterStatus.ResetFilterItem();
		if (m_FilterStatus.IsNextFilterItemReady()) {
			CFilter* pTmpFilter = m_FilterStatus.NextFilterItem();
			if (pTmpFilter == NULL) {
XATLTRACE(_T("filter item empty"));
			}
			else {
				ApplyFilter(pTmpFilter);
			}
		}
	}
}

// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
void CPreviewWnd::QuickFilterEnd()
{
	if (m_bQuickFilterEnd) {
		return;
	}
	m_dwQuickFilterEndTime = GetTickCount();
	m_bQuickFilterEnd = TRUE;
}
