// ===========================================================================
// profile.cpp
// application profile
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include "common.h"
#include "AnotherVision.h"
#include "Filter.h"
#include "Profile.h"



// ---------------------------------------------------------------------------

#define VIEW_HOTKEY_DEFAULT  0x2007a // Ctrl + F11


// ---------------------------------------------------------------------------

static
CAppProfile s_AppProfile;


// ---------------------------------------------------------------------------
// get profile instance
// ---------------------------------------------------------------------------
CAppProfile* GetAppProfile()
{
	return &s_AppProfile;
}

/////////////////////////////////////////////////////////////////////////////

// ---------------------------------------------------------------------------
// read profile from registry
// ---------------------------------------------------------------------------
BOOL CAppProfile::ReadProfile()
{
	DWORD dwValue;
	int x, y;
	int width, height;
	TCHAR szBuffer[4096];
	ReadRegText(APP_REG_KEY, _T("DesktopPanelPosition"), szBuffer, sizeof(szBuffer), _T("0,0,200,200"));
	if (_stscanf(szBuffer, _T("%d,%d,%d,%d"), &x, &y, &width, &height) == 4) {
		RECT rc = {x, y, x + width, y + height};
		GetAppProfile()->SetViewPanelRect(rc);
	} else {
		RECT rc = {0, 0, 200, 200};
		GetAppProfile()->SetViewPanelRect(rc);
	}
	ReadRegInt(APP_REG_KEY, _T("NoSplash"), &dwValue, (DWORD)-1);
	GetAppProfile()->SetFirstRun((dwValue == (DWORD)-1) ? TRUE : FALSE);
	GetAppProfile()->SetNoSplash((dwValue == 1) ? TRUE : FALSE);


	ReadRegText(APP_REG_KEY, _T("QuickHotKeys"), szBuffer, sizeof(szBuffer), _T(""));
	LPTSTR psz = szBuffer;
	for (;m_QuickKeyList.size() < 20;) {
		xstr_t strID;
		xstr_t strHotKey;
		DWORD dwHotKey;
		LPTSTR pszSep = _tcschr(psz, _T(','));
		if (pszSep == NULL) {
			break;
		}
		*pszSep = _T('\0');
		strID = psz;
		psz = pszSep + 1;
		pszSep = _tcschr(psz, _T(','));
		if (pszSep == NULL) {
			strHotKey = psz;
		}
		else {
			*pszSep = _T('\0');
			strHotKey = psz;
			psz = pszSep + 1;
		}
		strID.trim();
		strHotKey.trim();
		if (strID.length() == 0 ||
			strHotKey.length() == 0) {
			break;
		}
		dwHotKey = _tcstoul((LPCTSTR)strHotKey, NULL, 0);
		if (dwHotKey == 0) {
			break;
		}
		m_QuickKeyList.push_back(CQuickKeyItem((LPCTSTR)strID, dwHotKey));
		if (pszSep == NULL) {
			break;
		}
	}
	ReadRegInt(APP_REG_KEY, _T("DesktopHotKey"), &dwValue, VIEW_HOTKEY_DEFAULT);
	SetViewHotKey(dwValue);
	ReadRegInt(APP_REG_KEY, _T("DesktopAdjustInitialPanelHeight"), &dwValue, 1);
	SetViewAdjustInitialPanelHeight((dwValue == 1) ? TRUE : FALSE);

	ReadRegText(APP_REG_KEY, _T("EnableFilters"), szBuffer, sizeof(szBuffer), _T(""));
	if (lstrcmp(szBuffer, _T("")) == 0) {
		lstrcpy(szBuffer, DEFAULT_ENABLE_FILTERS);
	}
	psz = szBuffer;
	for (;;) {
		LPTSTR pszSep = _tcschr(psz, _T(','));
		if (pszSep == NULL) {
			TrimText(psz);
			if (*psz != '\0') {
				m_EnableFilterList.push_back(psz);
			}
			break;
		}
		*pszSep = _T('\0');
		TrimText(psz);
		if (*psz != _T('\0')) {
			m_EnableFilterList.push_back(psz);
		}
		psz = pszSep + 1;
	}
	XSTR_ARRAY& FilterList = GetEnableFilterList();
	GetAppFilterManager()->SetEnableFilter(FilterList);

	ReadRegInt(APP_REG_KEY, _T("QuickWaitingTime"), &dwValue, 1000);
	SetQuickWaitingTime(dwValue);

	ReadRegInt(APP_REG_KEY, _T("WindowUpdateInterval"), &dwValue, 100);
	SetWindowUpdateInterval(dwValue);
	ReadRegInt(APP_REG_KEY, _T("WindowColorDisplayFormat"), &dwValue, RGB_DISPLAY_TYPE_RGB_DEC);
	SetWindowColorDisplayFormat(dwValue);

	ReadRegInt(APP_REG_KEY, _T("WindowUnvisibleTaskBar"), &dwValue, 0);
	SetWindowUnvisibleTaskBar((dwValue == 1) ? TRUE : FALSE);
	ReadRegInt(APP_REG_KEY, _T("WindowSimpleStatusbarInfo"), &dwValue, 0);
	SetWindowSimpleStatusbarInfo((dwValue == 1) ? TRUE : FALSE);
	ReadRegInt(APP_REG_KEY, _T("WindowShowOriginalColorInfo"), &dwValue, 0);
	SetWindowShowOriginalColorInfo((dwValue == 1) ? TRUE : FALSE);
	ReadRegInt(APP_REG_KEY, _T("WindowLeftTopOrigin"), &dwValue, 0);
	SetWindowLeftTopOrigin((dwValue == 1) ? TRUE : FALSE);

	ReadRegText(APP_REG_KEY, _T("WindowSetting"), szBuffer, sizeof(szBuffer), _T(""));
	m_WindowSetting.Load(szBuffer);
	ReadRegInt(APP_REG_KEY, _T("MultiWindowSettingNum"), &dwValue, 0);
	int nNum = dwValue;
	if (nNum > 20) {
		nNum = 20;
	}
	int i;
	for (i = 0; i < nNum; i++) {
		TCHAR szKey[TEXT_BUFFER_SIZE];
		wsprintf(szKey, _T("MultiWindowSetting_%02d"), i + 1);
		ReadRegText(APP_REG_KEY, szKey, szBuffer, sizeof(szBuffer), _T(""));
		CWindowSetting setting;
		if (! setting.Load(szBuffer)) {
			break;
		}
		m_MultiWindowSettings.push_back(setting);
	}

	ReadRegText(APP_REG_KEY, _T("CopyColorFormat"), szBuffer, sizeof(szBuffer), _T("#%02XR%02XG%02XB"));
	SetCopyColorFormat(szBuffer);
	ReadRegInt(APP_REG_KEY, _T("ImageQuality"), &dwValue, 0);
	SetExtendColor(dwValue);

	return TRUE;
}

// ---------------------------------------------------------------------------
// write profile to registry
// ---------------------------------------------------------------------------
BOOL CAppProfile::WriteProfile()
{
//	TCHAR szBuffer[TEXT_BUFFER_SIZE];
	TCHAR szBuffer[4096];
	XSTR_ARRAY FilterList;
	GetAppFilterManager()->GetEnableFilter(FilterList);
	SetEnableFilterList(FilterList);

	RECT rc = GetAppProfile()->GetViewPanelRect();
	_stprintf(szBuffer, _T("%d,%d,%d,%d"), rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top);
	WriteRegText(APP_REG_KEY, _T("DesktopPanelPosition"), szBuffer);
	WriteRegInt(APP_REG_KEY, _T("NoSplash"), GetAppProfile()->IsNoSplash() ? 1 : 0);

	szBuffer[0] = '\0';
	int i;
	for (i = 0; i < (int)m_QuickKeyList.size(); i++) {
		CQuickKeyItem& key = m_QuickKeyList[i];
		if (szBuffer[0] != '\0') {
			lstrcat(szBuffer, _T(","));
		}
		TCHAR szBuffer2[TEXT_BUFFER_SIZE];
		wsprintf(szBuffer2, _T("%s,%d"), (LPCTSTR)key.GetFilterID(), key.GetHotKey());
		lstrcat(szBuffer, szBuffer2);
	}
	WriteRegText(APP_REG_KEY, _T("QuickHotKeys"), szBuffer);

	WriteRegInt(APP_REG_KEY, _T("DesktopHotKey"), GetViewHotKey());
	WriteRegInt(APP_REG_KEY, _T("DesktopAdjustInitialPanelHeight"), IsViewAdjustInitialPanelHeight() ? 1 : 0);

	WriteRegInt(APP_REG_KEY, _T("QuickWaitingTime"), GetQuickWaitingTime());
	szBuffer[0] = '\0';
	for (i = 0; i < (int)m_EnableFilterList.size(); i++) {
		xstr_t& s = m_EnableFilterList[i];
		if (szBuffer[0] != '\0') {
			lstrcat(szBuffer, _T(","));
		}
		lstrcat(szBuffer, (LPCTSTR)s);
	}
	WriteRegText(APP_REG_KEY, _T("EnableFilters"), szBuffer);

	WriteRegInt(APP_REG_KEY, _T("WindowUpdateInterval"), m_nWindowUpdateInterval);
	WriteRegInt(APP_REG_KEY, _T("WindowColorDisplayFormat"), m_nWindowColorDisplayFmt);
	WriteRegInt(APP_REG_KEY, _T("WindowUnvisibleTaskBar"), m_bWindowUnvisibleTaskBar ? 1 : 0);
	WriteRegInt(APP_REG_KEY, _T("WindowSimpleStatusbarInfo"), m_bWindowSimpleStatusbarInfo ? 1 : 0);
	WriteRegInt(APP_REG_KEY, _T("WindowShowOriginalColorInfo"), m_bWindowShowOriginalColorInfo ? 1 : 0);
//	WriteRegInt(APP_REG_KEY, _T("WindowLeftTopOrigin"), m_bWindowLeftTopOrigin ? 1 : 0);

	xstr_t strWindowSetting = m_WindowSetting.Save();
	WriteRegText(APP_REG_KEY, _T("WindowSetting"), (LPCTSTR)strWindowSetting);
	int nNum = (int)m_MultiWindowSettings.size();
	if (nNum > 20) {
		nNum = 20;
	}
	WriteRegInt(APP_REG_KEY, _T("MultiWindowSettingNum"), nNum);
	for (i = 0; i < nNum; i++) {
		CWindowSetting setting = m_MultiWindowSettings[i];
		TCHAR szKey[TEXT_BUFFER_SIZE];
		wsprintf(szKey, _T("MultiWindowSetting_%02d"), i + 1);
		strWindowSetting = setting.Save();
		WriteRegText(APP_REG_KEY, szKey, (LPCTSTR)strWindowSetting);
	}
	WriteRegInt(APP_REG_KEY, _T("ImageQuality"), GetExtendColor());

//	WriteRegText(APP_REG_KEY, _T("CopyColorFormat"), GetCopyColorFormat());

	return TRUE;
}
