// ===========================================================================
// profile.h
// application profile
//
//                                                       by kanegon 2006
// ===========================================================================

#pragma once

#include "common.h"
#include "AnotherVision.h"


// ---------------------------------------------------------------------------

class CAppProfile;
CAppProfile* GetAppProfile();


// ---------------------------------------------------------------------------
// window setting
// ---------------------------------------------------------------------------
class CWindowSetting {
private:
	xstr_t m_strFilter;
	UINT m_nZoom;
	BOOL m_bCenterMark;
	BOOL m_bTopmost;
	RECT m_rcWindow;
	BOOL m_bFixedMousePosition;
	POINT m_FixedCursorPos;
	BOOL m_bShowStatusBar;

public:
	CWindowSetting() {
		InitData();
	}
	CWindowSetting(const CWindowSetting& setting) {
		m_strFilter = setting.m_strFilter;
		m_nZoom = setting.m_nZoom;
		m_bCenterMark = setting.m_bCenterMark;
		m_bTopmost = setting.m_bTopmost;
		m_rcWindow = setting.m_rcWindow;
		m_bFixedMousePosition = setting.m_bFixedMousePosition;
		m_FixedCursorPos = setting.m_FixedCursorPos;
		if (! m_bFixedMousePosition) {
			ZeroMemory(&m_FixedCursorPos, sizeof(m_FixedCursorPos));
		}
		m_bShowStatusBar = setting.m_bShowStatusBar;
	}
	virtual ~CWindowSetting() {
	}

public:
	void InitData() {
		m_strFilter = _T("");
		m_nZoom = 1;
		m_bCenterMark = FALSE;
		m_bTopmost = FALSE;
		SetRectEmpty(&m_rcWindow);
		m_bFixedMousePosition = FALSE;
		ZeroMemory(&m_FixedCursorPos, sizeof(m_FixedCursorPos));
		m_bShowStatusBar = TRUE;
	}
	void SetFilter(LPCTSTR pszFilterID) {
		if (pszFilterID != NULL) {
			m_strFilter = pszFilterID;
		}
	}
	xstr_t GetFilter() {
		return m_strFilter;
	}
	void SetZoom(UINT nZoom) {
		int nTmpZoom = nZoom;
		if (nTmpZoom < MIN_ZOOM || nTmpZoom > MAX_ZOOM) {
			nTmpZoom = 1;
		}
		m_nZoom = nTmpZoom;
	}
	UINT GetZoom() {
		return m_nZoom;
	}
	void SetCenterMark(BOOL bEnable) {
		m_bCenterMark = bEnable;
	}
	BOOL IsCenterMark() {
		return m_bCenterMark;
	}
	void SetTopmost(BOOL bEnable) {
		m_bTopmost = bEnable;
	}
	BOOL IsTopmost() {
		return m_bTopmost;
	}
	void SetRect(RECT* pRect) {
		m_rcWindow = *pRect;
	}
	void GetRect(RECT* pRect) {
		*pRect = m_rcWindow;
	}
	void SetFixedMousePosition(BOOL bEnable, POINT* pos) {
		m_bFixedMousePosition = bEnable;
		m_FixedCursorPos = *pos;;
		if (! m_bFixedMousePosition) {
			ZeroMemory(&m_FixedCursorPos, sizeof(m_FixedCursorPos));
		}
	}
	void GetFixedMousePosition(BOOL& bEnable, POINT& pos) {
		bEnable = m_bFixedMousePosition;
		pos = m_FixedCursorPos;
	}
	void SetShowStatusBar(BOOL bShow) {
		m_bShowStatusBar = bShow;
	}
	BOOL IsShowStatusBar() {
		return m_bShowStatusBar;
	}

	xstr_t Save() {
		TCHAR szBuffer[TEXT_BUFFER_SIZE];
		_sntprintf(szBuffer, sizeof(szBuffer), _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%s"),
				m_nZoom,
				m_bCenterMark ? 1 : 0,
				m_bTopmost ? 1 : 0,
				m_bFixedMousePosition ? 1 : 0,
				m_FixedCursorPos.x, m_FixedCursorPos.y,
				m_bShowStatusBar ? 1 : 0,
				m_rcWindow.left, m_rcWindow.top,
				m_rcWindow.right - m_rcWindow.left, m_rcWindow.bottom - m_rcWindow.top,
				(LPCTSTR)m_strFilter);
		return szBuffer;
	}
	BOOL Load(LPCTSTR pszBuffer) {
		InitData();
		if (lstrlen(pszBuffer) >= TEXT_BUFFER_SIZE) {
			return FALSE;
		}
		int nZoom;
		int nCenterMark;
		int nTopmost;
		int nFixedMousePosition;
		int nShowStatusBar;
		int posx, posy;
		int x, y, width, height;
		TCHAR szFilter[TEXT_BUFFER_SIZE];
		int nResult = _stscanf(pszBuffer, _T("%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%d,%s"),
				&nZoom,
				&nCenterMark,
				&nTopmost,
				&nFixedMousePosition,
				&posx, &posy,
				&nShowStatusBar,
				&x, &y, &width, &height,
				szFilter);
		if (nResult != 12) {
			return FALSE;
		}
		SetZoom(nZoom);
		SetCenterMark((nCenterMark == 1) ? TRUE : FALSE);
		SetTopmost((nTopmost == 1) ? TRUE : FALSE);
		POINT pos;
		pos.x = posx;
		pos.y = posy;
		SetFixedMousePosition((nFixedMousePosition == 1) ? TRUE : FALSE, &pos);
		SetShowStatusBar((nShowStatusBar == 1) ? TRUE : FALSE);
		RECT rc;
		::SetRect(&rc, x, y, x + width, y + height);
		SetRect(&rc);
		SetFilter(szFilter);
		return TRUE;
	}
};

// ---------------------------------------------------------------------------

typedef std::vector<CWindowSetting> WINDOW_SETTING_ARRAY;


// ---------------------------------------------------------------------------
// 
// ---------------------------------------------------------------------------
class CQuickKeyItem {
private:
	xstr_t m_strFilterID;
	DWORD m_dwHotKey;

public:
	CQuickKeyItem() {
		m_dwHotKey = 0;
	}
	CQuickKeyItem(LPCTSTR pszFilterID, DWORD dwHotKey) {
		m_strFilterID = pszFilterID;
		m_dwHotKey = dwHotKey;
	}
	CQuickKeyItem(const CQuickKeyItem& key) {
		m_strFilterID = key.m_strFilterID;
		m_dwHotKey = key.m_dwHotKey;
	}
	virtual ~CQuickKeyItem() {
	}

	void SetFilterID(LPCTSTR pszFilterID) {
		m_strFilterID = pszFilterID;
	}
	xstr_t GetFilterID() {
		return m_strFilterID;
	}
	void SetHotKey(DWORD dwHotKey) {
		m_dwHotKey = dwHotKey;
	}
	DWORD GetHotKey() {
		return m_dwHotKey;
	}
};

// ---------------------------------------------------------------------------

typedef std::vector<CQuickKeyItem> QUICKKEY_ITEM_ARRAY;


// ---------------------------------------------------------------------------
// application profile class
// ---------------------------------------------------------------------------
class CAppProfile {
private:
	RECT m_rcViewPanel;
	BOOL m_bNoSplash;
	QUICKKEY_ITEM_ARRAY m_QuickKeyList;
	DWORD m_dwViewHotKey;
	BOOL m_bViewAdjustInitialPanelHeight;
	UINT m_nQuickWaitingTime;
	BOOL m_bFirstRun;
	XSTR_ARRAY m_EnableFilterList;
	UINT m_nWindowUpdateInterval;
	UINT m_nWindowColorDisplayFmt;
	BOOL m_bWindowUnvisibleTaskBar;
	BOOL m_bWindowSimpleStatusbarInfo;
	BOOL m_bWindowShowOriginalColorInfo;
	BOOL m_bWindowLeftTopOrigin;
	CWindowSetting m_WindowSetting;
	WINDOW_SETTING_ARRAY m_MultiWindowSettings;
	xstr_t m_strCopyColorFormat;
	DWORD m_dwExtendColor;

public:
	CAppProfile() {
		ZeroMemory(&m_rcViewPanel, sizeof(m_rcViewPanel));
		m_bNoSplash = FALSE;

		m_dwViewHotKey = 0;
		m_bViewAdjustInitialPanelHeight = TRUE;

		m_nQuickWaitingTime = 1000;
		m_bFirstRun = FALSE;
		m_nWindowUpdateInterval = 100;
		m_nWindowColorDisplayFmt = RGB_DISPLAY_TYPE_RGB_DEC;
		m_bWindowUnvisibleTaskBar = FALSE;
		m_bWindowSimpleStatusbarInfo = FALSE;
		m_bWindowShowOriginalColorInfo = FALSE;
		m_bWindowLeftTopOrigin = FALSE;
		m_dwExtendColor = 0;
	}
	CAppProfile(const CAppProfile& profile) {
		int i;
		m_rcViewPanel = profile.m_rcViewPanel;
		m_bNoSplash = profile.m_bNoSplash;

		for (i = 0; i < (int)profile.m_QuickKeyList.size(); i++) {
			m_QuickKeyList.push_back(profile.m_QuickKeyList[i]);
		}

		m_dwViewHotKey = profile.m_dwViewHotKey;
		m_bViewAdjustInitialPanelHeight = profile.m_bViewAdjustInitialPanelHeight;

		m_nQuickWaitingTime = profile.m_nQuickWaitingTime;
		m_bFirstRun = profile.m_bFirstRun;
		for (i = 0; i < (int)profile.m_EnableFilterList.size(); i++) {
			m_EnableFilterList.push_back(profile.m_EnableFilterList[i]);
		}
		m_nWindowUpdateInterval = profile.m_nWindowUpdateInterval;
		m_nWindowColorDisplayFmt = profile.m_nWindowColorDisplayFmt;
		m_bWindowUnvisibleTaskBar = profile.m_bWindowUnvisibleTaskBar;
		m_bWindowSimpleStatusbarInfo = profile.m_bWindowSimpleStatusbarInfo;
		m_bWindowShowOriginalColorInfo = profile.m_bWindowShowOriginalColorInfo;
		m_bWindowLeftTopOrigin = profile.m_bWindowLeftTopOrigin;

		m_WindowSetting = profile.m_WindowSetting;
		m_MultiWindowSettings.clear();
		for (i = 0; i < (int)profile.m_MultiWindowSettings.size(); i++) {
			m_MultiWindowSettings.push_back(profile.m_MultiWindowSettings[i]);
		}
		m_dwExtendColor = profile.m_dwExtendColor;
	}
	virtual ~CAppProfile() {
	}

	void SetViewPanelRect(RECT& rc) {
		m_rcViewPanel = rc;
	}
	RECT& GetViewPanelRect() {
		return m_rcViewPanel;
	}
	void SetNoSplash(BOOL bEnable) {
		m_bNoSplash = bEnable;
	}
	BOOL IsNoSplash() {
		return m_bNoSplash;
	}

	void SetFirstRun(BOOL bEnable) {
		m_bFirstRun = bEnable;
	}
	BOOL IsFirstRun() {
		return m_bFirstRun;
	}

	void SetQuickKeys(const QUICKKEY_ITEM_ARRAY& QuickKeyList) {
		m_QuickKeyList.clear();
		int i;
		for (i = 0; i < (int)QuickKeyList.size(); ++i) {
			m_QuickKeyList.push_back(QuickKeyList[i]);
		}
	}
	QUICKKEY_ITEM_ARRAY& GetQuickKeys() {
		return m_QuickKeyList;
	}

	void SetViewHotKey(DWORD dwHotKey) {
		m_dwViewHotKey = dwHotKey;
	}
	DWORD GetViewHotKey() {
		return m_dwViewHotKey;
	}
	void SetViewAdjustInitialPanelHeight(BOOL bEnable) {
		m_bViewAdjustInitialPanelHeight = bEnable;
	}
	BOOL IsViewAdjustInitialPanelHeight() {
		return m_bViewAdjustInitialPanelHeight;
	}
	void SetQuickWaitingTime(UINT nTime) {
		m_nQuickWaitingTime = nTime;
	}
	UINT GetQuickWaitingTime() {
		return m_nQuickWaitingTime;
	}
	void InitEnableFilter() {
		m_EnableFilterList.clear();
	}
	int GetEnableFilterCount() {
		return m_EnableFilterList.size();
	}
	void SetEnableFilter(LPCTSTR pszID) {
		m_EnableFilterList.push_back(pszID);
	}
	LPCTSTR GetEnableFilter(int index) {
		if (index < 0 || index >= (int)m_EnableFilterList.size()) {
			return NULL;
		}
		return m_EnableFilterList[index];
	}
	void SetEnableFilterList(XSTR_ARRAY& FilterList) {
		m_EnableFilterList.clear();
		int i;
		for (i = 0; i < (int)FilterList.size(); i++) {
			m_EnableFilterList.push_back(FilterList[i]);
		}
	}
	XSTR_ARRAY& GetEnableFilterList() {
		return m_EnableFilterList;
	}
	void SetWindowUpdateInterval(UINT nInterval) {
		m_nWindowUpdateInterval = nInterval;
	}
	UINT GetWindowUpdateInterval() {
		return m_nWindowUpdateInterval;
	}
	void SetWindowColorDisplayFormat(UINT nType) {
		if (nType >= 0 && nType < RGB_DISPLAY_TYPE_MAX) {
			m_nWindowColorDisplayFmt = nType;
		}
	}
	UINT GetWindowColorDisplayFormat() {
		if (m_nWindowColorDisplayFmt >= 0 && m_nWindowColorDisplayFmt < RGB_DISPLAY_TYPE_MAX) {
			return m_nWindowColorDisplayFmt;
		}
		return 0;
	}
	void SetWindowUnvisibleTaskBar(BOOL bEnable) {
		m_bWindowUnvisibleTaskBar = bEnable;
	}
	BOOL GetWindowUnvisibleTaskBar() {
		return m_bWindowUnvisibleTaskBar;
	}

	void SetWindowSimpleStatusbarInfo(BOOL bEnable) {
		m_bWindowSimpleStatusbarInfo = bEnable;
	}
	BOOL IsWindowSimpleStatusbarInfo() {
		return m_bWindowSimpleStatusbarInfo;
	}
	void SetWindowShowOriginalColorInfo(BOOL bEnable) {
		m_bWindowShowOriginalColorInfo = bEnable;
	}
	BOOL IsWindowShowOriginalColorInfo() {
		return m_bWindowShowOriginalColorInfo;
	}
	void SetWindowLeftTopOrigin(BOOL bEnable) {
		m_bWindowLeftTopOrigin = bEnable;
	}
	BOOL IsWindowLeftTopOrigin() {
		return m_bWindowLeftTopOrigin;
	}

	void SetCopyColorFormat(LPCTSTR pszFormat) {
		m_strCopyColorFormat = pszFormat;
	}
	xstr_t GetCopyColorFormat() {
		return m_strCopyColorFormat;
	}
	void SetExtendColor(DWORD dwExtendColor) {
		if (dwExtendColor > MAX_COLOR_EXTEND) {
			return;
		}
		m_dwExtendColor = dwExtendColor;
	}
	DWORD GetExtendColor() {
		return m_dwExtendColor;
	}

	void SetWindowSetting(const CWindowSetting& setting) {
		m_WindowSetting = setting;
	}
	CWindowSetting& GetWindowSetting() {
		return m_WindowSetting;
	}
	void ClearMultiWindowSettings() {
		m_MultiWindowSettings.clear();
	}
	void AddToMultiWindowSetting(const CWindowSetting& setting) {
		m_MultiWindowSettings.push_back(setting);
	}
	int GetMultiWindowSettingCount() {
		return (int)m_MultiWindowSettings.size();
	}
	CWindowSetting GetMultiWindowSetting(int nIndex) {
		if (nIndex < 0 || nIndex >= (int)m_MultiWindowSettings.size()) {
			// error
		}
		return m_MultiWindowSettings[nIndex];
	}

	BOOL ReadProfile();
	BOOL WriteProfile();
};
