// ===========================================================================
// SplashWnd.cpp
// splash window
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include "common.h"
#include "AnotherVision.h"
#include "SplashWnd.h"

// ---------------------------------------------------------------------------

#define SPLASH_TIMER_ID     100
#define SPLASH_TIMER_ELAPSE 2500


// ---------------------------------------------------------------------------
// show splash window
// ---------------------------------------------------------------------------
void ShowSplashWnd()
{
	DWORD dwStyle = WS_VISIBLE | WS_POPUP;
	DWORD dwExStyle = WS_EX_TOOLWINDOW | WS_EX_TOPMOST;
	if (IsWin2K()) {
		dwExStyle |= WS_EX_NOACTIVATE;
	}
	CSplashWnd* pSplashWnd = new CSplashWnd();
	if (pSplashWnd->Create(NULL, NULL, NULL, dwStyle, dwExStyle, (HMENU)NULL, NULL) == NULL) {
		// error
		delete pSplashWnd;
	}
}

// ---------------------------------------------------------------------------
// OnCreate
// ---------------------------------------------------------------------------
LRESULT CSplashWnd::OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	m_hBitmap = LoadBitmap(GetSystemResourceHandle(), MAKEINTRESOURCE(IDB_LOGO));
	if (m_hBitmap == NULL) {
		return -1;
	}
	GetObject(m_hBitmap, sizeof(m_BitmapInfo), (PTSTR)&m_BitmapInfo);
	MoveWindow(0, 0, m_BitmapInfo.bmWidth, m_BitmapInfo.bmHeight, FALSE);
	CenterWindow(NULL);
	m_uTimerID = SetTimer(SPLASH_TIMER_ID, SPLASH_TIMER_ELAPSE, NULL);
	return 0;
}

// ---------------------------------------------------------------------------
// OnClose
// ---------------------------------------------------------------------------
LRESULT CSplashWnd::OnClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	DestroyWindow();
	return 0;
}

// ---------------------------------------------------------------------------
// OnDestroy
// ---------------------------------------------------------------------------
LRESULT CSplashWnd::OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (m_hBitmap != NULL) {
		DeleteObject(m_hBitmap);
		m_hBitmap = NULL;
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnTimer
// ---------------------------------------------------------------------------
LRESULT CSplashWnd::OnTimer(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	if (wParam == SPLASH_TIMER_ID) {
		PostMessage(WM_CLOSE);
		KillTimer(SPLASH_TIMER_ID);
	}
	return 0;
}

// ---------------------------------------------------------------------------
// OnPaint
// ---------------------------------------------------------------------------
LRESULT CSplashWnd::OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint(&ps);
	DrawState(hdc, (HBRUSH)GetStockObject(GRAY_BRUSH), NULL,
			(LPARAM)m_hBitmap, 0,
			0, 0, m_BitmapInfo.bmWidth, m_BitmapInfo.bmHeight,
			DST_BITMAP);
	EndPaint(&ps);
	return 0;
}

// ---------------------------------------------------------------------------
// OnEraseBkGnd
// ---------------------------------------------------------------------------
LRESULT CSplashWnd::OnEraseBkGnd(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
	return 0;
}

// ---------------------------------------------------------------------------
// OnFinalMessage
// ---------------------------------------------------------------------------
void CSplashWnd::OnFinalMessage(HWND /*hWnd*/)
{
	delete this;
}
