// ===========================================================================
// common.cpp
// common utility
//
//                                                       by kanegon 2006
// ===========================================================================

#include "stdafx.h"
#include <commdlg.h>
#include <stdio.h>
#include <stdlib.h>
#include "AnotherVision.h"
#include "common.h"


// ---------------------------------------------------------------------------

#define OSTYPE_UNKNOWN  0
#define OSTYPE_9X       1 // 95,98,Me
#define OSTYPE_NT4      2 // NT4
#define OSTYPE_2000     3 // 2000
#define OSTYPE_XP       4 // XP
#define OSTYPE_2003     5 // 2003
#define OSTYPE_VISTA    6 // Vista or later

static DWORD s_dwOSType = OSTYPE_UNKNOWN;


// ---------------------------------------------------------------------------
// get application directory
// enable last '\\'
// ---------------------------------------------------------------------------
LPCTSTR GetModuleDir()
{
	static TCHAR szModuleDir[_MAX_PATH] = _T("");

	if (szModuleDir[0] != '\0') {
		return szModuleDir;
	}
	TCHAR szExePath[_MAX_PATH];
	GetModuleFileName(GetModuleHandle(NULL), szExePath, sizeof(szExePath));
	LPTSTR psz = szExePath;
	LPTSTR pszBackslash = NULL;
	while (*psz != '\0') {
		if (*psz == '\\') {
			pszBackslash = psz;
		}
		psz = CharNext(psz);
	}
	*(pszBackslash + 1) = '\0';
	lstrcpy(szModuleDir, szExePath);
#if defined(_DEBUG)
	PathAppend(szModuleDir, _T("..\\"));
#endif
	return szModuleDir;
}

// ---------------------------------------------------------------------------
// check file
// ---------------------------------------------------------------------------
BOOL IsFile(LPCTSTR pszPath)
{
	WIN32_FIND_DATA find;
	BOOL bResult = FALSE;
	HANDLE hFind = FindFirstFile(pszPath, &find);
	if (hFind == INVALID_HANDLE_VALUE) {
		return bResult;
	}
	bResult = ! IsFlag(find.dwFileAttributes, FILE_ATTRIBUTE_DIRECTORY);
	FindClose(hFind);
	return bResult;
}

// ---------------------------------------------------------------------------
// check dir
// ---------------------------------------------------------------------------
BOOL IsDirectory(LPCTSTR pszPath)
{
	WIN32_FIND_DATA find;
	BOOL bResult = FALSE;
	HANDLE hFind = FindFirstFile(pszPath, &find);
	if (hFind == INVALID_HANDLE_VALUE) {
		return bResult;
	}
	bResult = IsFlag(find.dwFileAttributes, FILE_ATTRIBUTE_DIRECTORY);
	FindClose(hFind);
	return bResult;
}

// ---------------------------------------------------------------------------
// read file
// ---------------------------------------------------------------------------
BOOL ExReadFile(LPCTSTR pszPath, LPBYTE* ppBuffer, DWORD* pdwBufferSize)
{
	HANDLE hFile;
	DWORD dwSize;
	LPBYTE pBuffer;
	DWORD dwRead;

	if (pszPath == NULL || ppBuffer == NULL || pdwBufferSize == NULL)
		return FALSE;
	*ppBuffer = NULL;
	*pdwBufferSize = 0;

	hFile = CreateFile(pszPath, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
		return FALSE;
	dwSize = GetFileSize(hFile, NULL);
	if (dwSize == 0xFFFFFFFF) {
		CloseHandle(hFile);
		return FALSE;
	}
	pBuffer = new BYTE[dwSize + 1];
	dwRead = 0;
	if (dwSize != 0) {
		BOOL bResult = ReadFile(hFile, pBuffer, dwSize, &dwRead, NULL);
		if (! bResult || dwSize != dwRead) {
			delete[] pBuffer;
			CloseHandle(hFile);
			return FALSE;
		}
	}
	CloseHandle(hFile);
	pBuffer[dwSize] = '\0';

	*ppBuffer = pBuffer;
	*pdwBufferSize = dwSize;

	return TRUE;
}

// ---------------------------------------------------------------------------
// write file
// ---------------------------------------------------------------------------
BOOL ExWriteFile(LPCTSTR pszPath, LPBYTE pBuffer, DWORD dwBufferSize)
{
	HANDLE hFile;
	DWORD dwWrote;
	BOOL bResult;

	if (pszPath == NULL || pBuffer == NULL)
		return FALSE;

	hFile = CreateFile(pszPath, GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
	if (hFile == INVALID_HANDLE_VALUE)
		return FALSE;
	dwWrote = 0;
	bResult = WriteFile(hFile, pBuffer, dwBufferSize, &dwWrote, NULL);
	if (! bResult || dwBufferSize != dwWrote) {
		CloseHandle(hFile);
		return FALSE;
	}
	CloseHandle(hFile);

	return TRUE;
}

// ---------------------------------------------------------------------------
// check OS type
// ---------------------------------------------------------------------------
static BOOL CheckOSType()
{
	OSVERSIONINFO os;
	ZeroMemory(&os, sizeof(os));
	os.dwOSVersionInfoSize = sizeof(os);

	s_dwOSType = OSTYPE_UNKNOWN;
	if (GetVersionEx(&os)) {
		switch (os.dwPlatformId) {
		case VER_PLATFORM_WIN32_NT:
			if (os.dwMajorVersion >= 6) {
				s_dwOSType = OSTYPE_VISTA;		// Vista or later
			}
			else if (os.dwMajorVersion == 5) {
				if (os.dwMinorVersion >= 2) {
					s_dwOSType = OSTYPE_2003;
				}
				else if (os.dwMinorVersion == 1) {
					s_dwOSType = OSTYPE_XP;
				}
				else {
					s_dwOSType = OSTYPE_2000;
				}
			} else {
				s_dwOSType = OSTYPE_NT4;
			}
			break;
		case VER_PLATFORM_WIN32_WINDOWS:
			s_dwOSType = OSTYPE_9X;
#if 0
			if (os.dwMajorVersion == 4 && os.dwMinorVersion == 0) {
				// 95
				if (os.szCSDVersion[1] == 'C' || os.szCSDVersion[1] == 'B') {
					// OSR2
				}
			}
			if (os.dwMajorVersion == 4 && os.dwMinorVersion == 10) {
				// 98
				if (os.szCSDVersion[1] == 'A') {
					// SE
				}
			}
			if (os.dwMajorVersion == 4 && os.dwMinorVersion == 90) {
				// Me
			}
#endif
			break;
		}
	}
	if (s_dwOSType == OSTYPE_UNKNOWN) {
		return FALSE;
	}
	return TRUE;
}

// ---------------------------------------------------------------------------
// check Windows NT Platform
// ---------------------------------------------------------------------------
BOOL IsWinNT()
{
	if (s_dwOSType == OSTYPE_UNKNOWN) {
		CheckOSType();
	}
	return (s_dwOSType == OSTYPE_NT4 || s_dwOSType == OSTYPE_2000 ||
			s_dwOSType == OSTYPE_XP  || s_dwOSType == OSTYPE_2003 ||
			s_dwOSType == OSTYPE_VISTA);
}

// ---------------------------------------------------------------------------
// check Windows 2000 or later
// ---------------------------------------------------------------------------
BOOL IsWin2K()
{
	if (s_dwOSType == OSTYPE_UNKNOWN) {
		CheckOSType();
	}
	return (s_dwOSType == OSTYPE_2000 ||
			s_dwOSType == OSTYPE_XP   || s_dwOSType == OSTYPE_2003 ||
			s_dwOSType == OSTYPE_VISTA);
}

// ---------------------------------------------------------------------------
// check Windows XP/2003 or later
// ---------------------------------------------------------------------------
BOOL IsWinXP()
{
	if (s_dwOSType == OSTYPE_UNKNOWN) {
		CheckOSType();
	}
	return (s_dwOSType == OSTYPE_XP || s_dwOSType == OSTYPE_2003 ||
			s_dwOSType == OSTYPE_VISTA);
}

// ---------------------------------------------------------------------------
// check Windows Vista or later
// ---------------------------------------------------------------------------
BOOL IsWinVista()
{
	if (s_dwOSType == OSTYPE_UNKNOWN) {
		CheckOSType();
	}
	return (s_dwOSType == OSTYPE_VISTA);
}

// ---------------------------------------------------------------------------
// read text value from registry
// ---------------------------------------------------------------------------
BOOL ReadRegText(LPCTSTR pszRegKey, LPCTSTR pszRegName, LPTSTR pszValue, DWORD dwValueSize, LPCTSTR pszDefault)
{
	HKEY hKey;

	LONG lResult = RegOpenKeyEx(HKEY_CURRENT_USER, pszRegKey, 0, KEY_ALL_ACCESS, &hKey);
	if (lResult == ERROR_SUCCESS) {
		DWORD dwReadSize = dwValueSize;
		DWORD dwType = 0;
		lResult = RegQueryValueEx(hKey, pszRegName, NULL, &dwType, (LPBYTE)pszValue, &dwReadSize);
		RegCloseKey(hKey);
		if (lResult == ERROR_SUCCESS &&
			dwType == REG_SZ) {
			return TRUE;
		}
	}
	lstrcpy(pszValue, pszDefault);
	return TRUE;
}

// ---------------------------------------------------------------------------
// read int value from registry
// ---------------------------------------------------------------------------
BOOL ReadRegInt(LPCTSTR pszRegKey, LPCTSTR pszRegName, DWORD* pdwValue, DWORD dwDefaultValue)
{
	HKEY hKey;

	LONG lResult = RegOpenKeyEx(HKEY_CURRENT_USER, pszRegKey, 0, KEY_ALL_ACCESS, &hKey);
	if (lResult == ERROR_SUCCESS) {
		DWORD dwReadSize = sizeof(DWORD);
		DWORD dwType = 0;
		lResult = RegQueryValueEx(hKey, pszRegName, NULL, &dwType, (LPBYTE)pdwValue, &dwReadSize);
		RegCloseKey(hKey);
		if (lResult == ERROR_SUCCESS &&
			dwType == REG_DWORD) {
			return TRUE;
		}
	}
	*pdwValue = dwDefaultValue;
	return TRUE;
}

// ---------------------------------------------------------------------------
// write text value to registry
// ---------------------------------------------------------------------------
VOID WriteRegText(LPCTSTR pszRegKey, LPCTSTR pszRegName, LPCTSTR pszValue)
{
	HKEY hKey;

	if (RegCreateKeyEx(HKEY_CURRENT_USER, pszRegKey, 0, _T(""), REG_OPTION_NON_VOLATILE,
				KEY_ALL_ACCESS, NULL, &hKey, NULL) == ERROR_SUCCESS) {
		RegSetValueEx(hKey, pszRegName, 0, REG_SZ, (LPBYTE)pszValue, (lstrlen(pszValue) + 1) * sizeof(TCHAR));
		RegCloseKey(hKey);
	}
}

// ---------------------------------------------------------------------------
// write int value to registry
// ---------------------------------------------------------------------------
VOID WriteRegInt(LPCTSTR pszRegKey, LPCTSTR pszRegName, DWORD dwValue)
{
	HKEY hKey;

	if (RegCreateKeyEx(HKEY_CURRENT_USER, pszRegKey, 0, _T(""), REG_OPTION_NON_VOLATILE,
				KEY_ALL_ACCESS, NULL, &hKey, NULL) == ERROR_SUCCESS) {
		RegSetValueEx(hKey, pszRegName, 0, REG_DWORD, (LPBYTE)&dwValue, sizeof(DWORD));
		RegCloseKey(hKey);
	}
}

// ---------------------------------------------------------------------------
// validate HRESULT and throw exception
// ---------------------------------------------------------------------------
void ValidateHResult(HRESULT hr)
{
	if FAILED(hr) _com_issue_error(hr);
};

// ---------------------------------------------------------------------------
// trim text
// ---------------------------------------------------------------------------
void TrimText(LPTSTR pszText)
{
	if (pszText == NULL) {
		return;
	}
	LPCTSTR pszTextFirst = NULL;
	LPCTSTR pszTextLast = NULL;
	LPCTSTR psz = pszText;
	while (*psz != '\0') {
		if (! _istspace((BYTE)*psz)) {
			if (pszTextFirst == NULL) {
				pszTextFirst = psz;
			}
			pszTextLast = psz;
		}
		psz++;
	}
	if (pszTextFirst == NULL) {
		return;
	}
	int nLen = pszTextLast - pszTextFirst + 1;
	if (pszTextFirst != pszText) {
		memmove(pszText, pszTextFirst, nLen);
	}
	pszText[nLen] = '\0';
}

// ---------------------------------------------------------------------------
// set window foreground
// ---------------------------------------------------------------------------
BOOL ExSetForegroundWindow(HWND hwnd)
{
	if (hwnd== NULL) {
		return FALSE;
	}
	if (! IsWindow(hwnd)) {
		return FALSE;
	}
	HWND hwndForeground = ::GetForegroundWindow();
	if (hwndForeground == NULL) {
		return ::SetForegroundWindow(hwnd);
	}
	DWORD dwForegroundProcessID;
	DWORD dwForegroundThreadID = ::GetWindowThreadProcessId(hwndForeground, &dwForegroundProcessID);
	if (dwForegroundProcessID == GetCurrentProcessId()) {
		return ::SetForegroundWindow(hwnd);
	}
	DWORD dwTargetThreadID = ::GetWindowThreadProcessId(hwnd, NULL);
	BOOL bResult = FALSE;
	if (::AttachThreadInput(dwTargetThreadID, dwForegroundThreadID, TRUE)) {
		bResult = ::SetForegroundWindow(hwnd);
		AttachThreadInput(dwTargetThreadID, dwForegroundThreadID, FALSE);
	}
	return bResult;
}

// ---------------------------------------------------------------------------
// execute external application
// ---------------------------------------------------------------------------
BOOL ExecCmd(LPCTSTR pszAppPath)
{
	STARTUPINFO si;
	ZeroMemory(&si, sizeof(si));
	si.cb = sizeof(si);
	PROCESS_INFORMATION pi;
	ZeroMemory(&pi,sizeof(pi));
	BOOL bResult = CreateProcess(
					pszAppPath,
					NULL,
					NULL,
					NULL,
					FALSE,
					0,
					NULL,
					NULL,
					&si,
					&pi);

	if (! bResult) {
		return FALSE;
	}
	CloseHandle(pi.hThread);
	CloseHandle(pi.hProcess);
	return TRUE;
}

// ---------------------------------------------------------------------------
// read product version from resource of Windows .exe/.dll file
// ---------------------------------------------------------------------------
DWORD GetProductVersionMS(LPCTSTR pszPath)
{
	DWORD dwDummy = 0;
	DWORD dwSize = GetFileVersionInfoSize((LPTSTR)pszPath, &dwDummy);
	if (dwSize == 0) {
		return (DWORD)-1;
	}
	BYTE* pBuffer = new BYTE[dwSize];
	if (! GetFileVersionInfo((LPTSTR)pszPath, 0, dwSize, pBuffer)) {
		delete[] pBuffer;
		return (DWORD)-1;
	}
	VS_FIXEDFILEINFO* pVer;
	UINT nVer;
	if (! VerQueryValue(pBuffer, _T("\\"), (LPVOID*)&pVer, &nVer)) {
		delete[] pBuffer;
		return (DWORD)-1;
	}
	DWORD dwProductVersionMS = pVer->dwProductVersionMS;
	delete[] pBuffer;

	return dwProductVersionMS;
}

// ---------------------------------------------------------------------------
// load string from resource
// ---------------------------------------------------------------------------
BOOL ExLoadString(UINT nID, LPTSTR pszBuffer, int nBufferSize)
{
	ATLASSERT(pszBuffer != NULL);
	pszBuffer[0] = '\0';
	if (LoadString(GetResourceHandle(), nID, pszBuffer, nBufferSize)) {
		return TRUE;
	}
	if (LoadString(GetSystemResourceHandle(), nID, pszBuffer, nBufferSize)) {
		return TRUE;
	}
	ExMessageBoxText(MB_OK | MB_ICONSTOP, MSG_ERROR_RESOURCE_MISSING, nID);
	SetFatalError();
	return FALSE;
}

// ---------------------------------------------------------------------------
// copy text to clipboard
// ---------------------------------------------------------------------------
void CopyClipboard(LPCTSTR pszText)
{
	if (! OpenClipboard(NULL)) {
		return;
	}
	if (! EmptyClipboard()) {
		CloseClipboard();
		return;
	}
	HGLOBAL hClipData = GlobalAlloc(GMEM_MOVEABLE | GMEM_DDESHARE, 
			sizeof(TCHAR) * (lstrlen(pszText) + 1));
	PTSTR pClipData = (PTSTR)GlobalLock(hClipData);
	lstrcpy(pClipData, pszText);
#ifdef UNICODE
	BOOL bSuccess = (SetClipboardData(CF_UNICODETEXT, hClipData) == hClipData);
#else
	BOOL bSuccess = (SetClipboardData(CF_TEXT, hClipData) == hClipData);
#endif
	CloseClipboard();
	if (! bSuccess) {
		GlobalFree(hClipData);
	}
}

// ---------------------------------------------------------------------------
// message box wrapper (text)
// ---------------------------------------------------------------------------
int ExMessageBoxText(UINT nType, LPCTSTR pszFmt, ...)
{
	ATLASSERT(pszFmt != NULL);
	TCHAR szText[TEXT_BUFFER_SIZE];
	va_list vaPtr;

	if ((nType & 0x000000F0L) == MB_ICONSTOP) {
#if 0
		if (IsFatalError() &&
			(nType & 0x0000000FL) == MB_OK) {
			return MB_OK;
		}
#endif
		SetFatalError();
	}
	ZeroMemory(szText, sizeof(szText));
	va_start(vaPtr, pszFmt);
	_vsntprintf(szText, sizeof(szText) - 1, pszFmt, vaPtr);
	va_end(vaPtr);

	return ::MessageBox(NULL, szText, APP_WINDOW_TITLE, nType);
}

// ---------------------------------------------------------------------------
// message box wrapper (resource)
// ---------------------------------------------------------------------------
int ExMessageBox(UINT nType, UINT nID, ...)
{
	TCHAR szRes[TEXT_BUFFER_SIZE];
	TCHAR szText[TEXT_BUFFER_SIZE];
	va_list vaPtr;

	if ((nType & 0x000000F0L) == MB_ICONSTOP) {
#if 0
		if (IsFatalError() &&
			(nType & 0x0000000FL) == MB_OK) {
			return MB_OK;
		}
#endif
		SetFatalError();
	}

	ZeroMemory(szRes, sizeof(szRes));
	ZeroMemory(szText, sizeof(szText));
	if (! ExLoadString(nID, szRes, sizeof(szRes))) {
		//TODO: error handling
		return IDOK;
	}
	va_start(vaPtr, nID);
	_vsntprintf(szText, sizeof(szText) - 1, szRes, vaPtr);
	va_end(vaPtr);

	return ::MessageBox(NULL, szText, APP_WINDOW_TITLE, nType);
}

