// ===========================================================================
// xstr.cpp
//
// simple string
//
//                                                       by kanegon 2003-2006
// ===========================================================================

#include <windows.h>
#include <comdef.h>
#include <tchar.h>
#include <new>  // use std::bad_alloc
#include <stdio.h>
#include "xstr.h"


//////////////////////////////////////////////////////////////////////////////
// helper function

BSTR mb2wstr(const char* psz)
{
	if (psz == NULL) {
		return NULL;
	}
	int len = MultiByteToWideChar(CP_ACP, 0, psz, -1, NULL, 0);
	if (len == 0) {
		return NULL;
	}
	BSTR bstr = SysAllocStringLen(NULL, len - 1);
	if (bstr == NULL) {
		SetLastError(ERROR_NOT_ENOUGH_MEMORY);
		return NULL;
	}
	len = MultiByteToWideChar(CP_ACP, 0, psz, -1, bstr, len);
	if (len == 0) {
		return NULL;
	}
    return bstr;
}

char* wstr2mb(const wchar_t* pwsz) {
	if (pwsz == NULL) {
		return NULL;
	}
	int len = WideCharToMultiByte(CP_ACP, 0, pwsz, -1, NULL, 0, NULL, NULL);
	if (len == 0) {
		return NULL;
	}
	char* psz = new char[len];
	if (psz == NULL) {
		SetLastError(ERROR_NOT_ENOUGH_MEMORY);
		return NULL;
	}
	len = WideCharToMultiByte(CP_ACP, 0, pwsz, -1, psz, len, NULL, NULL);
	if (len == 0) {
		return NULL;
	}
	return psz;
}


//////////////////////////////////////////////////////////////////////////////
// xastr_t

xastr_t::xastr_t()
{
	m_pszBuffer = NULL;
	m_bAllowNull = FALSE;
}

xastr_t::xastr_t(const char* psz, BOOL bAllowNull/*= TRUE*/)
{
	assign(psz);
	m_bAllowNull = bAllowNull;
}

xastr_t::xastr_t(const xastr_t& str)
{
	if (str.m_pszBuffer == NULL) {
		m_pszBuffer = NULL;
	} else {
		m_pszBuffer = new char[lstrlenA(str.m_pszBuffer) + 1];
		if (m_pszBuffer == NULL) {
			throw std::bad_alloc();
		}
		lstrcpyA(m_pszBuffer, str.m_pszBuffer);
	}
	m_bAllowNull = str.m_bAllowNull;
}

xastr_t::~xastr_t()
{
	clearBuffer();
}

void xastr_t::clearBuffer()
{
	if (m_pszBuffer != NULL) {
		delete[] m_pszBuffer;
	}
	m_pszBuffer = NULL;
}

void xastr_t::assign(const char* psz)
{
	if (psz == NULL) {
		m_pszBuffer = NULL;
	} else {
		m_pszBuffer = new char[lstrlenA(psz) + 1];
		if (m_pszBuffer == NULL) {
			throw std::bad_alloc();
		}
		lstrcpyA(m_pszBuffer, psz);
	}
}

xastr_t& xastr_t::operator=(const char* psz)
{
	if (psz == m_pszBuffer) {
		return *this;
	}
	clearBuffer();
	assign(psz);
	return *this;
}

xastr_t& xastr_t::operator=(const xastr_t& str)
{
	if (str.m_pszBuffer == m_pszBuffer) {
		return *this;
	}
	// m_bAllowNull attribute is not changed.
	clearBuffer();
	assign(str.m_pszBuffer);
	return *this;
}

xastr_t::operator const char*() const
{
	if (m_pszBuffer == NULL && ! m_bAllowNull) {
		return "";
	}
	return m_pszBuffer;
}

xastr_t xastr_t::operator+(const char* psz)
{
	xastr_t strNew(*this);
	strNew.append(psz);
	return strNew;
}

xastr_t& xastr_t::operator+=(const char* psz)
{
	append(psz);
	return *this;
}

int xastr_t::length() const
{
	return (m_pszBuffer != NULL) ? lstrlenA(m_pszBuffer) : 0;
}

// notice: pointer life is temporary.
char* xastr_t::allocBuffer(int nSize)
{
	clearBuffer();
	m_pszBuffer = new char[nSize + 1];
	if (m_pszBuffer == NULL) {
		throw std::bad_alloc();
	}
	ZeroMemory(m_pszBuffer, nSize + 1);
	return m_pszBuffer;
}

char* xastr_t::getBuffer()
{
	return m_pszBuffer;
}

xastr_t& xastr_t::append(const char* psz)
{
	if (psz == NULL) {
		return *this;
	}
	int newlen = length() + lstrlenA(psz) + 1;
	char* pszNewBuffer = new char[newlen];
	if (pszNewBuffer == NULL) {
		throw std::bad_alloc();
	}
	if (m_pszBuffer != NULL) {
		lstrcpyA(pszNewBuffer, m_pszBuffer);
		lstrcatA(pszNewBuffer, psz);
	}
	else {
		lstrcpyA(pszNewBuffer, psz);
	}
	clearBuffer();
	m_pszBuffer = pszNewBuffer;

	return *this;
}

xastr_t xastr_t::left(int len) const
{
	return left(m_pszBuffer, len);
}

xastr_t xastr_t::right(int len) const
{
	return right(m_pszBuffer, len);
}

xastr_t xastr_t::mid(int index, int len) const
{
	return mid(m_pszBuffer, index, len);
}

xastr_t& xastr_t::trim()
{
	if (m_pszBuffer == NULL) {
		return *this;
	}
	char* psz = m_pszBuffer;
	char* pszTextStart = psz;
	while (*psz != '\0' && isspace((unsigned char)*psz)) {
		psz = CharNextA(psz);
		pszTextStart = psz;
	}
	char* pszTextEnd = psz;
	while (*psz != '\0') {
		char* pszNext = CharNextA(psz);
		if (! isspace((unsigned char)*psz)) {
			pszTextEnd = pszNext;
		}
		psz = pszNext;
	}
	int len = pszTextEnd - pszTextStart;
	char* pszNewBuffer = new char[len + 1];
	if (pszNewBuffer == NULL) {
		throw std::bad_alloc();
	}
	memcpy(pszNewBuffer, pszTextStart, len);
	pszNewBuffer[len] = '\0';
	clearBuffer();
	m_pszBuffer = pszNewBuffer;

	return *this;
}

xastr_t xastr_t::left(const char* psz, int len)
{
	if (psz == NULL || len < 0)
		return "";
	if (len > lstrlenA(psz))
		len = lstrlenA(psz);
	xastr_t str;
	char* pszBuffer = str.allocBuffer(len);
	memcpy(pszBuffer, psz, len);
	pszBuffer[len] = '\0';
	return str;
}

xastr_t xastr_t::right(const char* psz, int len)
{
	if (psz == NULL || len < 0) {
		return "";
	}
	if (len > lstrlenA(psz)) {
		len = lstrlenA(psz);
	}
	xastr_t str;
	char* pszBuffer = str.allocBuffer(len);
	memcpy(pszBuffer, &psz[lstrlenA(psz) - len], len);
	pszBuffer[len] = '\0';
	return str;
}

xastr_t xastr_t::mid(const char* psz, int index, int len)
{
	if (psz == NULL || len < 0) {
		return "";
	}
	int bufferlen = lstrlenA(psz);
	if (index > bufferlen) {
		return "";
	}
	if (index + len > bufferlen) {
		len = bufferlen - index;
	}
	xastr_t str;
	char* pszBuffer = str.allocBuffer(len);
	memcpy(pszBuffer, &psz[index], len);
	pszBuffer[len] = '\0';
	return str;
}

//////////////////////////////////////////////////////////////////////////////
// xwstr_t

xwstr_t::xwstr_t()
{
	m_pszBuffer = NULL;
	m_bAllowNull = FALSE;
}

xwstr_t::xwstr_t(const wchar_t* psz, BOOL bAllowNull/*= TRUE*/)
{
	assign(psz);
	m_bAllowNull = bAllowNull;
}

xwstr_t::xwstr_t(const xwstr_t& str)
{
	if (str.m_pszBuffer == NULL) {
		m_pszBuffer = NULL;
	} else {
		m_pszBuffer = new wchar_t[lstrlenW(str.m_pszBuffer) + 1];
		if (m_pszBuffer == NULL) {
			throw std::bad_alloc();
		}
		lstrcpyW(m_pszBuffer, str.m_pszBuffer);
	}
	m_bAllowNull = str.m_bAllowNull;
}

xwstr_t::~xwstr_t()
{
	clearBuffer();
}

void xwstr_t::clearBuffer()
{
	if (m_pszBuffer != NULL) {
		delete[] m_pszBuffer;
	}
	m_pszBuffer = NULL;
}

void xwstr_t::assign(const wchar_t* psz)
{
	if (psz == NULL) {
		m_pszBuffer = NULL;
	} else {
		m_pszBuffer = new wchar_t[lstrlenW(psz) + 1];
		if (m_pszBuffer == NULL) {
			throw std::bad_alloc();
		}
		lstrcpyW(m_pszBuffer, psz);
	}
}

xwstr_t& xwstr_t::operator=(const wchar_t* psz)
{
	if (psz == m_pszBuffer) {
		return *this;
	}
	clearBuffer();
	assign(psz);
	return *this;
}

xwstr_t& xwstr_t::operator=(const xwstr_t& str)
{
	if (str.m_pszBuffer == m_pszBuffer) {
		return *this;
	}
	// m_bAllowNull attribute is not changed.
	clearBuffer();
	assign(str.m_pszBuffer);
	return *this;
}

xwstr_t::operator const wchar_t*() const
{
	if (m_pszBuffer == NULL && ! m_bAllowNull) {
		return L"";
	}
	return m_pszBuffer;
}

xwstr_t xwstr_t::operator+(const wchar_t* psz)
{
	xwstr_t strNew(*this);
	strNew.append(psz);
	return strNew;
}

xwstr_t& xwstr_t::operator+=(const wchar_t* psz)
{
	append(psz);
	return *this;
}

int xwstr_t::length() const
{
	return (m_pszBuffer != NULL) ? lstrlenW(m_pszBuffer) : 0;
}

// notice: pointer life is temporary.
wchar_t* xwstr_t::allocBuffer(int nSize)
{
	clearBuffer();
	m_pszBuffer = new wchar_t[nSize + 1];
	if (m_pszBuffer == NULL) {
		throw std::bad_alloc();
	}
	ZeroMemory(m_pszBuffer, sizeof(wchar_t) * (nSize + 1));
	return m_pszBuffer;
}

wchar_t* xwstr_t::getBuffer()
{
	return m_pszBuffer;
}

xwstr_t& xwstr_t::append(const wchar_t* psz)
{
	if (psz == NULL) {
		return *this;
	}
	int newlen = length() + lstrlenW(psz) + 1;
	wchar_t* pszNewBuffer = new wchar_t[newlen];
	if (pszNewBuffer == NULL) {
		throw std::bad_alloc();
	}
	if (m_pszBuffer != NULL) {
		lstrcpyW(pszNewBuffer, m_pszBuffer);
		lstrcatW(pszNewBuffer, psz);
	}
	else {
		lstrcpyW(pszNewBuffer, psz);
	}
	clearBuffer();
	m_pszBuffer = pszNewBuffer;

	return *this;
}

xwstr_t xwstr_t::left(int len) const
{
	return left(m_pszBuffer, len);
}

xwstr_t xwstr_t::right(int len) const
{
	return right(m_pszBuffer, len);
}

xwstr_t xwstr_t::mid(int index, int len) const
{
	return mid(m_pszBuffer, index, len);
}

xwstr_t& xwstr_t::trim()
{
	if (m_pszBuffer == NULL) {
		return *this;
	}
	wchar_t* psz = m_pszBuffer;
	wchar_t* pszTextStart = psz;
	while (*psz != '\0' && iswspace(*psz)) {
		++psz;
		pszTextStart = psz;
	}
	wchar_t* pszTextEnd = psz;
	while (*psz != '\0') {
		wchar_t* pszNext = psz+1;
		if (! iswspace(*psz)) {
			pszTextEnd = pszNext;
		}
		psz = pszNext;
	}
	int len = pszTextEnd - pszTextStart;
	wchar_t* pszNewBuffer = new wchar_t[len + 1];
	if (pszNewBuffer == NULL) {
		throw std::bad_alloc();
	}
	memcpy(pszNewBuffer, pszTextStart, len * 2);
	pszNewBuffer[len] = L'\0';
	clearBuffer();
	m_pszBuffer = pszNewBuffer;

	return *this;
}

xwstr_t xwstr_t::left(const wchar_t* psz, int len)
{
	if (psz == NULL || len < 0) {
		return L"";
	}
	if (len > lstrlenW(psz)) {
		len = lstrlenW(psz);
	}
	xwstr_t str;
	wchar_t* pszBuffer = str.allocBuffer(len);
	memcpy(pszBuffer, psz, sizeof(wchar_t) * len);
	pszBuffer[len] = L'\0';
	return str;
}

xwstr_t xwstr_t::right(const wchar_t* psz, int len)
{
	if (psz == NULL || len < 0) {
		return L"";
	}
	if (len > lstrlenW(psz)) {
		len = lstrlenW(psz);
	}
	xwstr_t str;
	wchar_t* pszBuffer = str.allocBuffer(len);
	memcpy(pszBuffer, &psz[lstrlenW(psz) - len], sizeof(wchar_t) * len);
	pszBuffer[len] = L'\0';
	return str;
}

xwstr_t xwstr_t::mid(const wchar_t* psz, int index, int len)
{
	if (psz == NULL || len < 0) {
		return L"";
	}
	int bufferlen = lstrlenW(psz);
	if (index > bufferlen) {
		return L"";
	}
	if (index + len > bufferlen) {
		len = bufferlen - index;
	}
	xwstr_t str;
	wchar_t* pszBuffer = str.allocBuffer(len);
	memcpy(pszBuffer, &psz[index], sizeof(wchar_t) * len);
	pszBuffer[len] = L'\0';
	return str;
}

//////////////////////////////////////////////////////////////////////////////
// xbstr_t

xbstr_t::xbstr_t()
{
	m_pwszBuffer = NULL;
	m_pszConvertText = NULL;
	assign(L"");
	m_pwszBuffer = SysAllocString(L"");
	if (m_pwszBuffer == NULL) {
		throw std::bad_alloc();
	}
}

xbstr_t::xbstr_t(const char* psz)
{
	m_pwszBuffer = NULL;
	m_pszConvertText = NULL;
	assign(psz);
	if (m_pwszBuffer == NULL) {
		throw std::bad_alloc();
	}
}

xbstr_t::xbstr_t(wchar_t* pwsz, BOOL bAttach/*= FALSE*/)
{
	m_pwszBuffer = NULL;
	m_pszConvertText = NULL;
	if (bAttach && pwsz != NULL) {
		// pwsz must be BSTR
		m_pwszBuffer = pwsz;
	} else {
		assign(pwsz);
		if (m_pwszBuffer == NULL) {
			throw std::bad_alloc();
		}
	}
}

xbstr_t::~xbstr_t()
{
	clearBuffer();
}

void xbstr_t::clearBuffer()
{
	if (m_pwszBuffer != NULL) {
		SysFreeString(m_pwszBuffer);
		m_pwszBuffer = NULL;
	}
	if (m_pszConvertText != NULL) {
		delete[] m_pszConvertText;
		m_pszConvertText = NULL;
	}
}

void xbstr_t::assign(const char* psz)
{
	if (m_pwszBuffer != NULL) {
		clearBuffer();
	}
	if (psz == NULL) {
		m_pwszBuffer = SysAllocString(L"");
	} else {
		m_pwszBuffer = mb2wstr(psz);
	}
}

void xbstr_t::assign(const wchar_t* pwsz)
{
	if (m_pwszBuffer != NULL) {
		clearBuffer();
	}
	if (pwsz == NULL) {
		m_pwszBuffer = SysAllocString(L"");
	} else {
		m_pwszBuffer = SysAllocString(pwsz);
	}
}

xbstr_t& xbstr_t::operator=(const char* psz)
{
	if (psz == m_pszConvertText) {
		return *this;
	}
	assign(psz);
	if (m_pwszBuffer == NULL) {
		throw std::bad_alloc();
	}
	return *this;
}

xbstr_t& xbstr_t::operator=(const wchar_t* pwsz)
{
	if (pwsz == m_pwszBuffer) {
		return *this;
	}
	assign(pwsz);
	if (m_pwszBuffer == NULL) {
		throw std::bad_alloc();
	}
	return *this;
}

xbstr_t& xbstr_t::operator=(const xbstr_t& str)
{
	if (str.m_pwszBuffer == m_pwszBuffer) {
		return *this;
	}
	assign(str.m_pwszBuffer);
	if (m_pwszBuffer == NULL) {
		throw std::bad_alloc();
	}
	return *this;
}

xbstr_t::operator const char*()
{
	if (m_pszConvertText == NULL) {
		m_pszConvertText = wstr2mb(m_pwszBuffer);
		if (m_pszConvertText == NULL) {
			throw std::bad_alloc();
		}
	}
	return m_pszConvertText;
}

xbstr_t::operator wchar_t*() const
{
	return m_pwszBuffer;
}

int xbstr_t::length() const
{
	return SysStringLen(m_pwszBuffer);
}
