// ===========================================================================
// xstr.h
//
// simple string
// ANSI(char*) and UNICODE(wchar_t) support
//
//                                                       by kanegon 2003-2006
// ===========================================================================

#ifndef _XSTR_H_
#define _XSTR_H_

#include <windows.h>
#include <comdef.h>


//////////////////////////////////////////////////////////////////////////////
// helper function

BSTR mb2wstr(const char* psz);
char* wstr2mb(const wchar_t* pwsz);


//////////////////////////////////////////////////////////////////////////////
// simple string (char*)

class xastr_t
{
protected:
	char* m_pszBuffer;
	BOOL m_bAllowNull;

public:
	xastr_t();
	xastr_t(const char* psz, BOOL bAllowNull = FALSE);
	xastr_t(const xastr_t& str);
	virtual ~xastr_t();

protected:
	void clearBuffer();
	void assign(const char* psz);

public:
	xastr_t& operator=(const char* psz);
	xastr_t& operator=(const xastr_t& str);
	operator const char*() const;
	xastr_t operator+(const char* psz);
	xastr_t& operator+=(const char* psz);

	int length() const;

	char* allocBuffer(int nSize);
	char* getBuffer();

	xastr_t& append(const char* psz);

	xastr_t left(int len) const;
	xastr_t right(int len) const;
	xastr_t mid(int index, int len) const;

	xastr_t& trim();

	static xastr_t left(const char* psz, int len);
	static xastr_t right(const char* psz, int len);
	static xastr_t mid(const char* psz, int index, int len);
};


//////////////////////////////////////////////////////////////////////////////
// simple string (wchar_t*)

class xwstr_t
{
protected:
	wchar_t* m_pszBuffer;
	BOOL m_bAllowNull;

public:
	xwstr_t();
	xwstr_t(const wchar_t* psz, BOOL bAllowNull = FALSE);
	xwstr_t(const xwstr_t& str);
	virtual ~xwstr_t();

protected:
	void clearBuffer();
	void assign(const wchar_t* psz);

public:
	xwstr_t& operator=(const wchar_t* psz);
	xwstr_t& operator=(const xwstr_t& str);
	operator const wchar_t*() const;
	xwstr_t operator+(const wchar_t* psz);
	xwstr_t& operator+=(const wchar_t* psz);

	int length() const;

	wchar_t* allocBuffer(int nSize);
	wchar_t* getBuffer();

	xwstr_t& append(const wchar_t* psz);

	xwstr_t left(int len) const;
	xwstr_t right(int len) const;
	xwstr_t mid(int index, int len) const;

	xwstr_t& trim();

	static xwstr_t left(const wchar_t* psz, int len);
	static xwstr_t right(const wchar_t* psz, int len);
	static xwstr_t mid(const wchar_t* psz, int index, int len);
};


//////////////////////////////////////////////////////////////////////////////

#ifdef UNICODE
typedef xwstr_t xstr_t;
#else
typedef xastr_t xstr_t;
#endif


//////////////////////////////////////////////////////////////////////////////
// simple string (BSTR)

class xbstr_t {
protected:
	BSTR m_pwszBuffer;
	char* m_pszConvertText;

public:
	xbstr_t();
	xbstr_t(const char* psz);
	xbstr_t(wchar_t* pwsz, BOOL bAttach = FALSE);
	virtual ~xbstr_t();

protected:
	void clearBuffer();
	void assign(const char* psz);
	void assign(const wchar_t* psz);

public:
	xbstr_t& operator=(const char* psz);
	xbstr_t& operator=(const wchar_t* psz);
	xbstr_t& operator=(const xbstr_t& str);
	operator const char*();
	operator wchar_t*() const;

	int length() const;
};


#endif // _XSTR_H_
